@@ -34,9 +34,10 @@
 #include "objectgroup.h"
 #include "tile.h"
 #include "tilelayer.h"
-#include "tileset.h"
 #include "mapobject.h"
 
+#include <cmath>
+
 using namespace Tiled;
 
 Map::Map(Orientation orientation,
@@ -106,6 +107,26 @@ void Map::adjustDrawMargins(const QMargins &margins)
                               mDrawMargins);
 }
 
+/**
+ * Computes the extra margins due to layer offsets. These need to be taken into
+ * account when determining the bounding rect of the map for example.
+ */
+QMargins Map::computeLayerOffsetMargins() const
+{
+    QMargins offsetMargins;
+
+    for (const Layer *layer : mLayers) {
+        const QPointF offset = layer->offset();
+        offsetMargins = maxMargins(QMargins(std::ceil(-offset.x()),
+                                            std::ceil(-offset.y()),
+                                            std::ceil(offset.x()),
+                                            std::ceil(offset.y())),
+                                   offsetMargins);
+    }
+
+    return offsetMargins;
+}
+
 /**
  * Recomputes the draw margins for this map and each of its tile layers. Needed
  * after the tile offset of a tileset has changed for example.
@@ -197,42 +218,50 @@ void Map::adoptLayer(Layer *layer)
 Layer *Map::takeLayerAt(int index)
 {
     Layer *layer = mLayers.takeAt(index);
-    layer->setMap(0);
+    layer->setMap(nullptr);
     return layer;
 }
 
-void Map::addTileset(Tileset *tileset)
+void Map::addTileset(const SharedTileset &tileset)
 {
     mTilesets.append(tileset);
 }
 
-void Map::insertTileset(int index, Tileset *tileset)
+void Map::addTilesets(const QSet<SharedTileset> &tilesets)
+{
+    for (const SharedTileset &tileset : tilesets)
+        addTileset(tileset);
+}
+
+void Map::insertTileset(int index, const SharedTileset &tileset)
 {
     mTilesets.insert(index, tileset);
 }
 
-int Map::indexOfTileset(Tileset *tileset) const
+int Map::indexOfTileset(const SharedTileset &tileset) const
 {
     return mTilesets.indexOf(tileset);
 }
 
 void Map::removeTilesetAt(int index)
 {
-    mTilesets.removeAt(index);
+    mTilesets.remove(index);
 }
 
-void Map::replaceTileset(Tileset *oldTileset, Tileset *newTileset)
+void Map::replaceTileset(const SharedTileset &oldTileset,
+                         const SharedTileset &newTileset)
 {
     const int index = mTilesets.indexOf(oldTileset);
     Q_ASSERT(index != -1);
 
     foreach (Layer *layer, mLayers)
-        layer->replaceReferencesToTileset(oldTileset, newTileset);
+        layer->replaceReferencesToTileset(oldTileset.data(),
+                                          newTileset.data());
 
     mTilesets.replace(index, newTileset);
 }
 
-bool Map::isTilesetUsed(Tileset *tileset) const
+bool Map::isTilesetUsed(const Tileset *tileset) const
 {
     foreach (const Layer *layer, mLayers)
         if (layer->referencesTileset(tileset))
