@@ -38,7 +38,7 @@
 #include "mapobject.h"
 #include "tile.h"
 #include "tilelayer.h"
-#include "tileset.h"
+#include "tilesetformat.h"
 #include "terrain.h"
 
 #include <QCoreApplication>
@@ -63,12 +63,12 @@ class MapReaderPrivate
 public:
     MapReaderPrivate(MapReader *mapReader):
         p(mapReader),
-        mMap(0),
+        mMap(nullptr),
         mReadingExternalTileset(false)
     {}
 
     Map *readMap(QIODevice *device, const QString &path);
-    Tileset *readTileset(QIODevice *device, const QString &path);
+    SharedTileset readTileset(QIODevice *device, const QString &path);
 
     bool openFile(QFile *file);
 
@@ -79,17 +79,17 @@ private:
 
     Map *readMap();
 
-    Tileset *readTileset();
-    void readTilesetTile(Tileset *tileset);
-    void readTilesetImage(Tileset *tileset);
-    void readTilesetTerrainTypes(Tileset *tileset);
+    SharedTileset readTileset();
+    void readTilesetTile(SharedTileset &tileset);
+    void readTilesetImage(SharedTileset &tileset);
+    void readTilesetTerrainTypes(SharedTileset &tileset);
     QImage readImage();
 
     TileLayer *readLayer();
     void readLayerData(TileLayer *tileLayer);
     void decodeBinaryLayerData(TileLayer *tileLayer,
-                               const QStringRef &text,
-                               const QStringRef &compression);
+                               const QByteArray &data,
+                               Map::LayerDataFormat format);
     void decodeCSVLayerData(TileLayer *tileLayer, const QString &text);
 
     /**
@@ -118,7 +118,6 @@ private:
     QString mError;
     QString mPath;
     Map *mMap;
-    QList<Tileset*> mCreatedTilesets;
     GidMapper mGidMapper;
     bool mReadingExternalTileset;
 
@@ -132,7 +131,7 @@ Map *MapReaderPrivate::readMap(QIODevice *device, const QString &path)
 {
     mError.clear();
     mPath = path;
-    Map *map = 0;
+    Map *map = nullptr;
 
     xml.setDevice(device);
 
@@ -146,11 +145,11 @@ Map *MapReaderPrivate::readMap(QIODevice *device, const QString &path)
     return map;
 }
 
-Tileset *MapReaderPrivate::readTileset(QIODevice *device, const QString &path)
+SharedTileset MapReaderPrivate::readTileset(QIODevice *device, const QString &path)
 {
     mError.clear();
     mPath = path;
-    Tileset *tileset = 0;
+    SharedTileset tileset;
     mReadingExternalTileset = true;
 
     xml.setDevice(device);
@@ -203,15 +202,15 @@ Map *MapReaderPrivate::readMap()
 
     const QXmlStreamAttributes atts = xml.attributes();
     const int mapWidth =
-            atts.value(QLatin1String("width")).toString().toInt();
+            atts.value(QLatin1String("width")).toInt();
     const int mapHeight =
-            atts.value(QLatin1String("height")).toString().toInt();
+            atts.value(QLatin1String("height")).toInt();
     const int tileWidth =
-            atts.value(QLatin1String("tilewidth")).toString().toInt();
+            atts.value(QLatin1String("tilewidth")).toInt();
     const int tileHeight =
-            atts.value(QLatin1String("tileheight")).toString().toInt();
+            atts.value(QLatin1String("tileheight")).toInt();
     const int hexSideLength =
-            atts.value(QLatin1String("hexsidelength")).toString().toInt();
+            atts.value(QLatin1String("hexsidelength")).toInt();
 
     const QString orientationString =
             atts.value(QLatin1String("orientation")).toString();
@@ -239,7 +238,7 @@ Map *MapReaderPrivate::readMap()
             renderOrderFromString(renderOrderString);
 
     const int nextObjectId =
-            atts.value(QLatin1String("nextobjectid")).toString().toInt();
+            atts.value(QLatin1String("nextobjectid")).toInt();
 
     mMap = new Map(orientation, mapWidth, mapHeight, tileWidth, tileHeight);
     mMap->setHexSideLength(hexSideLength);
@@ -249,8 +248,6 @@ Map *MapReaderPrivate::readMap()
     if (nextObjectId)
         mMap->setNextObjectId(nextObjectId);
 
-    mCreatedTilesets.clear();
-
     QStringRef bgColorString = atts.value(QLatin1String("backgroundcolor"));
     if (!bgColorString.isEmpty())
         mMap->setBackgroundColor(QColor(bgColorString.toString()));
@@ -272,57 +269,51 @@ Map *MapReaderPrivate::readMap()
 
     // Clean up in case of error
     if (xml.hasError()) {
-        // The tilesets are not owned by the map
-        qDeleteAll(mCreatedTilesets);
-        mCreatedTilesets.clear();
-
         delete mMap;
-        mMap = 0;
+        mMap = nullptr;
     }
 
     return mMap;
 }
 
-Tileset *MapReaderPrivate::readTileset()
+SharedTileset MapReaderPrivate::readTileset()
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("tileset"));
 
     const QXmlStreamAttributes atts = xml.attributes();
     const QString source = atts.value(QLatin1String("source")).toString();
     const unsigned firstGid =
-            atts.value(QLatin1String("firstgid")).toString().toUInt();
+            atts.value(QLatin1String("firstgid")).toUInt();
 
-    Tileset *tileset = 0;
+    SharedTileset tileset;
 
     if (source.isEmpty()) { // Not an external tileset
         const QString name =
                 atts.value(QLatin1String("name")).toString();
         const int tileWidth =
-                atts.value(QLatin1String("tilewidth")).toString().toInt();
+                atts.value(QLatin1String("tilewidth")).toInt();
         const int tileHeight =
-                atts.value(QLatin1String("tileheight")).toString().toInt();
+                atts.value(QLatin1String("tileheight")).toInt();
         const int tileSpacing =
-                atts.value(QLatin1String("spacing")).toString().toInt();
+                atts.value(QLatin1String("spacing")).toInt();
         const int margin =
-                atts.value(QLatin1String("margin")).toString().toInt();
+                atts.value(QLatin1String("margin")).toInt();
 
         if (tileWidth < 0 || tileHeight < 0
             || (firstGid == 0 && !mReadingExternalTileset)) {
             xml.raiseError(tr("Invalid tileset parameters for tileset"
                               " '%1'").arg(name));
         } else {
-            tileset = new Tileset(name, tileWidth, tileHeight,
-                                  tileSpacing, margin);
-
-            mCreatedTilesets.append(tileset);
+            tileset = Tileset::create(name, tileWidth, tileHeight,
+                                      tileSpacing, margin);
 
             while (xml.readNextStartElement()) {
                 if (xml.name() == QLatin1String("tile")) {
                     readTilesetTile(tileset);
                 } else if (xml.name() == QLatin1String("tileoffset")) {
                     const QXmlStreamAttributes oa = xml.attributes();
-                    int x = oa.value(QLatin1String("x")).toString().toInt();
-                    int y = oa.value(QLatin1String("y")).toString().toInt();
+                    int x = oa.value(QLatin1String("x")).toInt();
+                    int y = oa.value(QLatin1String("y")).toInt();
                     tileset->setTileOffset(QPoint(x, y));
                     xml.skipCurrentElement();
                 } else if (xml.name() == QLatin1String("properties")) {
@@ -331,9 +322,7 @@ Tileset *MapReaderPrivate::readTileset()
                     if (tileWidth == 0 || tileHeight == 0) {
                         xml.raiseError(tr("Invalid tileset parameters for tileset"
                                           " '%1'").arg(name));
-                        delete tileset;
-                        tileset = 0;
-                        mCreatedTilesets.removeLast();
+                        tileset.clear();
                         break;
                     } else {
                         readTilesetImage(tileset);
@@ -359,17 +348,17 @@ Tileset *MapReaderPrivate::readTileset()
     }
 
     if (tileset && !mReadingExternalTileset)
-        mGidMapper.insert(firstGid, tileset);
+        mGidMapper.insert(firstGid, tileset.data());
 
     return tileset;
 }
 
-void MapReaderPrivate::readTilesetTile(Tileset *tileset)
+void MapReaderPrivate::readTilesetTile(SharedTileset &tileset)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("tile"));
 
     const QXmlStreamAttributes atts = xml.attributes();
-    const int id = atts.value(QLatin1String("id")).toString().toInt();
+    const int id = atts.value(QLatin1String("id")).toInt();
 
     if (id < 0) {
         xml.raiseError(tr("Invalid tile ID: %1").arg(id));
@@ -401,7 +390,7 @@ void MapReaderPrivate::readTilesetTile(Tileset *tileset)
         if (quadrants.size() == 4) {
             for (int i = 0; i < 4; ++i) {
                 int t = quadrants[i].isEmpty() ? -1 : quadrants[i].toInt();
-                tile->setCornerTerrain(i, t);
+                tile->setCornerTerrainId(i, t);
             }
         }
     }
@@ -409,7 +398,7 @@ void MapReaderPrivate::readTilesetTile(Tileset *tileset)
     // Read tile probability
     QStringRef probability = atts.value(QLatin1String("probability"));
     if (!probability.isEmpty())
-        tile->setTerrainProbability(probability.toString().toFloat());
+        tile->setProbability(probability.toFloat());
 
     while (xml.readNextStartElement()) {
         if (xml.name() == QLatin1String("properties")) {
@@ -450,7 +439,7 @@ void MapReaderPrivate::readTilesetTile(Tileset *tileset)
     }
 }
 
-void MapReaderPrivate::readTilesetImage(Tileset *tileset)
+void MapReaderPrivate::readTilesetImage(SharedTileset &tileset)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("image"));
 
@@ -468,8 +457,8 @@ void MapReaderPrivate::readTilesetImage(Tileset *tileset)
         source = p->resolveReference(source, mPath);
 
     // Set the width that the tileset had when the map was saved
-    const int width = atts.value(QLatin1String("width")).toString().toInt();
-    mGidMapper.setTilesetWidth(tileset, width);
+    const int width = atts.value(QLatin1String("width")).toInt();
+    mGidMapper.setTilesetWidth(tileset.data(), width);
 
     if (!tileset->loadFromImage(readImage(), source))
         xml.raiseError(tr("Error loading tileset image:\n'%1'").arg(source));
@@ -511,7 +500,7 @@ QImage MapReaderPrivate::readImage()
     return QImage();
 }
 
-void MapReaderPrivate::readTilesetTerrainTypes(Tileset *tileset)
+void MapReaderPrivate::readTilesetTerrainTypes(SharedTileset &tileset)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("terraintypes"));
 
@@ -519,7 +508,7 @@ void MapReaderPrivate::readTilesetTerrainTypes(Tileset *tileset)
         if (xml.name() == QLatin1String("terrain")) {
             const QXmlStreamAttributes atts = xml.attributes();
             QString name = atts.value(QLatin1String("name")).toString();
-            int tile = atts.value(QLatin1String("tile")).toString().toInt();
+            int tile = atts.value(QLatin1String("tile")).toInt();
 
             Terrain *terrain = tileset->addTerrain(name, tile);
 
@@ -542,13 +531,18 @@ static void readLayerAttributes(Layer *layer,
     const QStringRef visibleRef = atts.value(QLatin1String("visible"));
 
     bool ok;
-    const float opacity = opacityRef.toString().toFloat(&ok);
+    const float opacity = opacityRef.toFloat(&ok);
     if (ok)
         layer->setOpacity(opacity);
 
-    const int visible = visibleRef.toString().toInt(&ok);
+    const int visible = visibleRef.toInt(&ok);
     if (ok)
         layer->setVisible(visible);
+
+    const QPointF offset(atts.value(QLatin1String("offsetx")).toDouble(),
+                         atts.value(QLatin1String("offsety")).toDouble());
+
+    layer->setOffset(offset);
 }
 
 TileLayer *MapReaderPrivate::readLayer()
@@ -557,10 +551,10 @@ TileLayer *MapReaderPrivate::readLayer()
 
     const QXmlStreamAttributes atts = xml.attributes();
     const QString name = atts.value(QLatin1String("name")).toString();
-    const int x = atts.value(QLatin1String("x")).toString().toInt();
-    const int y = atts.value(QLatin1String("y")).toString().toInt();
-    const int width = atts.value(QLatin1String("width")).toString().toInt();
-    const int height = atts.value(QLatin1String("height")).toString().toInt();
+    const int x = atts.value(QLatin1String("x")).toInt();
+    const int y = atts.value(QLatin1String("y")).toInt();
+    const int width = atts.value(QLatin1String("width")).toInt();
+    const int height = atts.value(QLatin1String("height")).toInt();
 
     TileLayer *tileLayer = new TileLayer(name, x, y, width, height);
     readLayerAttributes(tileLayer, atts);
@@ -585,30 +579,36 @@ void MapReaderPrivate::readLayerData(TileLayer *tileLayer)
     QStringRef encoding = atts.value(QLatin1String("encoding"));
     QStringRef compression = atts.value(QLatin1String("compression"));
 
-    bool respect = true; // TODO: init from preferences
-    if (respect) {
-        if (encoding.isEmpty())
-            mMap->setLayerDataFormat(Map::XML);
-        else if (encoding == QLatin1String("csv"))
-            mMap->setLayerDataFormat(Map::CSV);
-        else if (encoding == QLatin1String("base64")) {
-            if (compression.isEmpty())
-                mMap->setLayerDataFormat(Map::Base64);
-            else if (compression == QLatin1String("gzip"))
-                mMap->setLayerDataFormat(Map::Base64Gzip);
-            else if (compression == QLatin1String("zlib"))
-                mMap->setLayerDataFormat(Map::Base64Zlib);
+    Map::LayerDataFormat layerDataFormat;
+    if (encoding.isEmpty()) {
+        layerDataFormat = Map::XML;
+    } else if (encoding == QLatin1String("csv")) {
+        layerDataFormat = Map::CSV;
+    } else if (encoding == QLatin1String("base64")) {
+        if (compression.isEmpty()) {
+            layerDataFormat = Map::Base64;
+        } else if (compression == QLatin1String("gzip")) {
+            layerDataFormat = Map::Base64Gzip;
+        } else if (compression == QLatin1String("zlib")) {
+            layerDataFormat = Map::Base64Zlib;
+        } else {
+            xml.raiseError(tr("Compression method '%1' not supported")
+                           .arg(compression.toString()));
+            return;
         }
-        // else, error handled below
+    } else {
+        xml.raiseError(tr("Unknown encoding: %1").arg(encoding.toString()));
+        return;
     }
+    mMap->setLayerDataFormat(layerDataFormat);
 
     int x = 0;
     int y = 0;
 
     while (xml.readNext() != QXmlStreamReader::Invalid) {
-        if (xml.isEndElement())
+        if (xml.isEndElement()) {
             break;
-        else if (xml.isStartElement()) {
+        } else if (xml.isStartElement()) {
             if (xml.name() == QLatin1String("tile")) {
                 if (y >= tileLayer->height()) {
                     xml.raiseError(tr("Too many <tile> elements"));
@@ -616,7 +616,7 @@ void MapReaderPrivate::readLayerData(TileLayer *tileLayer)
                 }
 
                 const QXmlStreamAttributes atts = xml.attributes();
-                unsigned gid = atts.value(QLatin1String("gid")).toString().toUInt();
+                unsigned gid = atts.value(QLatin1String("gid")).toUInt();
                 tileLayer->setCell(x, y, cellForGid(gid));
 
                 x++;
@@ -632,65 +632,33 @@ void MapReaderPrivate::readLayerData(TileLayer *tileLayer)
         } else if (xml.isCharacters() && !xml.isWhitespace()) {
             if (encoding == QLatin1String("base64")) {
                 decodeBinaryLayerData(tileLayer,
-                                      xml.text(),
-                                      compression);
+                                      xml.text().toLatin1(),
+                                      layerDataFormat);
             } else if (encoding == QLatin1String("csv")) {
                 decodeCSVLayerData(tileLayer, xml.text().toString());
-            } else {
-                xml.raiseError(tr("Unknown encoding: %1")
-                               .arg(encoding.toString()));
-                continue;
             }
         }
     }
 }
 
 void MapReaderPrivate::decodeBinaryLayerData(TileLayer *tileLayer,
-                                             const QStringRef &text,
-                                             const QStringRef &compression)
+                                             const QByteArray &data,
+                                             Map::LayerDataFormat format)
 {
-#if QT_VERSION < 0x040800
-    const QString textData = QString::fromRawData(text.unicode(), text.size());
-    const QByteArray latin1Text = textData.toLatin1();
-#else
-    const QByteArray latin1Text = text.toLatin1();
-#endif
-    QByteArray tileData = QByteArray::fromBase64(latin1Text);
-    const int size = (tileLayer->width() * tileLayer->height()) * 4;
+    GidMapper::DecodeError error = mGidMapper.decodeLayerData(*tileLayer, data, format);
 
-    if (compression == QLatin1String("zlib")
-        || compression == QLatin1String("gzip")) {
-        tileData = decompress(tileData, size);
-    } else if (!compression.isEmpty()) {
-        xml.raiseError(tr("Compression method '%1' not supported")
-                       .arg(compression.toString()));
+    switch (error) {
+    case GidMapper::CorruptLayerData:
+        xml.raiseError(tr("Corrupt layer data for layer '%1'").arg(tileLayer->name()));
         return;
-    }
-
-    if (size != tileData.length()) {
-        xml.raiseError(tr("Corrupt layer data for layer '%1'")
-                       .arg(tileLayer->name()));
+    case GidMapper::TileButNoTilesets:
+        xml.raiseError(tr("Tile used but no tilesets specified"));
         return;
-    }
-
-    const unsigned char *data =
-            reinterpret_cast<const unsigned char*>(tileData.constData());
-    int x = 0;
-    int y = 0;
-
-    for (int i = 0; i < size - 3; i += 4) {
-        const unsigned gid = data[i] |
-                             data[i + 1] << 8 |
-                             data[i + 2] << 16 |
-                             data[i + 3] << 24;
-
-        tileLayer->setCell(x, y, cellForGid(gid));
-
-        x++;
-        if (x == tileLayer->width()) {
-            x = 0;
-            y++;
-        }
+    case GidMapper::InvalidTile:
+        xml.raiseError(tr("Invalid tile: %1").arg(mGidMapper.invalidTile()));
+        return;
+    case GidMapper::NoError:
+        break;
     }
 }
 
@@ -742,10 +710,10 @@ ObjectGroup *MapReaderPrivate::readObjectGroup()
 
     const QXmlStreamAttributes atts = xml.attributes();
     const QString name = atts.value(QLatin1String("name")).toString();
-    const int x = atts.value(QLatin1String("x")).toString().toInt();
-    const int y = atts.value(QLatin1String("y")).toString().toInt();
-    const int width = atts.value(QLatin1String("width")).toString().toInt();
-    const int height = atts.value(QLatin1String("height")).toString().toInt();
+    const int x = atts.value(QLatin1String("x")).toInt();
+    const int y = atts.value(QLatin1String("y")).toInt();
+    const int width = atts.value(QLatin1String("width")).toInt();
+    const int height = atts.value(QLatin1String("height")).toInt();
 
     ObjectGroup *objectGroup = new ObjectGroup(name, x, y, width, height);
     readLayerAttributes(objectGroup, atts);
@@ -760,7 +728,7 @@ ObjectGroup *MapReaderPrivate::readObjectGroup()
         if (drawOrder == ObjectGroup::UnknownOrder) {
             delete objectGroup;
             xml.raiseError(tr("Invalid draw order: %1").arg(value));
-            return 0;
+            return nullptr;
         }
         objectGroup->setDrawOrder(drawOrder);
     }
@@ -783,10 +751,10 @@ ImageLayer *MapReaderPrivate::readImageLayer()
 
     const QXmlStreamAttributes atts = xml.attributes();
     const QString name = atts.value(QLatin1String("name")).toString();
-    const int x = atts.value(QLatin1String("x")).toString().toInt();
-    const int y = atts.value(QLatin1String("y")).toString().toInt();
-    const int width = atts.value(QLatin1String("width")).toString().toInt();
-    const int height = atts.value(QLatin1String("height")).toString().toInt();
+    const int x = atts.value(QLatin1String("x")).toInt();
+    const int y = atts.value(QLatin1String("y")).toInt();
+    const int width = atts.value(QLatin1String("width")).toInt();
+    const int height = atts.value(QLatin1String("height")).toInt();
 
     ImageLayer *imageLayer = new ImageLayer(name, x, y, width, height);
     readLayerAttributes(imageLayer, atts);
@@ -831,13 +799,13 @@ MapObject *MapReaderPrivate::readObject()
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("object"));
 
     const QXmlStreamAttributes atts = xml.attributes();
-    const int id = atts.value(QLatin1String("id")).toString().toInt();
+    const int id = atts.value(QLatin1String("id")).toInt();
     const QString name = atts.value(QLatin1String("name")).toString();
-    const unsigned gid = atts.value(QLatin1String("gid")).toString().toUInt();
-    const qreal x = atts.value(QLatin1String("x")).toString().toDouble();
-    const qreal y = atts.value(QLatin1String("y")).toString().toDouble();
-    const qreal width = atts.value(QLatin1String("width")).toString().toDouble();
-    const qreal height = atts.value(QLatin1String("height")).toString().toDouble();
+    const unsigned gid = atts.value(QLatin1String("gid")).toUInt();
+    const qreal x = atts.value(QLatin1String("x")).toDouble();
+    const qreal y = atts.value(QLatin1String("y")).toDouble();
+    const qreal width = atts.value(QLatin1String("width")).toDouble();
+    const qreal height = atts.value(QLatin1String("height")).toDouble();
     const QString type = atts.value(QLatin1String("type")).toString();
     const QStringRef visibleRef = atts.value(QLatin1String("visible"));
 
@@ -848,7 +816,7 @@ MapObject *MapReaderPrivate::readObject()
     object->setId(id);
 
     bool ok;
-    const qreal rotation = atts.value(QLatin1String("rotation")).toString().toDouble(&ok);
+    const qreal rotation = atts.value(QLatin1String("rotation")).toDouble(&ok);
     if (ok)
         object->setRotation(rotation);
 
@@ -864,7 +832,7 @@ MapObject *MapReaderPrivate::readObject()
         }
     }
 
-    const int visible = visibleRef.toString().toInt(&ok);
+    const int visible = visibleRef.toInt(&ok);
     if (ok)
         object->setVisible(visible);
 
@@ -936,8 +904,8 @@ QVector<Frame> MapReaderPrivate::readAnimationFrames()
             const QXmlStreamAttributes atts = xml.attributes();
 
             Frame frame;
-            frame.tileId = atts.value(QLatin1String("tileid")).toString().toInt();
-            frame.duration = atts.value(QLatin1String("duration")).toString().toInt();
+            frame.tileId = atts.value(QLatin1String("tileid")).toInt();
+            frame.duration = atts.value(QLatin1String("duration")).toInt();
             frames.append(frame);
 
             xml.skipCurrentElement();
@@ -1007,23 +975,23 @@ Map *MapReader::readMap(const QString &fileName)
 {
     QFile file(fileName);
     if (!d->openFile(&file))
-        return 0;
+        return nullptr;
 
     return readMap(&file, QFileInfo(fileName).absolutePath());
 }
 
-Tileset *MapReader::readTileset(QIODevice *device, const QString &path)
+SharedTileset MapReader::readTileset(QIODevice *device, const QString &path)
 {
     return d->readTileset(device, path);
 }
 
-Tileset *MapReader::readTileset(const QString &fileName)
+SharedTileset MapReader::readTileset(const QString &fileName)
 {
     QFile file(fileName);
     if (!d->openFile(&file))
-        return 0;
+        return SharedTileset();
 
-    Tileset *tileset = readTileset(&file, QFileInfo(fileName).absolutePath());
+    SharedTileset tileset = readTileset(&file, QFileInfo(fileName).absolutePath());
     if (tileset)
         tileset->setFileName(fileName);
 
@@ -1049,16 +1017,8 @@ QImage MapReader::readExternalImage(const QString &source)
     return QImage(source);
 }
 
-Tileset *MapReader::readExternalTileset(const QString &source,
-                                        QString *error)
+SharedTileset MapReader::readExternalTileset(const QString &source,
+                                             QString *error)
 {
-    MapReader reader;
-
-    Tileset *tileset = reader.readTileset(source);
-    if (!tileset)
-        *error = reader.errorString();
-    else
-        d->mCreatedTilesets.append(tileset);
-
-    return tileset;
+    return Tiled::readTileset(source, error);
 }
