@@ -59,13 +59,18 @@ class TILEDSHARED_EXPORT MapRenderer
 public:
     MapRenderer(const Map *map)
         : mMap(map)
-        , mFlags(0)
+        , mFlags(nullptr)
         , mObjectLineWidth(2)
         , mPainterScale(1)
     {}
 
     virtual ~MapRenderer() {}
 
+    /**
+     * Returns the map this renderer is associated with.
+     */
+    const Map *map() const;
+
     /**
      * Returns the size in pixels of the map associated with this renderer.
      */
@@ -209,12 +214,6 @@ public:
 
     static QPolygonF lineToPolygon(const QPointF &start, const QPointF &end);
 
-protected:
-    /**
-     * Returns the map this renderer is associated with.
-     */
-    const Map *map() const { return mMap; }
-
 private:
     const Map *mMap;
 
@@ -223,6 +222,11 @@ private:
     qreal mPainterScale;
 };
 
+inline const Map *MapRenderer::map() const
+{
+    return mMap;
+}
+
 inline QPointF MapRenderer::screenToTileCoords(const QPointF &point) const
 {
     return screenToTileCoords(point.x(), point.y());
