@@ -34,7 +34,6 @@
 #include "map.h"
 #include "mapobject.h"
 #include "tile.h"
-#include "tileset.h"
 
 #include <cmath>
 
@@ -80,14 +79,14 @@ int ObjectGroup::removeObject(MapObject *object)
     Q_ASSERT(index != -1);
 
     mObjects.removeAt(index);
-    object->setObjectGroup(0);
+    object->setObjectGroup(nullptr);
     return index;
 }
 
 void ObjectGroup::removeObjectAt(int index)
 {
     MapObject *object = mObjects.takeAt(index);
-    object->setObjectGroup(0);
+    object->setObjectGroup(nullptr);
 }
 
 void ObjectGroup::moveObjects(int from, int to, int count)
@@ -125,13 +124,13 @@ bool ObjectGroup::isEmpty() const
     return mObjects.isEmpty();
 }
 
-QSet<Tileset*> ObjectGroup::usedTilesets() const
+QSet<SharedTileset> ObjectGroup::usedTilesets() const
 {
-    QSet<Tileset*> tilesets;
+    QSet<SharedTileset> tilesets;
 
     foreach (const MapObject *object, mObjects)
         if (const Tile *tile = object->cell().tile)
-            tilesets.insert(tile->tileset());
+            tilesets.insert(tile->sharedTileset());
 
     return tilesets;
 }
@@ -160,9 +159,9 @@ void ObjectGroup::replaceReferencesToTileset(Tileset *oldTileset,
     }
 }
 
-void ObjectGroup::offset(const QPointF &offset,
-                         const QRectF &bounds,
-                         bool wrapX, bool wrapY)
+void ObjectGroup::offsetObjects(const QPointF &offset,
+                                const QRectF &bounds,
+                                bool wrapX, bool wrapY)
 {
     foreach (MapObject *object, mObjects) {
         const QPointF objectCenter = object->bounds().center();
