@@ -33,28 +33,32 @@
 
 using namespace Tiled;
 
-Tile::Tile(const QPixmap &image, int id, Tileset *tileset):
+Tile::Tile(const QPixmap &image,
+           int id,
+           Tileset *tileset):
     Object(TileType),
     mId(id),
     mTileset(tileset),
     mImage(image),
     mTerrain(-1),
-    mTerrainProbability(1.f),
-    mObjectGroup(0),
+    mProbability(1.f),
+    mObjectGroup(nullptr),
     mCurrentFrameIndex(0),
     mUnusedTime(0)
 {}
 
-Tile::Tile(const QPixmap &image, const QString &imageSource,
-           int id, Tileset *tileset):
+Tile::Tile(const QPixmap &image,
+           const QString &imageSource,
+           int id,
+           Tileset *tileset):
     Object(TileType),
     mId(id),
     mTileset(tileset),
     mImage(image),
     mImageSource(imageSource),
     mTerrain(-1),
-    mTerrainProbability(1.f),
-    mObjectGroup(0),
+    mProbability(1.f),
+    mObjectGroup(nullptr),
     mCurrentFrameIndex(0),
     mUnusedTime(0)
 {}
@@ -64,6 +68,14 @@ Tile::~Tile()
     delete mObjectGroup;
 }
 
+/**
+ * Returns the tileset that this tile is part of as a shared pointer.
+ */
+QSharedPointer<Tileset> Tile::sharedTileset() const
+{
+    return mTileset->sharedPointer();
+}
+
 /**
  * Returns the image for rendering this tile, taking into account tile
  * animations.
@@ -78,11 +90,25 @@ const QPixmap &Tile::currentFrameImage() const
     }
 }
 
+/**
+ * Returns the drawing offset of the tile (in pixels).
+ */
+QPoint Tile::offset() const
+{
+    return mTileset->tileOffset();
+}
+
+/**
+ * Returns the Terrain of a given corner.
+ */
 Terrain *Tile::terrainAtCorner(int corner) const
 {
     return mTileset->terrain(cornerTerrainId(corner));
 }
 
+/**
+ * Set the terrain for each corner of the tile.
+ */
 void Tile::setTerrain(unsigned terrain)
 {
     if (mTerrain == terrain)
