@@ -38,11 +38,11 @@
 #include <QMargins>
 #include <QString>
 #include <QVector>
+#include <QSharedPointer>
 
 namespace Tiled {
 
 class Tile;
-class Tileset;
 
 /**
  * A cell on a tile layer grid.
@@ -51,7 +51,7 @@ class Cell
 {
 public:
     Cell() :
-        tile(0),
+        tile(nullptr),
         flippedHorizontally(false),
         flippedVertically(false),
         flippedAntiDiagonally(false)
@@ -64,7 +64,7 @@ public:
         flippedAntiDiagonally(false)
     {}
 
-    bool isEmpty() const { return tile == 0; }
+    bool isEmpty() const { return tile == nullptr; }
 
     bool operator == (const Cell &other) const
     {
@@ -206,7 +206,7 @@ public:
     /**
      * Computes and returns the set of tilesets used by this tile layer.
      */
-    QSet<Tileset*> usedTilesets() const;
+    QSet<SharedTileset> usedTilesets() const override;
 
     /**
      * Returns whether this tile layer has any cell for which the given
@@ -218,7 +218,7 @@ public:
     /**
      * Returns whether this tile layer is referencing the given tileset.
      */
-    bool referencesTileset(const Tileset *tileset) const;
+    bool referencesTileset(const Tileset *tileset) const override;
 
     /**
      * Removes all references to the given tileset. This sets all tiles on this
@@ -229,7 +229,8 @@ public:
     /**
      * Replaces all tiles from \a oldTileset with tiles from \a newTileset.
      */
-    void replaceReferencesToTileset(Tileset *oldTileset, Tileset *newTileset);
+    void replaceReferencesToTileset(Tileset *oldTileset,
+                                    Tileset *newTileset) override;
 
     /**
      * Resizes this tile layer to \a size, while shifting all tiles by
@@ -241,14 +242,14 @@ public:
      * Offsets the tiles in this layer within \a bounds by \a offset,
      * and optionally wraps them.
      *
-     * \sa ObjectGroup::offset()
+     * \sa ObjectGroup::offsetObjects()
      */
-    void offset(const QPoint &offset,
-                const QRect &bounds,
-                bool wrapX, bool wrapY);
+    void offsetTiles(const QPoint &offset,
+                     const QRect &bounds,
+                     bool wrapX, bool wrapY);
 
-    bool canMergeWith(Layer *other) const;
-    Layer *mergedWith(Layer *other) const;
+    bool canMergeWith(Layer *other) const override;
+    Layer *mergedWith(Layer *other) const override;
 
     /**
      * Returns the region where this tile layer and the given tile layer
@@ -260,9 +261,15 @@ public:
     /**
      * Returns true if all tiles in the layer are empty.
      */
-    bool isEmpty() const;
+    bool isEmpty() const override;
 
-    virtual Layer *clone() const;
+    virtual Layer *clone() const override;
+
+    // Enable easy iteration over cells with range-based for
+    QVector<Cell>::iterator begin() { return mGrid.begin(); }
+    QVector<Cell>::iterator end() { return mGrid.end(); }
+    QVector<Cell>::const_iterator begin() const { return mGrid.begin(); }
+    QVector<Cell>::const_iterator end() const { return mGrid.end(); }
 
 protected:
     TileLayer *initializeClone(TileLayer *clone) const;
@@ -301,19 +308,16 @@ QRegion TileLayer::region(Condition condition) const
 template<typename Condition>
 bool TileLayer::hasCell(Condition condition) const
 {
-    for (int i = 0, i_end = mGrid.size(); i < i_end; ++i)
-        if (condition(mGrid.at(i)))
+    for (const Cell &cell : mGrid)
+        if (condition(cell))
             return true;
 
     return false;
 }
 
-
-static inline bool cellInUse(const Cell &cell) { return !cell.isEmpty(); }
-
 inline QRegion TileLayer::region() const
 {
-    return region(cellInUse);
+    return region([] (const Cell &cell) { return !cell.isEmpty(); });
 }
 
 inline const Cell &TileLayer::cellAt(int x, int y) const
@@ -327,6 +331,8 @@ inline const Cell &TileLayer::cellAt(const QPoint &point) const
     return cellAt(point.x(), point.y());
 }
 
+typedef QSharedPointer<TileLayer> SharedTileLayer;
+
 } // namespace Tiled
 
 #endif // TILELAYER_H
