@@ -25,13 +25,12 @@
 #include "gidmapper.h"
 #include "map.h"
 #include "mapobject.h"
-#include "mapreader.h"
 #include "tile.h"
 #include "tilelayer.h"
 #include "tileset.h"
 #include "objectgroup.h"
 
-#include <QFile>
+#include <QSaveFile>
 #include <QFileInfo>
 #include <QDir>
 #include <QSettings>
@@ -48,13 +47,14 @@ FlarePlugin::FlarePlugin()
 Tiled::Map *FlarePlugin::read(const QString &fileName)
 {
     QFile file(fileName);
+
     if (!file.open (QIODevice::ReadOnly)) {
         mError = tr("Could not open file for reading.");
-        return 0;
+        return nullptr;
     }
 
     // default to values of the original flare alpha game.
-    Map *map = new Map(Map::Isometric, 256, 256, 64, 32);
+    QScopedPointer<Map> map(new Map(Map::Isometric, 256, 256, 64, 32));
 
     QTextStream stream (&file);
     QString line;
@@ -64,9 +64,9 @@ Tiled::Map *FlarePlugin::read(const QString &fileName)
     int base = 10;
     GidMapper gidMapper;
     int gid = 1;
-    TileLayer *tilelayer = 0;
-    ObjectGroup *objectgroup = 0;
-    MapObject *mapobject = 0;
+    TileLayer *tilelayer = nullptr;
+    ObjectGroup *objectgroup = nullptr;
+    MapObject *mapobject = nullptr;
     bool tilesetsSectionFound = false;
     bool headerSectionFound = false;
     bool tilelayerSectionFound = false; // tile layer or objects
@@ -123,20 +123,19 @@ Tiled::Map *FlarePlugin::read(const QString &fileName)
                     tilesetheight = list[2].toInt();
                 }
 
-                Tileset *tileset = new Tileset(QFileInfo(absoluteSource).fileName(),
-                                               tilesetwidth, tilesetheight);
+                SharedTileset tileset(Tileset::create(QFileInfo(absoluteSource).fileName(),
+                                                      tilesetwidth, tilesetheight));
                 bool ok = tileset->loadFromImage(absoluteSource);
 
                 if (!ok) {
                     mError = tr("Error loading tileset %1, which expands to %2. Path not found!")
                             .arg(list[0], absoluteSource);
-                    delete map;
-                    return 0;
+                    return nullptr;
                 } else {
                     if (list.size() > 4)
                         tileset->setTileOffset(QPoint(list[3].toInt(),list[4].toInt()));
 
-                    gidMapper.insert(gid, tileset);
+                    gidMapper.insert(gid, tileset.data());
                     if (list.size() > 5) {
                         gid += list[5].toInt();
                     } else {
@@ -148,8 +147,7 @@ Tiled::Map *FlarePlugin::read(const QString &fileName)
         } else if (sectionName == QLatin1String("layer")) {
             if (!tilesetsSectionFound) {
                 mError = tr("No tilesets section found before layer section.");
-                delete map;
-                return 0;
+                return nullptr;
             }
             tilelayerSectionFound = true;
             int epos = line.indexOf(QChar('='));
@@ -173,12 +171,11 @@ Tiled::Map *FlarePlugin::read(const QString &fileName)
                         QStringList l = line.split(QChar(','));
                         for (int x=0; x < qMin(map->width(), l.size()); x++) {
                             bool ok;
-                            int tileid = l[x].toInt(0, base);
+                            int tileid = l[x].toInt(nullptr, base);
                             Cell c = gidMapper.gidToCell(tileid, ok);
                             if (!ok) {
                                 mError += tr("Error mapping tile id %1.").arg(tileid);
-                                delete map;
-                                return 0;
+                                return nullptr;
                             }
                             tilelayer->setCell(x, y, c);
                         }
@@ -250,11 +247,10 @@ Tiled::Map *FlarePlugin::read(const QString &fileName)
         mError = tr("This seems to be no valid flare map. "
                     "A Flare map consists of at least a header "
                     "section, a tileset section and one tile layer.");
-        delete map;
-        return 0;
+        return nullptr;
     }
 
-    return map;
+    return map.take();
 }
 
 bool FlarePlugin::supportsFile(const QString &fileName) const
@@ -274,7 +270,8 @@ QString FlarePlugin::errorString() const
 
 bool FlarePlugin::write(const Tiled::Map *map, const QString &fileName)
 {
-    QFile file(fileName);
+    QSaveFile file(fileName);
+
     if (!file.open(QFile::WriteOnly | QFile::Text)) {
         mError = tr("Could not open file for writing.");
         return false;
@@ -305,21 +302,21 @@ bool FlarePlugin::write(const Tiled::Map *map, const QString &fileName)
     QDir mapDir = QFileInfo(fileName).absoluteDir();
 
     out << "[tilesets]\n";
-    foreach (Tileset *ts, map->tilesets()) {
-        const QString &imageSource = ts->imageSource();
+    for (const SharedTileset &tileset : map->tilesets()) {
+        const QString &imageSource = tileset->imageSource();
         QString source = mapDir.relativeFilePath(imageSource);
         out << "tileset=" << source
-            << "," << ts->tileWidth()
-            << "," << ts->tileHeight()
-            << "," << ts->tileOffset().x()
-            << "," << ts->tileOffset().y()
+            << "," << tileset->tileWidth()
+            << "," << tileset->tileHeight()
+            << "," << tileset->tileOffset().x()
+            << "," << tileset->tileOffset().y()
             << "\n";
     }
     out << "\n";
 
     GidMapper gidMapper(map->tilesets());
     // write layers
-    foreach (Layer *layer, map->layers()) {
+    for (Layer *layer : map->layers()) {
         if (TileLayer *tileLayer = layer->asTileLayer()) {
             out << "[layer]\n";
             out << "type=" << layer->name() << "\n";
@@ -341,14 +338,13 @@ bool FlarePlugin::write(const Tiled::Map *map, const QString &fileName)
             out << "\n";
         }
         if (ObjectGroup *group = layer->asObjectGroup()) {
-            foreach (const MapObject *o, group->objects()) {
-                if ((!o->type().isEmpty())) {
+            for (const MapObject *o : group->objects()) {
+                if (!o->type().isEmpty()) {
                     out << "[" << group->name() << "]\n";
 
                     // display object name as comment
-                    if (!(o->name().isEmpty())) {
+                    if (!o->name().isEmpty())
                         out << "# " << o->name() << "\n";
-                    }
 
                     out << "type=" << o->type() << "\n";
                     int x,y,w,h;
@@ -377,10 +373,11 @@ bool FlarePlugin::write(const Tiled::Map *map, const QString &fileName)
             }
         }
     }
-    file.close();
+
+    if (!file.commit()) {
+        mError = file.errorString();
+        return false;
+    }
+
     return true;
 }
-
-#if QT_VERSION < 0x050000
-Q_EXPORT_PLUGIN2(Flare, FlarePlugin)
-#endif
