@@ -23,7 +23,6 @@
 #include "luatablewriter.h"
 
 #include "imagelayer.h"
-#include "map.h"
 #include "mapobject.h"
 #include "objectgroup.h"
 #include "properties.h"
@@ -34,14 +33,7 @@
 
 #include <QFile>
 #include <QCoreApplication>
-
-#if QT_VERSION >= 0x050100
-#define HAS_QSAVEFILE_SUPPORT
-#endif
-
-#ifdef HAS_QSAVEFILE_SUPPORT
 #include <QSaveFile>
-#endif
 
 /**
  * See below for an explanation of the different formats. One of these needs
@@ -60,11 +52,8 @@ LuaPlugin::LuaPlugin()
 
 bool LuaPlugin::write(const Map *map, const QString &fileName)
 {
-#ifdef HAS_QSAVEFILE_SUPPORT
     QSaveFile file(fileName);
-#else
-    QFile file(fileName);
-#endif
+
     if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
         mError = tr("Could not open file for writing.");
         return false;
@@ -82,12 +71,10 @@ bool LuaPlugin::write(const Map *map, const QString &fileName)
         return false;
     }
 
-#ifdef HAS_QSAVEFILE_SUPPORT
     if (!file.commit()) {
         mError = file.errorString();
         return false;
     }
-#endif
 
     return true;
 }
@@ -111,8 +98,10 @@ void LuaPlugin::writeMap(LuaTableWriter &writer, const Map *map)
     writer.writeKeyAndValue("tiledversion", QCoreApplication::applicationVersion());
 
     const QString orientation = orientationToString(map->orientation());
+    const QString renderOrder = renderOrderToString(map->renderOrder());
 
     writer.writeKeyAndValue("orientation", orientation);
+    writer.writeKeyAndValue("renderorder", renderOrder);
     writer.writeKeyAndValue("width", map->width());
     writer.writeKeyAndValue("height", map->height());
     writer.writeKeyAndValue("tilewidth", map->tileWidth());
@@ -149,9 +138,9 @@ void LuaPlugin::writeMap(LuaTableWriter &writer, const Map *map)
 
     mGidMapper.clear();
     unsigned firstGid = 1;
-    foreach (Tileset *tileset, map->tilesets()) {
-        writeTileset(writer, tileset, firstGid);
-        mGidMapper.insert(firstGid, tileset);
+    foreach (const SharedTileset &tileset, map->tilesets()) {
+        writeTileset(writer, tileset.data(), firstGid);
+        mGidMapper.insert(firstGid, tileset.data());
         firstGid += tileset->tileCount();
     }
     writer.writeEndTable();
@@ -160,7 +149,9 @@ void LuaPlugin::writeMap(LuaTableWriter &writer, const Map *map)
     foreach (const Layer *layer, map->layers()) {
         switch (layer->layerType()) {
         case Layer::TileLayerType:
-            writeTileLayer(writer, static_cast<const TileLayer*>(layer));
+            writeTileLayer(writer,
+                           static_cast<const TileLayer*>(layer),
+                           map->layerDataFormat());
             break;
         case Layer::ObjectGroupType:
             writeObjectGroup(writer, static_cast<const ObjectGroup*>(layer));
@@ -200,7 +191,7 @@ static bool includeTile(const Tile *tile)
         return true;
     if (tile->terrain() != 0xFFFFFFFF)
         return true;
-    if (tile->terrainProbability() != 1.f)
+    if (tile->probability() != 1.f)
         return true;
 
     return false;
@@ -261,6 +252,7 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
     }
     writer.writeEndTable();
 
+    writer.writeKeyAndValue("tilecount", tileset->tileCount());
     writer.writeStartTable("tiles");
     for (int i = 0; i < tileset->tileCount(); ++i) {
         const Tile *tile = tileset->tileAt(i);
@@ -295,8 +287,8 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
             writer.setSuppressNewlines(false);
         }
 
-        if (tile->terrainProbability() != 1.f)
-            writer.writeKeyAndValue("probability", tile->terrainProbability());
+        if (tile->probability() != 1.f)
+            writer.writeKeyAndValue("probability", tile->probability());
 
         if (ObjectGroup *objectGroup = tile->objectGroup())
             writeObjectGroup(writer, objectGroup, "objectGroup");
@@ -322,7 +314,8 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
 }
 
 void LuaPlugin::writeTileLayer(LuaTableWriter &writer,
-                               const TileLayer *tileLayer)
+                               const TileLayer *tileLayer,
+                               Map::LayerDataFormat format)
 {
     writer.writeStartTable();
 
@@ -334,18 +327,43 @@ void LuaPlugin::writeTileLayer(LuaTableWriter &writer,
     writer.writeKeyAndValue("height", tileLayer->height());
     writer.writeKeyAndValue("visible", tileLayer->isVisible());
     writer.writeKeyAndValue("opacity", tileLayer->opacity());
+
+    const QPointF offset = tileLayer->offset();
+    writer.writeKeyAndValue("offsetx", offset.x());
+    writer.writeKeyAndValue("offsety", offset.y());
+
     writeProperties(writer, tileLayer->properties());
 
-    writer.writeKeyAndValue("encoding", "lua");
-    writer.writeStartTable("data");
-    for (int y = 0; y < tileLayer->height(); ++y) {
-        if (y > 0)
-            writer.prepareNewLine();
+    switch (format) {
+    case Map::XML:
+    case Map::CSV:
+        writer.writeKeyAndValue("encoding", "lua");
+        writer.writeStartTable("data");
+        for (int y = 0; y < tileLayer->height(); ++y) {
+            if (y > 0)
+                writer.prepareNewLine();
 
-        for (int x = 0; x < tileLayer->width(); ++x)
-            writer.writeValue(mGidMapper.cellToGid(tileLayer->cellAt(x, y)));
+            for (int x = 0; x < tileLayer->width(); ++x)
+                writer.writeValue(mGidMapper.cellToGid(tileLayer->cellAt(x, y)));
+        }
+        writer.writeEndTable();
+        break;
+
+    case Map::Base64:
+    case Map::Base64Zlib:
+    case Map::Base64Gzip: {
+        writer.writeKeyAndValue("encoding", "base64");
+
+        if (format == Map::Base64Zlib)
+            writer.writeKeyAndValue("compression", "zlib");
+        else if (format == Map::Base64Gzip)
+            writer.writeKeyAndValue("compression", "gzip");
+
+        QByteArray layerData = mGidMapper.encodeLayerData(*tileLayer, format);
+        writer.writeKeyAndValue("data", layerData);
+        break;
+    }
     }
-    writer.writeEndTable();
 
     writer.writeEndTable();
 }
@@ -363,6 +381,11 @@ void LuaPlugin::writeObjectGroup(LuaTableWriter &writer,
     writer.writeKeyAndValue("name", objectGroup->name());
     writer.writeKeyAndValue("visible", objectGroup->isVisible());
     writer.writeKeyAndValue("opacity", objectGroup->opacity());
+
+    const QPointF offset = objectGroup->offset();
+    writer.writeKeyAndValue("offsetx", offset.x());
+    writer.writeKeyAndValue("offsety", offset.y());
+
     writeProperties(writer, objectGroup->properties());
 
     writer.writeStartTable("objects");
@@ -512,7 +535,3 @@ void LuaPlugin::writeMapObject(LuaTableWriter &writer,
 
     writer.writeEndTable();
 }
-
-#if QT_VERSION < 0x050000
-Q_EXPORT_PLUGIN2(Lua, LuaPlugin)
-#endif
