@@ -34,7 +34,7 @@ AbstractTool::AbstractTool(const QString &name, const QIcon &icon,
     , mIcon(icon)
     , mShortcut(shortcut)
     , mEnabled(false)
-    , mMapDocument(0)
+    , mMapDocument(nullptr)
 {
 }
 
@@ -50,13 +50,23 @@ void AbstractTool::setStatusInfo(const QString &statusInfo)
     }
 }
 
+/**
+ * Sets the cursor used by this tool. This will be the cursor set on the
+ * viewport of the MapView while the tool is active.
+ */
+void AbstractTool::setCursor(const QCursor &cursor)
+{
+    mCursor = cursor;
+    emit cursorChanged(cursor);
+}
+
 void AbstractTool::setEnabled(bool enabled)
 {
     if (mEnabled == enabled)
         return;
 
     mEnabled = enabled;
-    emit enabledChanged(mEnabled);
+    emit enabledChanged(enabled);
 }
 
 void AbstractTool::keyPressed(QKeyEvent *event)
@@ -70,10 +80,10 @@ void AbstractTool::setMapDocument(MapDocument *mapDocument)
         return;
 
     if (mMapDocument) {
-        disconnect(mMapDocument, SIGNAL(layerChanged(int)),
-                   this, SLOT(updateEnabledState()));
-        disconnect(mMapDocument, SIGNAL(currentLayerIndexChanged(int)),
-                   this, SLOT(updateEnabledState()));
+        disconnect(mMapDocument, &MapDocument::layerChanged,
+                   this, &AbstractTool::updateEnabledState);
+        disconnect(mMapDocument, &MapDocument::currentLayerIndexChanged,
+                   this, &AbstractTool::updateEnabledState);
     }
 
     MapDocument *oldDocument = mMapDocument;
@@ -81,15 +91,15 @@ void AbstractTool::setMapDocument(MapDocument *mapDocument)
     mapDocumentChanged(oldDocument, mMapDocument);
 
     if (mMapDocument) {
-        connect(mMapDocument, SIGNAL(layerChanged(int)),
-                this, SLOT(updateEnabledState()));
-        connect(mMapDocument, SIGNAL(currentLayerIndexChanged(int)),
-                this, SLOT(updateEnabledState()));
+        connect(mMapDocument, &MapDocument::layerChanged,
+                this, &AbstractTool::updateEnabledState);
+        connect(mMapDocument, &MapDocument::currentLayerIndexChanged,
+                this, &AbstractTool::updateEnabledState);
     }
     updateEnabledState();
 }
 
 void AbstractTool::updateEnabledState()
 {
-    setEnabled(mMapDocument != 0);
+    setEnabled(mMapDocument != nullptr);
 }
