@@ -35,7 +35,6 @@
 #include "objectgroup.h"
 #include "tile.h"
 #include "tilelayer.h"
-#include "tileset.h"
 #include "tilesetmanager.h"
 
 #include <QDebug>
@@ -54,10 +53,10 @@ using namespace Tiled::Internal;
 AutoMapper::AutoMapper(MapDocument *workingDocument, Map *rules,
                        const QString &rulePath)
     : mMapDocument(workingDocument)
-    , mMapWork(workingDocument ? workingDocument->map() : 0)
+    , mMapWork(workingDocument ? workingDocument->map() : nullptr)
     , mMapRules(rules)
-    , mLayerInputRegions(0)
-    , mLayerOutputRegions(0)
+    , mLayerInputRegions(nullptr)
+    , mLayerOutputRegions(nullptr)
     , mRulePath(rulePath)
     , mDeleteTiles(false)
     , mAutoMappingRadius(0)
@@ -404,17 +403,17 @@ bool AutoMapper::setupCorrectIndexes()
 // because here mAddedTileset is modified.
 bool AutoMapper::setupTilesets(Map *src, Map *dst)
 {
-    QList<Tileset*> existingTilesets = dst->tilesets();
+    const QVector<SharedTileset> &existingTilesets = dst->tilesets();
     TilesetManager *tilesetManager = TilesetManager::instance();
 
     // Add tilesets that are not yet part of dst map
-    foreach (Tileset *tileset, src->tilesets()) {
+    foreach (const SharedTileset &tileset, src->tilesets()) {
         if (existingTilesets.contains(tileset))
             continue;
 
         QUndoStack *undoStack = mMapDocument->undoStack();
 
-        Tileset *replacement = tileset->findSimilarTileset(existingTilesets);
+        SharedTileset replacement = tileset->findSimilarTileset(existingTilesets);
         if (!replacement) {
             mAddedTilesets.append(tileset);
             undoStack->push(new AddTileset(mMapDocument, tileset));
@@ -533,10 +532,9 @@ QRect AutoMapper::applyRule(const int ruleIndex, const QRect &where)
     // been altered by exactly this rule. We store all the altered parts to
     // make sure there are no overlaps of the same rule applied to
     // (neighbouring) places
-    QList<QRegion> appliedRegions;
+    QVector<QRegion> appliedRegions;
     if (mNoOverlappingRules)
-        for (int i = 0; i < mMapWork->layerCount(); i++)
-            appliedRegions.append(QRegion());
+        appliedRegions.resize(mMapWork->layerCount());
 
     for (int y = minY; y <= maxY; ++y)
     for (int x = minX; x <= maxX; ++x) {
@@ -641,7 +639,7 @@ static QVector<Cell> cellsInRegion(const QVector<TileLayer*> &list,
  * automapping.
  * In this function a certain region (of the set layer) is compared to
  * several other layers (ruleSet and ruleNotSet).
- * This comparision will determine if a rule of automapping matches,
+ * This comparison will determine if a rule of automapping matches,
  * so if this rule is applied at this region given
  * by a QRegion and Offset given by a QPoint.
  *
@@ -691,7 +689,7 @@ static QVector<Cell> cellsInRegion(const QVector<TileLayer*> &list,
  *      This exception was added to have a better functionality
  *      (need of less layers.)
  *      It was not added to the case, when having only listNo layers to
- *      avoid total symmetrie between those lists.
+ *      avoid total symmetry between those lists.
  *
  * If all positions are considered good, return true.
  * return false otherwise.
@@ -872,16 +870,16 @@ void AutoMapper::cleanAll()
 
 void AutoMapper::cleanTilesets()
 {
-    foreach (Tileset *tileset, mAddedTilesets) {
-        if (mMapWork->isTilesetUsed(tileset))
+    foreach (const SharedTileset &tileset, mAddedTilesets) {
+        if (mMapWork->isTilesetUsed(tileset.data()))
             continue;
 
-        const int layerIndex = mMapWork->indexOfTileset(tileset);
-        if (layerIndex == -1)
+        const int index = mMapWork->indexOfTileset(tileset);
+        if (index == -1)
             continue;
 
         QUndoStack *undo = mMapDocument->undoStack();
-        undo->push(new RemoveTileset(mMapDocument, layerIndex, tileset));
+        undo->push(new RemoveTileset(mMapDocument, index));
     }
     mAddedTilesets.clear();
 }
@@ -916,7 +914,7 @@ void AutoMapper::cleanUpRulesMap()
     tilesetManager->removeReferences(mMapRules->tilesets());
 
     delete mMapRules;
-    mMapRules = 0;
+    mMapRules = nullptr;
 
     cleanUpRuleMapLayers();
     mRulesInput.clear();
@@ -933,7 +931,7 @@ void AutoMapper::cleanUpRuleMapLayers()
 
     mLayerList.clear();
     // do not delete mLayerRuleRegions, it is owned by the rulesmap
-    mLayerInputRegions = 0;
-    mLayerOutputRegions = 0;
+    mLayerInputRegions = nullptr;
+    mLayerOutputRegions = nullptr;
     mInputRules.clear();
 }
