@@ -25,7 +25,7 @@
 #include "mapdocument.h"
 #include "tilelayer.h"
 #include "tilesetmanager.h"
-#include "tmxmapreader.h"
+#include "tmxmapformat.h"
 #include "preferences.h"
 
 #include <QFileInfo>
@@ -36,7 +36,7 @@ using namespace Tiled::Internal;
 
 AutomappingManager::AutomappingManager(QObject *parent)
     : QObject(parent)
-    , mMapDocument(0)
+    , mMapDocument(nullptr)
     , mLoaded(false)
 {
 }
@@ -55,7 +55,7 @@ void AutomappingManager::autoMap()
     int w = map->width();
     int h = map->height();
 
-    autoMapInternal(QRect(0, 0, w, h), 0);
+    autoMapInternal(QRect(0, 0, w, h), nullptr);
 }
 
 void AutomappingManager::autoMap(const QRegion &where, Layer *touchedLayer)
@@ -72,7 +72,7 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
     if (!mMapDocument)
         return;
 
-    const bool automatic = touchedLayer != 0;
+    const bool automatic = touchedLayer != nullptr;
 
     if (!mLoaded) {
         const QString mapPath = QFileInfo(mMapDocument->fileName()).path();
@@ -85,10 +85,6 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
         }
     }
 
-    // use a pointer to the region, so each automapper can manipulate it and the
-    // following automappers do see the impact
-    QRegion *passedRegion = new QRegion(where);
-
     QVector<AutoMapper*> passedAutoMappers;
     if (touchedLayer) {
         foreach (AutoMapper *a, mAutoMappers) {
@@ -99,9 +95,13 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
         passedAutoMappers = mAutoMappers;
     }
     if (!passedAutoMappers.isEmpty()) {
+        // use a copy of the region, so each automapper can manipulate it and the
+        // following automappers do see the impact
+        QRegion region(where);
+
         QUndoStack *undoStack = mMapDocument->undoStack();
         undoStack->beginMacro(tr("Apply AutoMap rules"));
-        AutoMapperWrapper *aw = new AutoMapperWrapper(mMapDocument, passedAutoMappers, passedRegion);
+        AutoMapperWrapper *aw = new AutoMapperWrapper(mMapDocument, passedAutoMappers, &region);
         undoStack->push(aw);
         undoStack->endMacro();
     }
@@ -110,9 +110,6 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
         mError += automapper->errorString();
     }
 
-    mMapDocument->emitRegionChanged(*passedRegion);
-    delete passedRegion;
-
     if (!mWarning.isEmpty())
         emit warningsOccurred(automatic);
 
@@ -156,13 +153,13 @@ bool AutomappingManager::loadFile(const QString &filePath)
             continue;
         }
         if (rulePath.endsWith(QLatin1String(".tmx"), Qt::CaseInsensitive)) {
-            TmxMapReader mapReader;
+            TmxMapFormat tmxFormat;
 
-            Map *rules = mapReader.read(rulePath);
+            Map *rules = tmxFormat.read(rulePath);
 
             if (!rules) {
                 mError += tr("Opening rules map failed:\n%1").arg(
-                        mapReader.errorString()) + QLatin1Char('\n');
+                        tmxFormat.errorString()) + QLatin1Char('\n');
                 ret = false;
                 continue;
             }
