@@ -36,12 +36,14 @@ using namespace Tiled;
 using namespace Tiled::Internal;
 
 BrushItem::BrushItem():
-    mMapDocument(0),
-    mTileLayer(0)
+    mMapDocument(nullptr)
 {
     setFlag(QGraphicsItem::ItemUsesExtendedStyleOption);
 }
 
+/**
+ * Sets the map document this brush is operating on.
+ */
 void BrushItem::setMapDocument(MapDocument *mapDocument)
 {
     if (mMapDocument == mapDocument)
@@ -50,25 +52,25 @@ void BrushItem::setMapDocument(MapDocument *mapDocument)
     mMapDocument = mapDocument;
 
     // The tiles in the stamp may no longer be valid
-    setTileLayer(0);
-    updateBoundingRect();
+    clear();
 }
 
-void BrushItem::setTileLayer(const TileLayer *tileLayer)
+/**
+ * Sets a tile layer representing this brush. When no tile layer is set,
+ * the brush only draws the selection color.
+ */
+void BrushItem::setTileLayer(const SharedTileLayer &tileLayer)
 {
-    delete mTileLayer;
+    mTileLayer = tileLayer;
+    mRegion = tileLayer ? tileLayer->region() : QRegion();
 
-    if (tileLayer) {
-        mTileLayer = static_cast<TileLayer*>(tileLayer->clone());
-        mRegion = mTileLayer->region();
-    } else {
-        mTileLayer = 0;
-        mRegion = QRegion();
-    }
     updateBoundingRect();
     update();
 }
 
+/**
+ * Changes the position of the tile layer, if one is set.
+ */
 void BrushItem::setTileLayerPosition(const QPoint &pos)
 {
     if (!mTileLayer)
@@ -84,6 +86,9 @@ void BrushItem::setTileLayerPosition(const QPoint &pos)
     updateBoundingRect();
 }
 
+/**
+ * Sets the region of tiles that this brush item occupies.
+ */
 void BrushItem::setTileRegion(const QRegion &region)
 {
     if (mRegion == region)
@@ -93,6 +98,14 @@ void BrushItem::setTileRegion(const QRegion &region)
     updateBoundingRect();
 }
 
+/**
+ * Sets the layer offset used by the currently active layer.
+ */
+void BrushItem::setLayerOffset(const QPointF &offset)
+{
+    setPos(offset);
+}
+
 QRectF BrushItem::boundingRect() const
 {
     return mBoundingRect;
@@ -116,7 +129,7 @@ void BrushItem::paint(QPainter *painter,
     if (mTileLayer) {
         const qreal opacity = painter->opacity();
         painter->setOpacity(0.75);
-        renderer->drawTileLayer(painter, mTileLayer, option->exposedRect);
+        renderer->drawTileLayer(painter, mTileLayer.data(), option->exposedRect);
         painter->setOpacity(opacity);
     }
 
