@@ -23,12 +23,13 @@
 
 #include "bucketfilltool.h"
 
+#include "addremovetileset.h"
 #include "brushitem.h"
-#include "filltiles.h"
 #include "tilepainter.h"
 #include "tile.h"
 #include "mapscene.h"
 #include "mapdocument.h"
+#include "painttilelayer.h"
 
 #include <QApplication>
 
@@ -41,8 +42,6 @@ BucketFillTool::BucketFillTool(QObject *parent)
                                ":images/22x22/stock-tool-bucket-fill.png")),
                        QKeySequence(tr("F")),
                        parent)
-    , mStamp(0)
-    , mFillOverlay(0)
     , mIsActive(false)
     , mLastShiftStatus(false)
     , mIsRandom(false)
@@ -52,8 +51,6 @@ BucketFillTool::BucketFillTool(QObject *parent)
 
 BucketFillTool::~BucketFillTool()
 {
-    delete mStamp;
-    delete mFillOverlay;
 }
 
 void BucketFillTool::activate(MapScene *scene)
@@ -72,26 +69,49 @@ void BucketFillTool::deactivate(MapScene *scene)
     mIsActive = false;
 }
 
+static void fillWithStamp(TileLayer &layer,
+                          const TileStamp &stamp,
+                          const QRegion &mask)
+{
+    const QSize size = stamp.maxSize();
+
+    // Fill the entire layer with random variations of the stamp
+    for (int y = 0; y < layer.height(); y += size.height()) {
+        for (int x = 0; x < layer.width(); x += size.width()) {
+            const TileStampVariation variation = stamp.randomVariation();
+            layer.setCells(x, y, variation.tileLayer());
+        }
+    }
+
+    // Erase tiles outside of the masked region. This can easily be faster than
+    // avoiding to place tiles outside of the region in the first place.
+    layer.erase(QRegion(0, 0, layer.width(), layer.height()) - mask);
+}
+
 void BucketFillTool::tilePositionChanged(const QPoint &tilePos)
 {
-    bool shiftPressed = QApplication::keyboardModifiers() & Qt::ShiftModifier;
-    bool fillRegionChanged = false;
+    // Skip filling if the stamp is empty
+    if (mStamp.isEmpty())
+        return;
 
     // Make sure that a tile layer is selected
     TileLayer *tileLayer = currentTileLayer();
     if (!tileLayer)
         return;
 
-    // Skip filling if the stamp is empty
-    if (!mStamp || mStamp->isEmpty())
-        return;
+    bool shiftPressed = QApplication::keyboardModifiers() & Qt::ShiftModifier;
+    bool fillRegionChanged = false;
 
     TilePainter regionComputer(mapDocument(), tileLayer);
+
     // If the stamp is a single tile, ignore it when making the region
-    if (mStamp->width() == 1 && mStamp->height() == 1 && !shiftPressed &&
-            mStamp->cellAt(0, 0) == regionComputer.cellAt(tilePos.x(),
-                                                           tilePos.y()))
-        return;
+    if (!shiftPressed && mStamp.variations().size() == 1) {
+        const TileStampVariation &variation = mStamp.variations().first();
+        TileLayer *stampLayer = variation.tileLayer();
+        if (stampLayer->size() == QSize(1, 1) &&
+                stampLayer->cellAt(0, 0) == regionComputer.cellAt(tilePos))
+            return;
+    }
 
     // This clears the connections so we don't get callbacks
     clearConnections(mapDocument());
@@ -99,7 +119,7 @@ void BucketFillTool::tilePositionChanged(const QPoint &tilePos)
     // Optimization: we don't need to recalculate the fill area
     // if the new mouse position is still over the filled region
     // and the shift modifier hasn't changed.
-    if (!mFillRegion.contains(tilePos) || shiftPressed != mLastShiftStatus || mIsRandom) {
+    if (!mFillRegion.contains(tilePos) || shiftPressed != mLastShiftStatus) {
 
         // Clear overlay to make way for a new one
         clearOverlay();
@@ -138,22 +158,22 @@ void BucketFillTool::tilePositionChanged(const QPoint &tilePos)
     if (!mFillOverlay) {
         // Create a new overlay region
         const QRect fillBounds = mFillRegion.boundingRect();
-        mFillOverlay = new TileLayer(QString(),
-                                     fillBounds.x(),
-                                     fillBounds.y(),
-                                     fillBounds.width(),
-                                     fillBounds.height());
+        mFillOverlay = SharedTileLayer(new TileLayer(QString(),
+                                                     fillBounds.x(),
+                                                     fillBounds.y(),
+                                                     fillBounds.width(),
+                                                     fillBounds.height()));
     }
 
     // Paint the new overlay
-    TilePainter tilePainter(mapDocument(), mFillOverlay);
     if (!mIsRandom) {
-        if (fillRegionChanged)
-            tilePainter.drawStamp(mStamp, mFillRegion);
+        if (fillRegionChanged || mStamp.variations().size() > 1) {
+            fillWithStamp(*mFillOverlay, mStamp,
+                          mFillRegion.translated(-mFillOverlay->position()));
+            fillRegionChanged = true;
+        }
     } else {
-        TileLayer *stamp = getRandomTileLayer(mFillRegion);
-        tilePainter.drawStamp(stamp, mFillRegion);
-        delete stamp;
+        randomFill(*mFillOverlay, mFillRegion);
         fillRegionChanged = true;
     }
 
@@ -172,13 +192,27 @@ void BucketFillTool::mousePressed(QGraphicsSceneMouseEvent *event)
     if (!brushItem()->isVisible())
         return;
 
-    FillTiles *fillTiles = new FillTiles(mapDocument(),
-                                         currentTileLayer(),
-                                         mFillRegion,
-                                         brushItem()->tileLayer());
+    const TileLayer *preview = mFillOverlay.data();
+    if (!preview)
+        return;
+
+    PaintTileLayer *paint = new PaintTileLayer(mapDocument(),
+                                               currentTileLayer(),
+                                               preview->x(),
+                                               preview->y(),
+                                               preview);
+
+    paint->setText(QCoreApplication::translate("Undo Commands", "Fill Area"));
+
+    if (!mMissingTilesets.isEmpty()) {
+        for (const SharedTileset &tileset : mMissingTilesets)
+            new AddTileset(mapDocument(), tileset, paint);
+
+        mMissingTilesets.clear();
+    }
 
     QRegion fillRegion(mFillRegion);
-    mapDocument()->undoStack()->push(fillTiles);
+    mapDocument()->undoStack()->push(paint);
     mapDocument()->emitRegionEdited(fillRegion, currentTileLayer());
 }
 
@@ -208,21 +242,20 @@ void BucketFillTool::mapDocumentChanged(MapDocument *oldDocument,
 
     clearConnections(oldDocument);
 
-    // Reset things that are probably invalid now
-    setStamp(0);
+    if (newDocument)
+        updateRandomListAndMissingTilesets();
+
     clearOverlay();
 }
 
-void BucketFillTool::setStamp(TileLayer *stamp)
+void BucketFillTool::setStamp(const TileStamp &stamp)
 {
     // Clear any overlay that we presently have with an old stamp
     clearOverlay();
 
-    delete mStamp;
     mStamp = stamp;
 
-    if (mIsRandom)
-        updateRandomList();
+    updateRandomListAndMissingTilesets();
 
     if (mIsActive && brushItem()->isVisible())
         tilePositionChanged(tilePosition());
@@ -234,12 +267,9 @@ void BucketFillTool::clearOverlay()
     // risk of getting a callback and causing an infinite loop
     clearConnections(mapDocument());
 
-    brushItem()->setTileLayer(0);
-    delete mFillOverlay;
-    mFillOverlay = 0;
-
+    brushItem()->clear();
+    mFillOverlay.clear();
     mFillRegion = QRegion();
-    brushItem()->setTileRegion(QRegion());
 }
 
 void BucketFillTool::makeConnections()
@@ -248,17 +278,17 @@ void BucketFillTool::makeConnections()
         return;
 
     // Overlay may need to be cleared if a region changed
-    connect(mapDocument(), SIGNAL(regionChanged(QRegion)),
-            this, SLOT(clearOverlay()));
+    connect(mapDocument(), &MapDocument::regionChanged,
+            this, &BucketFillTool::clearOverlay);
 
     // Overlay needs to be cleared if we switch to another layer
-    connect(mapDocument(), SIGNAL(currentLayerIndexChanged(int)),
-            this, SLOT(clearOverlay()));
+    connect(mapDocument(), &MapDocument::currentLayerIndexChanged,
+            this, &BucketFillTool::clearOverlay);
 
     // Overlay needs be cleared if the selection changes, since
     // the overlay may be bound or may need to be bound to the selection
-    connect(mapDocument(), SIGNAL(selectedAreaChanged(QRegion,QRegion)),
-            this, SLOT(clearOverlay()));
+    connect(mapDocument(), &MapDocument::selectedAreaChanged,
+            this, &BucketFillTool::clearOverlay);
 }
 
 void BucketFillTool::clearConnections(MapDocument *mapDocument)
@@ -266,14 +296,14 @@ void BucketFillTool::clearConnections(MapDocument *mapDocument)
     if (!mapDocument)
         return;
 
-    disconnect(mapDocument, SIGNAL(regionChanged(QRegion)),
-               this, SLOT(clearOverlay()));
+    disconnect(mapDocument, &MapDocument::regionChanged,
+               this, &BucketFillTool::clearOverlay);
 
-    disconnect(mapDocument, SIGNAL(currentLayerIndexChanged(int)),
-               this, SLOT(clearOverlay()));
+    disconnect(mapDocument, &MapDocument::currentLayerIndexChanged,
+               this, &BucketFillTool::clearOverlay);
 
-    disconnect(mapDocument, SIGNAL(selectedAreaChanged(QRegion,QRegion)),
-               this, SLOT(clearOverlay()));
+    disconnect(mapDocument, &MapDocument::selectedAreaChanged,
+               this, &BucketFillTool::clearOverlay);
 }
 
 void BucketFillTool::setRandom(bool value)
@@ -282,11 +312,7 @@ void BucketFillTool::setRandom(bool value)
         return;
 
     mIsRandom = value;
-
-    if (mIsRandom)
-        updateRandomList();
-    else
-        mRandomList.clear();
+    updateRandomListAndMissingTilesets();
 
     // Don't need to recalculate fill region if there was no fill region
     if (!mFillOverlay)
@@ -295,38 +321,34 @@ void BucketFillTool::setRandom(bool value)
     tilePositionChanged(tilePosition());
 }
 
-TileLayer *BucketFillTool::getRandomTileLayer(const QRegion &region) const
+void BucketFillTool::randomFill(TileLayer &tileLayer, const QRegion &region) const
 {
-    QRect bb = region.boundingRect();
-    TileLayer *result = new TileLayer(QString(), bb.x(), bb.y(),
-                                      bb.width(), bb.height());
+    if (region.isEmpty() || mRandomCellPicker.isEmpty())
+        return;
 
-    if (region.isEmpty() || mRandomList.empty())
-        return result;
-
-    foreach (const QRect &rect, region.rects()) {
+    for (const QRect &rect : region.translated(-tileLayer.position()).rects()) {
         for (int _x = rect.left(); _x <= rect.right(); ++_x) {
             for (int _y = rect.top(); _y <= rect.bottom(); ++_y) {
-
-                result->setCell(_x - bb.x(),
-                                _y - bb.y(),
-                                mRandomList.at(rand() % mRandomList.size()));
+                tileLayer.setCell(_x, _y,
+                                  mRandomCellPicker.pick());
             }
         }
     }
-    return result;
 }
 
-void BucketFillTool::updateRandomList()
+void BucketFillTool::updateRandomListAndMissingTilesets()
 {
-    mRandomList.clear();
+    mRandomCellPicker.clear();
+    mMissingTilesets.clear();
 
-    if (!mStamp)
-        return;
+    for (const TileStampVariation &variation : mStamp.variations()) {
+        mapDocument()->unifyTilesets(variation.map, mMissingTilesets);
 
-    for (int x = 0; x < mStamp->width(); x++)
-        for (int y = 0; y < mStamp->height(); y++)
-            if (!mStamp->cellAt(x, y).isEmpty())
-                mRandomList.append(mStamp->cellAt(x, y));
+        if (mIsRandom) {
+            for (const Cell &cell : *variation.tileLayer()) {
+                if (!cell.isEmpty())
+                    mRandomCellPicker.add(cell, cell.tile->probability());
+            }
+        }
+    }
 }
-
