@@ -40,8 +40,8 @@ public:
                           int to,
                           int count);
 
-    void undo();
-    void redo();
+    void undo() override;
+    void redo() override;
 
 private:
     MapDocument *mMapDocument;
