@@ -47,8 +47,8 @@ public:
                      const QString &kind,
                      Object *object,
                      const Properties &newProperties);
-    void undo();
-    void redo();
+    void undo() override;
+    void redo() override;
 
 private:
     void swapProperties();
@@ -73,10 +73,10 @@ public:
                 const QList<Object*> &objects,
                 const QString &name,
                 const QString &value,
-                QUndoCommand *parent = 0);
+                QUndoCommand *parent = nullptr);
 
-    void undo();
-    void redo();
+    void undo() override;
+    void redo() override;
 
 private:
     struct ObjectProperty {
@@ -103,10 +103,10 @@ public:
     RemoveProperty(MapDocument *mapDocument,
                    const QList<Object*> &objects,
                    const QString &name,
-                   QUndoCommand *parent = 0);
+                   QUndoCommand *parent = nullptr);
 
-    void undo();
-    void redo();
+    void undo() override;
+    void redo() override;
 
 private:
     MapDocument *mMapDocument;
