@@ -25,8 +25,8 @@ public:
 
     ~ChangeTileObjectGroup();
 
-    void undo() { swap(); }
-    void redo() { swap(); }
+    void undo() override { swap(); }
+    void redo() override { swap(); }
 
 private:
     void swap();
