@@ -38,8 +38,8 @@ public:
                           Tile *tile,
                           float probability);
 
-    void undo() { swap(); }
-    void redo() { swap(); }
+    void undo() override { swap(); }
+    void redo() override { swap(); }
 
 private:
     void swap();
