@@ -28,8 +28,7 @@
 #include "mapview.h"
 #include "objectgroup.h"
 #include "snaphelper.h"
-#include "tmxmapreader.h"
-#include "tmxmapwriter.h"
+#include "tmxmapformat.h"
 #include "tile.h"
 #include "tilelayer.h"
 
@@ -44,7 +43,7 @@ static const char * const TMX_MIMETYPE = "text/tmx";
 using namespace Tiled;
 using namespace Tiled::Internal;
 
-ClipboardManager *ClipboardManager::mInstance = 0;
+ClipboardManager *ClipboardManager::mInstance;
 
 ClipboardManager::ClipboardManager() :
     mHasMap(false)
@@ -65,7 +64,7 @@ ClipboardManager *ClipboardManager::instance()
 void ClipboardManager::deleteInstance()
 {
     delete mInstance;
-    mInstance = 0;
+    mInstance = nullptr;
 }
 
 Map *ClipboardManager::map() const
@@ -73,18 +72,18 @@ Map *ClipboardManager::map() const
     const QMimeData *mimeData = mClipboard->mimeData();
     const QByteArray data = mimeData->data(QLatin1String(TMX_MIMETYPE));
     if (data.isEmpty())
-        return 0;
+        return nullptr;
 
-    TmxMapReader reader;
-    return reader.fromByteArray(data);
+    TmxMapFormat format;
+    return format.fromByteArray(data);
 }
 
 void ClipboardManager::setMap(const Map *map)
 {
-    TmxMapWriter mapWriter;
+    TmxMapFormat format;
 
     QMimeData *mimeData = new QMimeData;
-    mimeData->setData(QLatin1String(TMX_MIMETYPE), mapWriter.toByteArray(map));
+    mimeData->setData(QLatin1String(TMX_MIMETYPE), format.toByteArray(map));
 
     mClipboard->setMimeData(mimeData);
 }
@@ -99,7 +98,7 @@ void ClipboardManager::copySelection(const MapDocument *mapDocument)
     const QRegion &selectedArea = mapDocument->selectedArea();
     const QList<MapObject*> &selectedObjects = mapDocument->selectedObjects();
     const TileLayer *tileLayer = dynamic_cast<const TileLayer*>(currentLayer);
-    Layer *copyLayer = 0;
+    Layer *copyLayer = nullptr;
 
     if (!selectedArea.isEmpty() && tileLayer) {
         // Copy the selected part of the layer
@@ -123,7 +122,7 @@ void ClipboardManager::copySelection(const MapDocument *mapDocument)
     copyMap.setRenderOrder(map->renderOrder());
 
     // Resolve the set of tilesets used by this layer
-    foreach (Tileset *tileset, copyLayer->usedTilesets())
+    foreach (const SharedTileset &tileset, copyLayer->usedTilesets())
         copyMap.addTileset(tileset);
 
     copyMap.addLayer(copyLayer);
