@@ -19,8 +19,10 @@
  */
 
 #include "command.h"
+
 #include "documentmanager.h"
 #include "mapdocument.h"
+#include "mapobject.h"
 
 #include <QMessageBox>
 #include <QSettings>
@@ -109,7 +111,7 @@ CommandProcess::CommandProcess(const Command &command, bool inTerminal)
 
     // Modify the command to run in a terminal
     if (inTerminal) {
-#ifdef Q_WS_X11
+#ifdef Q_OS_LINUX
         static bool hasGnomeTerminal = QProcess::execute(
                                     QLatin1String("which gnome-terminal")) == 0;
 
@@ -120,7 +122,7 @@ CommandProcess::CommandProcess(const Command &command, bool inTerminal)
 #elif defined(Q_OS_MAC)
         // The only way I know to launch a Terminal with a command on mac is
         // to make a .command file and open it. The client command invoke the
-        // exectuable directly (rather than using open) in order to get std
+        // executable directly (rather than using open) in order to get std
         // output in the terminal. Otherwise, you can use the Console
         // application to see the output.
 
