@@ -33,13 +33,18 @@ class CreateMultipointObjectTool : public CreateObjectTool
 
 public:
     CreateMultipointObjectTool(QObject *parent);
-    void startNewMapObject(const QPointF &pos, ObjectGroup *objectGroup);
-    void languageChanged() = 0;
+    ~CreateMultipointObjectTool();
+
+    void startNewMapObject(const QPointF &pos, ObjectGroup *objectGroup) override;
 
 protected:
     void mouseMovedWhileCreatingObject(const QPointF &pos,
-                                       Qt::KeyboardModifiers modifiers);
-    void mousePressedWhileCreatingObject(QGraphicsSceneMouseEvent *event);
+                                       Qt::KeyboardModifiers modifiers) override;
+    void mousePressedWhileCreatingObject(QGraphicsSceneMouseEvent *event) override;
+
+private:
+    MapObject *mOverlayPolygonObject;
+    ObjectGroup *mOverlayObjectGroup;
 };
 
 } // namespace Internal
