@@ -24,7 +24,6 @@
 #include "createmultipointobjecttool.h"
 
 namespace Tiled {
-
 namespace Internal {
 
 class CreatePolygonObjectTool: public CreateMultipointObjectTool
@@ -32,13 +31,13 @@ class CreatePolygonObjectTool: public CreateMultipointObjectTool
     Q_OBJECT
 public:
     CreatePolygonObjectTool(QObject *parent);
-    void languageChanged();
+    void languageChanged() override;
 protected:
-    MapObject *createNewMapObject();
-    void finishNewMapObject();
+    MapObject *createNewMapObject() override;
+    void finishNewMapObject() override;
 };
 
-}
-}
+} // namespace Internal
+} // namespace Tiled
 
 #endif // CREATEPOLYGONOBJECTTOOL_H
