@@ -21,6 +21,7 @@
 #include "createtileobjecttool.h"
 
 #include "mapdocument.h"
+#include "mapobject.h"
 #include "mapobjectitem.h"
 #include "maprenderer.h"
 #include "snaphelper.h"
@@ -82,7 +83,7 @@ void CreateTileObjectTool::languageChanged()
 MapObject *CreateTileObjectTool::createNewMapObject()
 {
     if (!mTile)
-        return 0;
+        return nullptr;
 
     MapObject *newMapObject = new MapObject;
     newMapObject->setShape(MapObject::Rectangle);
