@@ -27,6 +27,7 @@
 #include "mapdocument.h"
 #include "mapobject.h"
 #include "mapobjectitem.h"
+#include "mapobjectmodel.h"
 #include "maprenderer.h"
 #include "mapscene.h"
 #include "rangeset.h"
@@ -67,7 +68,7 @@ public:
     }
 
     enum { Type = UserType + 2 };
-    int type() const { return Type; }
+    int type() const override { return Type; }
 
     MapObjectItem *mapObjectItem() const { return mMapObjectItem; }
     MapObject *mapObject() const { return mMapObjectItem->mapObject(); }
@@ -80,10 +81,10 @@ public:
     void setSelected(bool selected) { mSelected = selected; update(); }
     bool isSelected() const { return mSelected; }
 
-    QRectF boundingRect() const;
+    QRectF boundingRect() const override;
     void paint(QPainter *painter,
                const QStyleOptionGraphicsItem *option,
-               QWidget *widget = 0);
+               QWidget *widget = nullptr) override;
 
 private:
     MapObjectItem *mMapObjectItem;
@@ -131,8 +132,8 @@ EditPolygonTool::EditPolygonTool(QObject *parent)
           parent)
     , mSelectionRectangle(new SelectionRectangle)
     , mMousePressed(false)
-    , mClickedHandle(0)
-    , mClickedObjectItem(0)
+    , mClickedHandle(nullptr)
+    , mClickedObjectItem(nullptr)
     , mMode(NoMode)
 {
 }
@@ -174,7 +175,7 @@ void EditPolygonTool::deactivate(MapScene *scene)
 
     mHandles.clear();
     mSelectedHandles.clear();
-    mClickedHandle = 0;
+    mClickedHandle = nullptr;
 
     AbstractObjectTool::deactivate(scene);
 }
@@ -218,7 +219,7 @@ static T *first(const QList<QGraphicsItem *> &items)
         if (T *t = qgraphicsitem_cast<T*>(item))
             return t;
     }
-    return 0;
+    return nullptr;
 }
 
 static QTransform viewTransform(QGraphicsSceneMouseEvent *event)
@@ -324,7 +325,7 @@ void EditPolygonTool::mouseReleased(QGraphicsSceneMouseEvent *event)
     }
 
     mMousePressed = false;
-    mClickedHandle = 0;
+    mClickedHandle = nullptr;
 }
 
 void EditPolygonTool::modifiersChanged(Qt::KeyboardModifiers modifiers)
@@ -533,12 +534,22 @@ void EditPolygonTool::updateMovingItems(const QPointF &pos,
 
     int i = 0;
     foreach (PointHandle *handle, mSelectedHandles) {
+        // update handle position
+        const QPointF newScreenPos = mOldHandlePositions.at(i) + diff;
+        handle->setPos(newScreenPos);
+
+        // calculate new pixel position of polygon node
         const MapObjectItem *item = handle->mapObjectItem();
-        const QPointF newPixelPos = mOldHandlePositions.at(i) + diff;
-        const QPointF newInternalPos = item->mapFromScene(newPixelPos);
+        const QPointF newInternalPos = item->mapFromScene(newScreenPos);
         const QPointF newScenePos = item->pos() + newInternalPos;
-        handle->setPos(newPixelPos);
-        handle->setPointPosition(renderer->screenToPixelCoords(newScenePos));
+        const QPointF newPixelPos = renderer->screenToPixelCoords(newScenePos);
+
+        // update the polygon
+        MapObject *mapObject = item->mapObject();
+        QPolygonF polygon = mapObject->polygon();
+        polygon[handle->pointIndex()] = newPixelPos - mapObject->position();
+        mapDocument()->mapObjectModel()->setObjectPolygon(mapObject, polygon);
+
         ++i;
     }
 }
