@@ -26,7 +26,7 @@
 #include "pluginmanager.h"
 #include "mapdocument.h"
 #include "mapreader.h"
-#include "mapwriterinterface.h"
+#include "mapformat.h"
 #include "preferences.h"
 #include "tiledapplication.h"
 #include "tileset.h"
@@ -37,21 +37,18 @@
 #include <QStyle>
 #include <QStyleFactory>
 
-#ifdef STATIC_BUILD
-Q_IMPORT_PLUGIN(qgif)
-Q_IMPORT_PLUGIN(qjpeg)
-Q_IMPORT_PLUGIN(qtiff)
-#endif
-
 #define STRINGIFY(x) #x
 #define AS_STRING(x) STRINGIFY(x)
 
+using namespace Tiled;
 using namespace Tiled::Internal;
 
 namespace {
 
 class CommandLineHandler : public CommandLineParser
 {
+    Q_DECLARE_TR_FUNCTIONS(CommandLineHandler)
+
 public:
     CommandLineHandler();
 
@@ -92,34 +89,34 @@ CommandLineHandler::CommandLineHandler()
     option<&CommandLineHandler::showVersion>(
                 QLatin1Char('v'),
                 QLatin1String("--version"),
-                QLatin1String("Display the version"));
+                tr("Display the version"));
 
     option<&CommandLineHandler::justQuit>(
                 QChar(),
                 QLatin1String("--quit"),
-                QLatin1String("Only check validity of arguments"));
+                tr("Only check validity of arguments"));
 
     option<&CommandLineHandler::setDisableOpenGL>(
                 QChar(),
                 QLatin1String("--disable-opengl"),
-                QLatin1String("Disable hardware accelerated rendering"));
+                tr("Disable hardware accelerated rendering"));
 
     option<&CommandLineHandler::setExportMap>(
                 QChar(),
                 QLatin1String("--export-map"),
-                QLatin1String("Export the specified tmx file to target"));
+                tr("Export the specified tmx file to target"));
 
     option<&CommandLineHandler::showExportFormats>(
                 QChar(),
                 QLatin1String("--export-formats"),
-                QLatin1String("Print a list of supported export formats"));
+                tr("Print a list of supported export formats"));
 }
 
 void CommandLineHandler::showVersion()
 {
     if (!showedVersion) {
         showedVersion = true;
-        qWarning() << qPrintable(QApplication::applicationName())
+        qWarning() << qPrintable(QApplication::applicationDisplayName())
                    << qPrintable(QApplication::applicationVersion());
         quit = true;
     }
@@ -142,14 +139,13 @@ void CommandLineHandler::setExportMap()
 
 void CommandLineHandler::showExportFormats()
 {
-    PluginManager *pluginManager = PluginManager::instance();
-    pluginManager->loadPlugins();
+    PluginManager::instance()->loadPlugins();
 
-    qWarning() << "Export formats:";
-    QList<Tiled::MapWriterInterface*> writers = pluginManager->interfaces<Tiled::MapWriterInterface>();
-    foreach (Tiled::MapWriterInterface *writer, writers) {
-        foreach (const QString &filter, writer->nameFilters())
-            qWarning() << " " << filter;
+    qWarning() << qPrintable(tr("Export formats:"));
+    auto formats = PluginManager::objects<MapFormat>();
+    for (MapFormat *format : formats) {
+        if (format->hasCapabilities(MapFormat::Write))
+            qWarning() << " " << format->nameFilter();
     }
 
     quit = true;
@@ -157,33 +153,28 @@ void CommandLineHandler::showExportFormats()
 
 int main(int argc, char *argv[])
 {
-    /*
-     * On X11, Tiled uses the 'raster' graphics system by default, because the
-     * X11 native graphics system has performance problems with drawing the
-     * tile grid.
-     */
-#ifdef Q_WS_X11
-    QApplication::setGraphicsSystem(QLatin1String("raster"));
-#endif
-
     TiledApplication a(argc, argv);
 
     a.setOrganizationDomain(QLatin1String("mapeditor.org"));
+#if defined(Q_OS_MAC) || defined(Q_OS_WIN)
     a.setApplicationName(QLatin1String("Tiled"));
+#else
+    a.setApplicationName(QLatin1String("tiled"));
+#endif
+    a.setApplicationDisplayName(QLatin1String("Tiled"));
+
 #ifdef BUILD_INFO_VERSION
     a.setApplicationVersion(QLatin1String(AS_STRING(BUILD_INFO_VERSION)));
 #else
-    a.setApplicationVersion(QLatin1String("0.12.3"));
+    a.setApplicationVersion(QLatin1String("0.14.2"));
 #endif
 
 #ifdef Q_OS_MAC
     a.setAttribute(Qt::AA_DontShowIconsInMenus);
 #endif
 
-#if QT_VERSION >= 0x050100
     // Enable support for highres images (added in Qt 5.1, but off by default)
     a.setAttribute(Qt::AA_UseHighDpiPixmaps);
-#endif
 
 #ifndef Q_OS_WIN
     QString baseName = QApplication::style()->objectName();
@@ -225,38 +216,53 @@ int main(int argc, char *argv[])
             return 1;
         }
         int index = 0;
-        const QString *filter = commandLine.filesToOpen().length() > 2 ? &commandLine.filesToOpen().at(index++) : 0;
+        const QString *filter = commandLine.filesToOpen().length() > 2 ? &commandLine.filesToOpen().at(index++) : nullptr;
         const QString &sourceFile = commandLine.filesToOpen().at(index++);
         const QString &targetFile = commandLine.filesToOpen().at(index++);
 
-        // Find the map writer interface for the target file
-        Tiled::MapWriterInterface *chosenWriter = 0;
-        QString suffix = QFileInfo(targetFile).completeSuffix();
-        QList<Tiled::MapWriterInterface*> writers = PluginManager::instance()->interfaces<Tiled::MapWriterInterface>();
-        foreach (Tiled::MapWriterInterface *writer, writers) {
-            if (filter) {
-                if (writer->nameFilters().contains(*filter, Qt::CaseInsensitive)) {
-                    chosenWriter = writer;
+        MapFormat *chosenFormat = nullptr;
+        auto formats = PluginManager::objects<MapFormat>();
+
+        if (filter) {
+            // Find the map format supporting the given filter
+            for (MapFormat *format : formats) {
+                if (!format->hasCapabilities(MapFormat::Write))
+                    continue;
+                if (format->nameFilter().compare(*filter, Qt::CaseInsensitive) == 0) {
+                    chosenFormat = format;
+                    break;
                 }
             }
-            else if (!writer->nameFilters().filter(suffix, Qt::CaseInsensitive).isEmpty()) {
-                if (chosenWriter) {
-                    qWarning() << qPrintable(QCoreApplication::translate("Command line",
-                                                                         "Non-unique file extension. Can't determine correct export format."));
-                    return 1;
+            if (!chosenFormat) {
+                qWarning() << qPrintable(QCoreApplication::translate("Command line",
+                                                                     "Format not recognized (see --export-formats)"));
+                return 1;
+            }
+        } else {
+            // Find the map format based on target file extension
+            QString suffix = QFileInfo(targetFile).completeSuffix();
+            for (MapFormat *format : formats) {
+                if (!format->hasCapabilities(MapFormat::Write))
+                    continue;
+                if (format->nameFilter().contains(suffix, Qt::CaseInsensitive)) {
+                    if (chosenFormat) {
+                        qWarning() << qPrintable(QCoreApplication::translate("Command line",
+                                                                             "Non-unique file extension. Can't determine correct export format."));
+                        return 1;
+                    }
+                    chosenFormat = format;
                 }
-                chosenWriter = writer;
+            }
+            if (!chosenFormat) {
+                qWarning() << qPrintable(QCoreApplication::translate("Command line",
+                                                                     "No exporter found for target file."));
+                return 1;
             }
-        }
-        if (!chosenWriter) {
-            qWarning() << qPrintable(QCoreApplication::translate("Command line",
-                                                                 "No exporter found for target file."));
-            return 1;
         }
 
         // Load the source file
-        Tiled::MapReader reader;
-        Tiled::Map *map = reader.readMap(sourceFile);
+        MapReader reader;
+        QScopedPointer<Map> map(reader.readMap(sourceFile));
         if (!map) {
             qWarning() << qPrintable(QCoreApplication::translate("Command line",
                                                                  "Failed to load source map."));
@@ -264,10 +270,7 @@ int main(int argc, char *argv[])
         }
 
         // Write out the file
-        bool success = chosenWriter->write(map, targetFile);
-
-        qDeleteAll(map->tilesets());
-        delete map;
+        bool success = chosenFormat->write(map.data(), targetFile);
 
         if (!success) {
             qWarning() << qPrintable(QCoreApplication::translate("Command line",
@@ -286,7 +289,7 @@ int main(int argc, char *argv[])
     if (!commandLine.filesToOpen().isEmpty()) {
         foreach (const QString &fileName, commandLine.filesToOpen())
             w.openFile(fileName);
-    } else {
+    } else if (Preferences::instance()->openLastFilesOnStartup()) {
         w.openLastFiles();
     }
 
