@@ -43,7 +43,6 @@ namespace Tiled {
 
 class TileLayer;
 class Terrain;
-class MapReaderInterface;
 
 namespace Internal {
 
@@ -59,13 +58,14 @@ class MapView;
 class MiniMapDock;
 class ObjectsDock;
 class PropertiesDock;
-class QuickStampManager;
 class StampBrush;
 class TerrainBrush;
 class TerrainDock;
 class TileAnimationEditor;
 class TileCollisionEditor;
 class TilesetDock;
+class TileStamp;
+class TileStampManager;
 class ToolManager;
 class Zoomable;
 
@@ -80,25 +80,24 @@ class MainWindow : public QMainWindow
     Q_OBJECT
 
 public:
-    MainWindow(QWidget *parent = 0, Qt::WindowFlags flags = 0);
+    MainWindow(QWidget *parent = nullptr, Qt::WindowFlags flags = nullptr);
     ~MainWindow();
 
     void commitData(QSessionManager &manager);
 
     /**
-     * Opens the given file. When opened succesfully, the file is added to the
+     * Opens the given file. When opened successfully, the file is added to the
      * list of recent files.
      *
-     * When a \a reader is given, it is used to open the file. Otherwise, a
-     * reader is searched using MapReaderInterface::supportsFile.
+     * When a \a format is given, it is used to open the file. Otherwise, a
+     * format is searched using MapFormat::supportsFile.
      *
-     * @return whether the file was succesfully opened
+     * @return whether the file was successfully opened
      */
-    bool openFile(const QString &fileName, MapReaderInterface *reader);
+    bool openFile(const QString &fileName, MapFormat *format);
 
     /**
      * Attempt to open the previously opened file.
-     * TODO: Opening last file should be optional
      */
     void openLastFiles();
 
@@ -106,21 +105,22 @@ public slots:
     bool openFile(const QString &fileName);
 
 protected:
-    void closeEvent(QCloseEvent *event);
-    void changeEvent(QEvent *event);
+    void closeEvent(QCloseEvent *event) override;
+    void changeEvent(QEvent *event) override;
 
-    void keyPressEvent(QKeyEvent *);
-    void keyReleaseEvent(QKeyEvent *);
+    void keyPressEvent(QKeyEvent *) override;
+    void keyReleaseEvent(QKeyEvent *) override;
 
-    void dragEnterEvent(QDragEnterEvent *);
-    void dropEvent(QDropEvent *);
+    void dragEnterEvent(QDragEnterEvent *) override;
+    void dropEvent(QDropEvent *) override;
 
-public slots:
+private slots:
     void newMap();
     void openFile();
     bool saveFile();
     bool saveFileAs();
-    void export_();
+    void saveAll();
+    void export_(); // 'export' is a reserved word
     void exportAs();
     void exportAsImage();
     void reload();
@@ -130,9 +130,10 @@ public slots:
     void cut();
     void copy();
     void paste();
-    void delete_(); // delete is a reserved word
+    void delete_(); // 'delete' is a reserved word
     void openPreferences();
 
+    void labelVisibilityActionTriggered(QAction *action);
     void zoomIn();
     void zoomOut();
     void zoomNormal();
@@ -148,6 +149,7 @@ public slots:
     void updateWindowTitle();
     void updateActions();
     void updateZoomLabel();
+    void openDocumentation();
     void becomePatron();
     void aboutTiled();
     void openRecentFile();
@@ -161,9 +163,8 @@ public slots:
     void flip(FlipDirection direction);
     void rotate(RotateDirection direction);
 
-    void setStampBrush(const TileLayer *tiles);
+    void setStamp(const TileStamp &stamp);
     void setTerrainBrush(const Terrain *terrain);
-    void saveQuickStamp(int index);
     void updateStatusInfoLabel(const QString &statusInfo);
 
     void mapDocumentChanged(MapDocument *mapDocument);
@@ -198,7 +199,7 @@ private:
     bool confirmAllSave();
 
     /**
-     * Save the current map to the given file name. When saved succesfully, the
+     * Save the current map to the given file name. When saved successfully, the
      * file is added to the list of recent files.
      * @return <code>true</code> on success, <code>false</code> on failure
      */
@@ -251,8 +252,8 @@ private:
 
     AutomappingManager *mAutomappingManager;
     DocumentManager *mDocumentManager;
-    QuickStampManager *mQuickStampManager;
     ToolManager *mToolManager;
+    TileStampManager *mTileStampManager;
 };
 
 } // namespace Internal
