@@ -26,6 +26,7 @@
 #include "addremovetileset.h"
 #include "changeproperties.h"
 #include "changeselectedarea.h"
+#include "containerhelpers.h"
 #include "flipmapobjects.h"
 #include "hexagonalrenderer.h"
 #include "imagelayer.h"
@@ -51,9 +52,7 @@
 #include "tile.h"
 #include "tilelayer.h"
 #include "tilesetmanager.h"
-#include "tileset.h"
-#include "tmxmapreader.h"
-#include "tmxmapwriter.h"
+#include "tmxmapformat.h"
 
 #include <QFileInfo>
 #include <QRect>
@@ -67,7 +66,7 @@ MapDocument::MapDocument(Map *map, const QString &fileName):
     mMap(map),
     mLayerModel(new LayerModel(this)),
     mCurrentObject(map),
-    mRenderer(0),
+    mRenderer(nullptr),
     mMapObjectModel(new MapObjectModel(this)),
     mTerrainModel(new TerrainModel(this, this)),
     mUndoStack(new QUndoStack(this))
@@ -78,11 +77,14 @@ MapDocument::MapDocument(Map *map, const QString &fileName):
     mLayerModel->setMapDocument(this);
 
     // Forward signals emitted from the layer model
-    connect(mLayerModel, SIGNAL(layerAdded(int)), SLOT(onLayerAdded(int)));
-    connect(mLayerModel, SIGNAL(layerAboutToBeRemoved(int)),
-            SLOT(onLayerAboutToBeRemoved(int)));
-    connect(mLayerModel, SIGNAL(layerRemoved(int)), SLOT(onLayerRemoved(int)));
-    connect(mLayerModel, SIGNAL(layerChanged(int)), SIGNAL(layerChanged(int)));
+    connect(mLayerModel, &LayerModel::layerAdded,
+            this, &MapDocument::onLayerAdded);
+    connect(mLayerModel, &LayerModel::layerAboutToBeRemoved,
+            this, &MapDocument::onLayerAboutToBeRemoved);
+    connect(mLayerModel, &LayerModel::layerRemoved,
+            this, &MapDocument::onLayerRemoved);
+    connect(mLayerModel, &LayerModel::layerChanged,
+            this, &MapDocument::layerChanged);
 
     // Forward signals emitted from the map object model
     mMapObjectModel->setMapDocument(this);
@@ -127,19 +129,15 @@ bool MapDocument::save(QString *error)
 
 bool MapDocument::save(const QString &fileName, QString *error)
 {
-    PluginManager *pm = PluginManager::instance();
+    MapFormat *mapFormat = mWriterFormat;
 
-    MapWriterInterface *chosenWriter = 0;
-    if (const Plugin *plugin = pm->pluginByFileName(mWriterPluginFileName))
-        chosenWriter = qobject_cast<MapWriterInterface*>(plugin->instance);
+    TmxMapFormat tmxMapFormat;
+    if (!mapFormat)
+        mapFormat = &tmxMapFormat;
 
-    TmxMapWriter mapWriter;
-    if (!chosenWriter)
-        chosenWriter = &mapWriter;
-
-    if (!chosenWriter->write(map(), fileName)) {
+    if (!mapFormat->write(map(), fileName)) {
         if (error)
-            *error = chosenWriter->errorString();
+            *error = mapFormat->errorString();
         return false;
     }
 
@@ -152,48 +150,44 @@ bool MapDocument::save(const QString &fileName, QString *error)
 }
 
 MapDocument *MapDocument::load(const QString &fileName,
-                               MapReaderInterface *mapReader,
+                               MapFormat *mapFormat,
                                QString *error)
 {
-    TmxMapReader tmxMapReader;
+    TmxMapFormat tmxMapFormat;
 
-    const PluginManager *pm = PluginManager::instance();
-    if (!mapReader && !tmxMapReader.supportsFile(fileName)) {
+    if (!mapFormat && !tmxMapFormat.supportsFile(fileName)) {
         // Try to find a plugin that implements support for this format
-        QList<MapReaderInterface*> readers =
-                pm->interfaces<MapReaderInterface>();
-
-        foreach (MapReaderInterface *reader, readers) {
-            if (reader->supportsFile(fileName)) {
-                mapReader = reader;
+        auto formats = PluginManager::objects<MapFormat>();
+        for (MapFormat *format : formats) {
+            if (format->supportsFile(fileName)) {
+                mapFormat = format;
                 break;
             }
         }
     }
 
-    // check if we can save in that format as well
-    QString readerPluginFileName;
-    QString writerPluginFileName;
-    if (mapReader) {
-        if (const Plugin *plugin = pm->plugin(mapReader)) {
-            readerPluginFileName = plugin->fileName;
-            if (qobject_cast<MapWriterInterface*>(plugin->instance))
-                writerPluginFileName = plugin->fileName;
-        }
+    Map *map = nullptr;
+    QString errorString;
+    if (mapFormat) {
+        map = mapFormat->read(fileName);
+        errorString = mapFormat->errorString();
     } else {
-        mapReader = &tmxMapReader;
+        map = tmxMapFormat.read(fileName);
+        errorString = tmxMapFormat.errorString();
     }
 
-    Map *map = mapReader->read(fileName);
     if (!map) {
         if (error)
-            *error = mapReader->errorString();
-        return 0;
+            *error = errorString;
+        return nullptr;
     }
 
     MapDocument *mapDocument = new MapDocument(map, fileName);
-    mapDocument->setReaderPluginFileName(readerPluginFileName);
-    mapDocument->setWriterPluginFileName(writerPluginFileName);
+    if (mapFormat) {
+        mapDocument->setReaderFormat(mapFormat);
+        if (mapFormat->hasCapabilities(MapFormat::Write))
+            mapDocument->setWriterFormat(mapFormat);
+    }
     return mapDocument;
 }
 
@@ -207,6 +201,36 @@ void MapDocument::setFileName(const QString &fileName)
     emit fileNameChanged(fileName, oldFileName);
 }
 
+MapFormat *MapDocument::readerFormat() const
+{
+    return mReaderFormat;
+}
+
+void MapDocument::setReaderFormat(MapFormat *format)
+{
+    mReaderFormat = format;
+}
+
+MapFormat *MapDocument::writerFormat() const
+{
+    return mWriterFormat;
+}
+
+void MapDocument::setWriterFormat(MapFormat *format)
+{
+    mWriterFormat = format;
+}
+
+MapFormat *MapDocument::exportFormat() const
+{
+    return mExportFormat;
+}
+
+void MapDocument::setExportFormat(MapFormat *format)
+{
+    mExportFormat = format;
+}
+
 /**
  * Returns the name with which to display this map. It is the file name without
  * its path, or 'untitled.tmx' when the map has no file name.
@@ -254,7 +278,7 @@ void MapDocument::setCurrentLayerIndex(int index)
 Layer *MapDocument::currentLayer() const
 {
     if (mCurrentLayerIndex == -1)
-        return 0;
+        return nullptr;
 
     return mMap->layerAt(mCurrentLayerIndex);
 }
@@ -413,7 +437,7 @@ void MapDocument::rotateSelectedObjects(RotateDirection direction)
  */
 void MapDocument::addLayer(Layer::TypeFlag layerType)
 {
-    Layer *layer = 0;
+    Layer *layer = nullptr;
     QString name;
 
     switch (layerType) {
@@ -532,13 +556,13 @@ void MapDocument::toggleOtherLayers(int index)
  * Adds a tileset to this map at the given \a index. Emits the appropriate
  * signal.
  */
-void MapDocument::insertTileset(int index, Tileset *tileset)
+void MapDocument::insertTileset(int index, const SharedTileset &tileset)
 {
     emit tilesetAboutToBeAdded(index);
     mMap->insertTileset(index, tileset);
     TilesetManager *tilesetManager = TilesetManager::instance();
     tilesetManager->addReference(tileset);
-    emit tilesetAdded(index, tileset);
+    emit tilesetAdded(index, tileset.data());
 }
 
 static bool isFromTileset(Object *object, Tileset *tileset)
@@ -568,13 +592,13 @@ void MapDocument::removeTilesetAt(int index)
 {
     emit tilesetAboutToBeRemoved(index);
 
-    Tileset *tileset = mMap->tilesets().at(index);
+    SharedTileset tileset = mMap->tilesets().at(index);
 
-    if (tileset == mCurrentObject || isFromTileset(mCurrentObject, tileset))
-        setCurrentObject(0);
+    if (tileset.data() == mCurrentObject || isFromTileset(mCurrentObject, tileset.data()))
+        setCurrentObject(nullptr);
 
     mMap->removeTilesetAt(index);
-    emit tilesetRemoved(tileset);
+    emit tilesetRemoved(tileset.data());
 
     TilesetManager *tilesetManager = TilesetManager::instance();
     tilesetManager->removeReference(tileset);
@@ -585,7 +609,7 @@ void MapDocument::moveTileset(int from, int to)
     if (from == to)
         return;
 
-    Tileset *tileset = mMap->tilesets().at(from);
+    SharedTileset tileset = mMap->tilesets().at(from);
     mMap->removeTilesetAt(from);
     mMap->insertTileset(to, tileset);
     emit tilesetMoved(from, to);
@@ -629,11 +653,11 @@ QList<Object*> MapDocument::currentObjects() const
     QList<Object*> objects;
     if (mCurrentObject) {
         if (mCurrentObject->typeId() == Object::MapObjectType && !mSelectedObjects.isEmpty()) {
-            foreach (MapObject *mapObj, mSelectedObjects) {
+            for (MapObject *mapObj : mSelectedObjects) {
                 objects.append(mapObj);
             }
         } else if (mCurrentObject->typeId() == Object::TileType && !mSelectedTiles.isEmpty()) {
-            foreach (Tile *tile, mSelectedTiles) {
+            for (Tile *tile : mSelectedTiles) {
                 objects.append(tile);
             }
         } else {
@@ -650,19 +674,22 @@ QList<Object*> MapDocument::currentObjects() const
  * To reach the aim, all similar tilesets will be replaced by the version
  * in the current map document and all missing tilesets will be added to
  * the current map document.
+ *
+ * \warning This method assumes that the tilesets in \a map are managed by
+ *          the TilesetManager!
  */
 void MapDocument::unifyTilesets(Map *map)
 {
     QList<QUndoCommand*> undoCommands;
-    QList<Tileset*> existingTilesets = mMap->tilesets();
+    const QVector<SharedTileset> &existingTilesets = mMap->tilesets();
     TilesetManager *tilesetManager = TilesetManager::instance();
 
     // Add tilesets that are not yet part of this map
-    foreach (Tileset *tileset, map->tilesets()) {
+    foreach (const SharedTileset &tileset, map->tilesets()) {
         if (existingTilesets.contains(tileset))
             continue;
 
-        Tileset *replacement = tileset->findSimilarTileset(existingTilesets);
+        SharedTileset replacement = tileset->findSimilarTileset(existingTilesets);
         if (!replacement) {
             undoCommands.append(new AddTileset(this, tileset));
             continue;
@@ -693,6 +720,41 @@ void MapDocument::unifyTilesets(Map *map)
     }
 }
 
+/**
+ * Replaces tilesets in \a map by similar tilesets in this map when possible,
+ * and adds tilesets to \a missingTilesets whenever there is a tileset without
+ * replacement in this map.
+ *
+ * \warning This method assumes that the tilesets in \a map are managed by
+ *          the TilesetManager!
+ */
+void MapDocument::unifyTilesets(Map *map, QVector<SharedTileset> &missingTilesets)
+{
+    const QVector<SharedTileset> &existingTilesets = mMap->tilesets();
+    TilesetManager *tilesetManager = TilesetManager::instance();
+
+    foreach (const SharedTileset &tileset, map->tilesets()) {
+        // tileset already added
+        if (existingTilesets.contains(tileset))
+            continue;
+
+        SharedTileset replacement = tileset->findSimilarTileset(existingTilesets);
+
+        // tileset not present and no replacement tileset found
+        if (!replacement) {
+            if (!missingTilesets.contains(tileset))
+                missingTilesets.append(tileset);
+            continue;
+        }
+
+        // replacement tileset found, change given map
+        map->replaceTileset(tileset, replacement);
+
+        tilesetManager->addReference(replacement);
+        tilesetManager->removeReference(tileset);
+    }
+}
+
 /**
  * Emits the tileset changed signal. This signal is currently used when adding
  * or removing tiles from a tileset.
@@ -701,7 +763,7 @@ void MapDocument::unifyTilesets(Map *map)
  */
 void MapDocument::emitTilesetChanged(Tileset *tileset)
 {
-    Q_ASSERT(mMap->tilesets().contains(tileset));
+    Q_ASSERT(contains(mMap->tilesets(), tileset));
     emit tilesetChanged(tileset);
 }
 
@@ -770,7 +832,7 @@ void MapDocument::onLayerAboutToBeRemoved(int index)
 {
     Layer *layer = mMap->layerAt(index);
     if (layer == mCurrentObject)
-        setCurrentObject(0);
+        setCurrentObject(nullptr);
 
     // Deselect any objects on this layer when necessary
     if (ObjectGroup *og = dynamic_cast<ObjectGroup*>(layer))
@@ -796,7 +858,7 @@ void MapDocument::onLayerRemoved(int index)
 void MapDocument::onTerrainRemoved(Terrain *terrain)
 {
     if (terrain == mCurrentObject)
-        setCurrentObject(0);
+        setCurrentObject(nullptr);
 }
 
 void MapDocument::deselectObjects(const QList<MapObject *> &objects)
@@ -804,7 +866,7 @@ void MapDocument::deselectObjects(const QList<MapObject *> &objects)
     // Unset the current object when it was part of this list of objects
     if (mCurrentObject && mCurrentObject->typeId() == Object::MapObjectType)
         if (objects.contains(static_cast<MapObject*>(mCurrentObject)))
-            setCurrentObject(0);
+            setCurrentObject(nullptr);
 
     int removedCount = 0;
     foreach (MapObject *object, objects)
