@@ -45,6 +45,7 @@ class MapDocument;
 class MapObjectItem;
 class MapScene;
 class ObjectGroupItem;
+class ObjectSelectionItem;
 
 /**
  * A graphics scene that represents the contents of a map.
@@ -116,19 +117,19 @@ protected:
     /**
      * QGraphicsScene::drawForeground override that draws the tile grid.
      */
-    void drawForeground(QPainter *painter, const QRectF &rect);
+    void drawForeground(QPainter *painter, const QRectF &rect) override;
 
     /**
      * Override for handling enter and leave events.
      */
-    bool event(QEvent *event);
+    bool event(QEvent *event) override;
 
-    void keyPressEvent(QKeyEvent *event);
-    void mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent);
-    void mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent);
-    void mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent);
+    void keyPressEvent(QKeyEvent *event) override;
+    void mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent) override;
+    void mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent) override;
+    void mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent) override;
 
-    void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
+    void dragEnterEvent(QGraphicsSceneDragDropEvent *event) override;
 
 private slots:
     void setGridVisible(bool visible);
@@ -148,7 +149,7 @@ private slots:
     /**
      * Repaints the specified region. The region is in tile coordinates.
      */
-    void repaintRegion(const QRegion &region);
+    void repaintRegion(const QRegion &region, Layer *layer);
 
     void currentLayerIndexChanged();
 
@@ -176,9 +177,10 @@ private slots:
 private:
     QGraphicsItem *createLayerItem(Layer *layer);
 
+    void updateSceneRect();
     void updateCurrentLayerHighlight();
 
-    bool eventFilter(QObject *object, QEvent *event);
+    bool eventFilter(QObject *object, QEvent *event) override;
 
     MapDocument *mMapDocument;
     AbstractTool *mSelectedTool;
@@ -193,6 +195,7 @@ private:
     QVector<QGraphicsItem*> mLayerItems;
     QGraphicsRectItem *mDarkRectangle;
     QColor mDefaultBackgroundColor;
+    ObjectSelectionItem *mObjectSelectionItem;
 
     typedef QMap<MapObject*, MapObjectItem*> ObjectItems;
     ObjectItems mObjectItems;
