@@ -106,7 +106,7 @@ void MapView::setUseOpenGL(bool useOpenGL)
         }
     } else {
         if (qobject_cast<QGLWidget*>(viewport()))
-            setViewport(0);
+            setViewport(nullptr);
     }
 
     QWidget *v = viewport();
@@ -195,7 +195,7 @@ void MapView::wheelEvent(QWheelEvent *event)
  */
 void MapView::mousePressEvent(QMouseEvent *event)
 {
-    if (event->button() == Qt::MidButton) {
+    if (event->button() == Qt::MidButton && isActiveWindow()) {
         setHandScrolling(true);
         return;
     }
