@@ -139,7 +139,7 @@ static QPainterPath createResizeArrow(bool straight)
 class Handle : public QGraphicsItem
 {
 public:
-    Handle(QGraphicsItem *parent = 0)
+    Handle(QGraphicsItem *parent = nullptr)
         : QGraphicsItem(parent)
         , mUnderMouse(false)
     {
@@ -150,9 +150,9 @@ public:
     }
 
 protected:
-    void hoverEnterEvent(QGraphicsSceneHoverEvent *) { mUnderMouse = true; update(); }
-    void hoverLeaveEvent(QGraphicsSceneHoverEvent *) { mUnderMouse = false; update(); }
-    QVariant itemChange(GraphicsItemChange change, const QVariant &value);
+    void hoverEnterEvent(QGraphicsSceneHoverEvent *) override { mUnderMouse = true; update(); }
+    void hoverLeaveEvent(QGraphicsSceneHoverEvent *) override { mUnderMouse = false; update(); }
+    QVariant itemChange(GraphicsItemChange change, const QVariant &value) override;
 
     bool mUnderMouse;
 };
@@ -172,15 +172,15 @@ QVariant Handle::itemChange(GraphicsItemChange change, const QVariant &value)
 class OriginIndicator : public Handle
 {
 public:
-    OriginIndicator(QGraphicsItem *parent = 0)
+    OriginIndicator(QGraphicsItem *parent = nullptr)
         : Handle(parent)
     {
         setFlag(QGraphicsItem::ItemIsMovable);
         setZValue(10000 + 1);
     }
 
-    QRectF boundingRect() const { return QRectF(-9, -9, 18, 18); }
-    void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *);
+    QRectF boundingRect() const override { return QRectF(-9, -9, 18, 18); }
+    void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *) override;
 };
 
 void OriginIndicator::paint(QPainter *painter,
@@ -205,7 +205,7 @@ void OriginIndicator::paint(QPainter *painter,
 class RotateHandle : public Handle
 {
 public:
-    RotateHandle(AnchorPosition corner, QGraphicsItem *parent = 0)
+    RotateHandle(AnchorPosition corner, QGraphicsItem *parent = nullptr)
         : Handle(parent)
         , mArrow(createRotateArrow())
     {
@@ -223,8 +223,8 @@ public:
         mArrow = transform.map(mArrow);
     }
 
-    QRectF boundingRect() const { return mArrow.boundingRect().adjusted(-1, -1, 1, 1); }
-    void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *);
+    QRectF boundingRect() const override { return mArrow.boundingRect().adjusted(-1, -1, 1, 1); }
+    void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *) override;
 
 private:
     QPainterPath mArrow;
@@ -249,7 +249,7 @@ void RotateHandle::paint(QPainter *painter, const QStyleOptionGraphicsItem *,
 class ResizeHandle : public Handle
 {
 public:
-    ResizeHandle(AnchorPosition anchorPosition, QGraphicsItem *parent = 0)
+    ResizeHandle(AnchorPosition anchorPosition, QGraphicsItem *parent = nullptr)
         : Handle(parent)
         , mAnchorPosition(anchorPosition)
         , mResizingLimitHorizontal(false)
@@ -284,8 +284,8 @@ public:
     bool resizingLimitHorizontal() const { return mResizingLimitHorizontal; }
     bool resizingLimitVertical() const { return mResizingLimitVertical; }
     
-    QRectF boundingRect() const { return mArrow.boundingRect().adjusted(-1, -1, 1, 1); }
-    void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *);
+    QRectF boundingRect() const override { return mArrow.boundingRect().adjusted(-1, -1, 1, 1); }
+    void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *) override;
 
 private:
     AnchorPosition mAnchorPosition;
@@ -320,9 +320,10 @@ ObjectSelectionTool::ObjectSelectionTool(QObject *parent)
     , mSelectionRectangle(new SelectionRectangle)
     , mOriginIndicator(new OriginIndicator)
     , mMousePressed(false)
-    , mClickedObjectItem(0)
-    , mClickedRotateHandle(0)
-    , mClickedResizeHandle(0)
+    , mHoveredObjectItem(nullptr)
+    , mClickedObjectItem(nullptr)
+    , mClickedRotateHandle(nullptr)
+    , mClickedResizeHandle(nullptr)
     , mResizingLimitHorizontal(false)
     , mResizingLimitVertical(false)
     , mMode(Resize)
@@ -446,12 +447,35 @@ void ObjectSelectionTool::mouseMoved(const QPointF &pos,
 {
     AbstractObjectTool::mouseMoved(pos, modifiers);
 
+    // Update the hovered item (for mouse cursor)
+    {
+        RotateHandle *hoveredRotateHandle = nullptr;
+        ResizeHandle *hoveredResizeHandle = nullptr;
+        MapObjectItem *hoveredObjectItem = nullptr;
+
+        if (QGraphicsView *view = mapScene()->views().first()) {
+            QGraphicsItem *hoveredItem = mapScene()->itemAt(pos,
+                                                            view->transform());
+
+            hoveredRotateHandle = dynamic_cast<RotateHandle*>(hoveredItem);
+            hoveredResizeHandle = dynamic_cast<ResizeHandle*>(hoveredItem);
+        }
+
+        if (!hoveredRotateHandle && !hoveredResizeHandle)
+            hoveredObjectItem = topMostObjectItemAt(pos);
+
+        mHoveredObjectItem = hoveredObjectItem;
+    }
+
     if (mAction == NoAction && mMousePressed) {
         QPoint screenPos = QCursor::pos();
         const int dragDistance = (mScreenStart - screenPos).manhattanLength();
         if (dragDistance >= QApplication::startDragDistance()) {
-            // Holding shift makes sure we'll start a selection operation
-            if ((mClickedObjectItem || modifiers & Qt::AltModifier) &&
+            const bool hasSelection = !mapScene()->selectedObjectItems().isEmpty();
+
+            // Holding Alt forces moving current selection
+            // Holding Shift forces selection rectangle
+            if ((mClickedObjectItem || ((modifiers & Qt::AltModifier) && hasSelection)) &&
                     !(modifiers & Qt::ShiftModifier)) {
                 startMoving(modifiers);
             } else if (mClickedRotateHandle) {
@@ -480,13 +504,15 @@ void ObjectSelectionTool::mouseMoved(const QPointF &pos,
     case NoAction:
         break;
     }
+
+    refreshCursor();
 }
 
 static QGraphicsView *findView(QGraphicsSceneEvent *event)
 {
     if (QWidget *viewport = event->widget())
         return qobject_cast<QGraphicsView*>(viewport->parent());
-    return 0;
+    return nullptr;
 }
 
 void ObjectSelectionTool::mousePressed(QGraphicsSceneMouseEvent *event)
@@ -500,8 +526,8 @@ void ObjectSelectionTool::mousePressed(QGraphicsSceneMouseEvent *event)
         mStart = event->scenePos();
         mScreenStart = event->screenPos();
 
-        RotateHandle *clickedRotateHandle = 0;
-        ResizeHandle *clickedResizeHandle = 0;
+        RotateHandle *clickedRotateHandle = nullptr;
+        ResizeHandle *clickedResizeHandle = nullptr;
 
         if (QGraphicsView *view = findView(event)) {
             QGraphicsItem *clickedItem = mapScene()->itemAt(event->scenePos(),
@@ -574,14 +600,17 @@ void ObjectSelectionTool::mouseReleased(QGraphicsSceneMouseEvent *event)
     }
 
     mMousePressed = false;
-    mClickedObjectItem = 0;
-    mClickedRotateHandle = 0;
-    mClickedResizeHandle = 0;
+    mClickedObjectItem = nullptr;
+    mClickedRotateHandle = nullptr;
+    mClickedResizeHandle = nullptr;
+
+    refreshCursor();
 }
 
 void ObjectSelectionTool::modifiersChanged(Qt::KeyboardModifiers modifiers)
 {
     mModifiers = modifiers;
+    refreshCursor();
 }
 
 void ObjectSelectionTool::languageChanged()
@@ -590,7 +619,7 @@ void ObjectSelectionTool::languageChanged()
     setShortcut(QKeySequence(tr("S")));
 }
 
-static QPointF alignmentOffset(QRectF &r, Alignment alignment)
+static QPointF alignmentOffset(const QRectF &r, Alignment alignment)
 {
     switch (alignment) {
     case TopLeft:       break;
@@ -709,11 +738,18 @@ static QTransform rotateAt(const QPointF &position, qreal rotation)
 
 static QTransform objectTransform(MapObject *object, MapRenderer *renderer)
 {
+    QTransform transform;
+
     if (object->rotation() != 0) {
         const QPointF pos = renderer->pixelToScreenCoords(object->position());
-        return rotateAt(pos, object->rotation());
+        transform = rotateAt(pos, object->rotation());
     }
-    return QTransform();
+
+    QPointF offset = object->objectGroup()->offset();
+    if (!offset.isNull())
+        transform *= QTransform::fromTranslate(offset.x(), offset.y());
+
+    return transform;
 }
 
 void ObjectSelectionTool::updateHandles()
@@ -727,7 +763,7 @@ void ObjectSelectionTool::updateHandles()
     if (showHandles) {
         MapRenderer *renderer = mapDocument()->renderer();
         QRectF boundingRect = objectBounds(objects.first(), renderer,
-                                            objectTransform(objects.first(), renderer));
+                                           objectTransform(objects.first(), renderer));
 
         for (int i = 1; i < objects.size(); ++i) {
             MapObject *object = objects.at(i);
@@ -914,8 +950,8 @@ void ObjectSelectionTool::startMoving(Qt::KeyboardModifiers modifiers)
 void ObjectSelectionTool::updateMovingItems(const QPointF &pos,
                                             Qt::KeyboardModifiers modifiers)
 {
-    MapRenderer *renderer = mapDocument()->renderer();
-    QPointF diff = snapToGrid(pos - mStart, modifiers);
+    const MapRenderer *renderer = mapDocument()->renderer();
+    const QPointF diff = snapToGrid(pos - mStart, modifiers);
 
     foreach (const MovingObject &object, mMovingObjects) {
         const QPointF newPixelPos = object.oldItemPosition + diff;
@@ -975,17 +1011,19 @@ void ObjectSelectionTool::updateRotatingItems(const QPointF &pos,
         angleDiff = std::floor((angleDiff + snap / 2) / snap) * snap;
 
     foreach (const MovingObject &object, mMovingObjects) {
-        const QPointF oldRelPos = object.oldItemPosition - mOrigin;
+        MapObject *mapObject = object.item->mapObject();
+        const QPointF offset = mapObject->objectGroup()->offset();
+
+        const QPointF oldRelPos = object.oldItemPosition + offset - mOrigin;
         const qreal sn = std::sin(angleDiff);
         const qreal cs = std::cos(angleDiff);
         const QPointF newRelPos(oldRelPos.x() * cs - oldRelPos.y() * sn,
                                 oldRelPos.x() * sn + oldRelPos.y() * cs);
-        const QPointF newPixelPos = mOrigin + newRelPos;
+        const QPointF newPixelPos = mOrigin + newRelPos - offset;
         const QPointF newPos = renderer->screenToPixelCoords(newPixelPos);
 
         const qreal newRotation = object.oldRotation + angleDiff * 180 / M_PI;
 
-        MapObject *mapObject = object.item->mapObject();
         mapObject->setPosition(newPos);
         mapObject->setRotation(newRotation);
     }
@@ -1076,18 +1114,22 @@ void ObjectSelectionTool::updateResizingItems(const QPointF &pos,
                      qMax((qreal)0.01, diff.y() / startDiff.y()));
     }
 
+    if (!std::isfinite(scale))
+        scale = 1;
+
     foreach (const MovingObject &object, mMovingObjects) {
-        const QPointF oldRelPos = object.oldItemPosition - resizingOrigin;
+        MapObject *mapObject = object.item->mapObject();
+        const QPointF offset = mapObject->objectGroup()->offset();
+
+        const QPointF oldRelPos = object.oldItemPosition + offset - resizingOrigin;
         const QPointF scaledRelPos(oldRelPos.x() * scale,
                                    oldRelPos.y() * scale);
-        const QPointF newScreenPos = resizingOrigin + scaledRelPos;
+        const QPointF newScreenPos = resizingOrigin + scaledRelPos - offset;
         const QPointF newPos = renderer->screenToPixelCoords(newScreenPos);
         const QSizeF origSize = object.oldSize;
         const QSizeF newSize(origSize.width() * scale,
                              origSize.height() * scale);
 
-        MapObject *mapObject = object.item->mapObject();
-
         if (mapObject->polygon().isEmpty() == false) {
             // For polygons, we have to scale in object space.
             qreal rotation = object.item->rotation() * M_PI / -180;
@@ -1123,15 +1165,21 @@ void ObjectSelectionTool::updateResizingSingleItem(const QPointF &resizingOrigin
     const MovingObject &object = mMovingObjects.first();
     MapObject *mapObject = object.item->mapObject();
 
+    /* The resizingOrigin, screenPos and mStart are affected by the ObjectGroup
+     * offset. We will un-apply it to these variables since the resize for
+     * single items happens in local coordinate space.
+     */
+    QPointF offset = mapObject->objectGroup()->offset();
+
     /* These transformations undo and redo the object rotation, which is always
      * applied in screen space.
      */
     QTransform unrotate = rotateAt(object.oldItemPosition, -object.oldRotation);
     QTransform rotate = rotateAt(object.oldItemPosition, object.oldRotation);
 
-    QPointF origin = resizingOrigin * unrotate;
-    QPointF pos = screenPos * unrotate;
-    QPointF start = mStart * unrotate;
+    QPointF origin = (resizingOrigin - offset) * unrotate;
+    QPointF pos = (screenPos - offset) * unrotate;
+    QPointF start = (mStart - offset) * unrotate;
     QPointF oldPos = object.oldItemPosition;
 
     /* In order for the resizing to work somewhat sanely in isometric mode,
@@ -1205,6 +1253,11 @@ void ObjectSelectionTool::updateResizingSingleItem(const QPointF &resizingOrigin
         QSizeF scalingFactor(qMax((qreal)0.01, relPos.x() / startDiff.x()),
                              qMax((qreal)0.01, relPos.y() / startDiff.y()));
 
+        if (!std::isfinite(scalingFactor.width()))
+            scalingFactor.setWidth(1);
+        if (!std::isfinite(scalingFactor.height()))
+            scalingFactor.setHeight(1);
+
         if (mResizingLimitHorizontal) {
             scalingFactor.setWidth(preserveAspect ? scalingFactor.height() : 1);
         } else if (mResizingLimitVertical) {
@@ -1295,8 +1348,34 @@ void ObjectSelectionTool::saveSelectionState()
     }
 }
 
-const QPointF ObjectSelectionTool::snapToGrid(const QPointF &diff,
-                                              Qt::KeyboardModifiers modifiers)
+void ObjectSelectionTool::refreshCursor()
+{
+    Qt::CursorShape cursorShape = Qt::ArrowCursor;
+
+    switch (mAction) {
+    case NoAction: {
+        const bool hasSelection = !mapScene()->selectedObjectItems().isEmpty();
+
+        if ((mHoveredObjectItem || ((mModifiers & Qt::AltModifier) && hasSelection)) &&
+                !(mModifiers & Qt::ShiftModifier)) {
+            cursorShape = Qt::SizeAllCursor;
+        }
+
+        break;
+    }
+    case Moving:
+        cursorShape = Qt::SizeAllCursor;
+        break;
+    default:
+        break;
+    }
+
+    if (cursor().shape() != cursorShape)
+        setCursor(cursorShape);
+}
+
+QPointF ObjectSelectionTool::snapToGrid(const QPointF &diff,
+                                        Qt::KeyboardModifiers modifiers)
 {
     MapRenderer *renderer = mapDocument()->renderer();
     SnapHelper snapHelper(renderer, modifiers);
