@@ -42,21 +42,21 @@ class ObjectSelectionTool : public AbstractObjectTool
     Q_OBJECT
 
 public:
-    explicit ObjectSelectionTool(QObject *parent = 0);
+    explicit ObjectSelectionTool(QObject *parent = nullptr);
     ~ObjectSelectionTool();
 
-    void activate(MapScene *scene);
-    void deactivate(MapScene *scene);
+    void activate(MapScene *scene) override;
+    void deactivate(MapScene *scene) override;
 
-    void keyPressed(QKeyEvent *);
-    void mouseEntered();
+    void keyPressed(QKeyEvent *) override;
+    void mouseEntered() override;
     void mouseMoved(const QPointF &pos,
-                    Qt::KeyboardModifiers modifiers);
-    void mousePressed(QGraphicsSceneMouseEvent *event);
-    void mouseReleased(QGraphicsSceneMouseEvent *event);
-    void modifiersChanged(Qt::KeyboardModifiers modifiers);
+                    Qt::KeyboardModifiers modifiers) override;
+    void mousePressed(QGraphicsSceneMouseEvent *event) override;
+    void mouseReleased(QGraphicsSceneMouseEvent *event) override;
+    void modifiersChanged(Qt::KeyboardModifiers modifiers) override;
 
-    void languageChanged();
+    void languageChanged() override;
 
 private slots:
     void updateHandles();
@@ -104,8 +104,10 @@ private:
     void setMode(Mode mode);
     void saveSelectionState();
 
-    const QPointF snapToGrid(const QPointF &pos,
-                             Qt::KeyboardModifiers modifiers);
+    void refreshCursor();
+
+    QPointF snapToGrid(const QPointF &pos,
+                       Qt::KeyboardModifiers modifiers);
 
     QList<MapObject*> changingObjects() const;
 
@@ -125,6 +127,7 @@ private:
     RotateHandle *mRotateHandles[4];
     ResizeHandle *mResizeHandles[8];
     bool mMousePressed;
+    MapObjectItem *mHoveredObjectItem;
     MapObjectItem *mClickedObjectItem;
     RotateHandle *mClickedRotateHandle;
     ResizeHandle *mClickedResizeHandle;
