@@ -25,11 +25,7 @@
 #include "mapdocument.h"
 #include "tilesetmanager.h"
 
-#if QT_VERSION >= 0x050000
 #include <QStandardPaths>
-#else
-#include <QDesktopServices>
-#endif
 
 #include <QFileInfo>
 #include <QSettings>
@@ -37,7 +33,7 @@
 using namespace Tiled;
 using namespace Tiled::Internal;
 
-Preferences *Preferences::mInstance = 0;
+Preferences *Preferences::mInstance;
 
 Preferences *Preferences::instance()
 {
@@ -49,7 +45,7 @@ Preferences *Preferences::instance()
 void Preferences::deleteInstance()
 {
     delete mInstance;
-    mInstance = 0;
+    mInstance = nullptr;
 }
 
 Preferences::Preferences()
@@ -57,14 +53,13 @@ Preferences::Preferences()
 {
     // Retrieve storage settings
     mSettings->beginGroup(QLatin1String("Storage"));
-    mLayerDataFormat = (Map::LayerDataFormat)
-            mSettings->value(QLatin1String("LayerDataFormat"),
-                             Map::Base64Zlib).toInt();
-    mMapRenderOrder = (Map::RenderOrder)
-            mSettings->value(QLatin1String("MapRenderOrder"),
-                             Map::RightDown).toInt();
+    mLayerDataFormat = static_cast<Map::LayerDataFormat>
+            (intValue("LayerDataFormat", Map::Base64Zlib));
+    mMapRenderOrder = static_cast<Map::RenderOrder>
+            (intValue("MapRenderOrder", Map::RightDown));
     mDtdEnabled = boolValue("DtdEnabled");
     mReloadTilesetsOnChange = boolValue("ReloadTilesets", true);
+    mStampsDirectory = stringValue("StampsDirectory");
     mSettings->endGroup();
 
     // Retrieve interface settings
@@ -81,6 +76,8 @@ Preferences::Preferences()
     mShowTilesetGrid = boolValue("ShowTilesetGrid", true);
     mLanguage = stringValue("Language");
     mUseOpenGL = boolValue("OpenGL");
+    mObjectLabelVisibility = static_cast<ObjectLabelVisiblity>
+            (intValue("ObjectLabelVisibility", AllObjectLabels));
     mSettings->endGroup();
 
     // Retrieve defined object types
@@ -118,12 +115,27 @@ Preferences::Preferences()
     }
     mSettings->setValue(QLatin1String("RunCount"), mRunCount);
     mSettings->endGroup();
+
+    // Retrieve startup settings
+    mSettings->beginGroup(QLatin1String("Startup"));
+    mOpenLastFilesOnStartup = boolValue("OpenLastFiles", true);
+    mSettings->endGroup();
 }
 
 Preferences::~Preferences()
 {
 }
 
+void Preferences::setObjectLabelVisibility(ObjectLabelVisiblity visibility)
+{
+    if (mObjectLabelVisibility == visibility)
+        return;
+
+    mObjectLabelVisibility = visibility;
+    mSettings->setValue(QLatin1String("Interface/ObjectLabelVisibility"), visibility);
+    emit objectLabelVisibilityChanged(visibility);
+}
+
 void Preferences::setShowGrid(bool showGrid)
 {
     if (mShowGrid == showGrid)
@@ -383,13 +395,8 @@ QString Preferences::lastPath(FileType fileType) const
     }
 
     if (path.isEmpty()) {
-#if QT_VERSION >= 0x050000
         path = QStandardPaths::writableLocation(
                     QStandardPaths::DocumentsLocation);
-#else
-        path = QDesktopServices::storageLocation(
-                    QDesktopServices::DocumentsLocation);
-#endif
     }
 
     return path;
@@ -435,6 +442,15 @@ void Preferences::setPatron(bool isPatron)
     emit isPatronChanged();
 }
 
+void Preferences::setOpenLastFilesOnStartup(bool open)
+{
+    if (mOpenLastFilesOnStartup == open)
+    	return;
+
+    mOpenLastFilesOnStartup = open;
+    mSettings->setValue(QLatin1String("Startup/OpenLastFiles"), open);
+}
+
 bool Preferences::boolValue(const char *key, bool defaultValue) const
 {
     return mSettings->value(QLatin1String(key), defaultValue).toBool();
@@ -464,3 +480,27 @@ qreal Preferences::realValue(const char *key, qreal defaultValue) const
 {
     return mSettings->value(QLatin1String(key), defaultValue).toReal();
 }
+
+QString Preferences::stampsDirectory() const
+{
+    if (mStampsDirectory.isEmpty()) {
+#if QT_VERSION >= 0x050400
+        QString appData = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
+#else
+        QString appData = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
+#endif
+        return appData + QLatin1String("/stamps");
+    }
+    return mStampsDirectory;
+}
+
+void Preferences::setStampsDirectory(const QString &stampsDirectory)
+{
+    if (mStampsDirectory == stampsDirectory)
+        return;
+
+    mStampsDirectory = stampsDirectory;
+    mSettings->setValue(QLatin1String("Storage/StampsDirectory"), stampsDirectory);
+
+    emit stampsDirectoryChanged(stampsDirectory);
+}
