@@ -57,6 +57,15 @@ public:
     bool highlightCurrentLayer() const { return mHighlightCurrentLayer; }
     bool showTilesetGrid() const { return mShowTilesetGrid; }
 
+    enum ObjectLabelVisiblity {
+        NoObjectLabels,
+        SelectedObjectLabels,
+        AllObjectLabels
+    };
+
+    ObjectLabelVisiblity objectLabelVisibility() const;
+    void setObjectLabelVisibility(ObjectLabelVisiblity visiblity);
+
     Map::LayerDataFormat layerDataFormat() const;
     void setLayerDataFormat(Map::LayerDataFormat layerDataFormat);
 
@@ -94,12 +103,18 @@ public:
     QString mapsDirectory() const;
     void setMapsDirectory(const QString &path);
 
+    QString stampsDirectory() const;
+    void setStampsDirectory(const QString &stampsDirectory);
+
     QDate firstRun() const;
     int runCount() const;
 
     bool isPatron() const;
     void setPatron(bool isPatron);
 
+    bool openLastFilesOnStartup() const;
+    void setOpenLastFilesOnStartup(bool load);
+
     /**
      * Provides access to the QSettings instance to allow storing/retrieving
      * arbitrary values. The naming style for groups and keys is CamelCase.
@@ -129,12 +144,14 @@ signals:
     void objectLineWidthChanged(qreal lineWidth);
     void highlightCurrentLayerChanged(bool highlight);
     void showTilesetGridChanged(bool showTilesetGrid);
+    void objectLabelVisibilityChanged(ObjectLabelVisiblity);
 
     void useOpenGLChanged(bool useOpenGL);
 
     void objectTypesChanged();
 
     void mapsDirectoryChanged();
+    void stampsDirectoryChanged(const QString &stampsDirectory);
 
     void isPatronChanged();
 
@@ -160,6 +177,8 @@ private:
     qreal mObjectLineWidth;
     bool mHighlightCurrentLayer;
     bool mShowTilesetGrid;
+    bool mOpenLastFilesOnStartup;
+    ObjectLabelVisiblity mObjectLabelVisibility;
 
     Map::LayerDataFormat mLayerDataFormat;
     Map::RenderOrder mMapRenderOrder;
@@ -172,6 +191,7 @@ private:
     bool mAutoMapDrawing;
 
     QString mMapsDirectory;
+    QString mStampsDirectory;
 
     QDate mFirstRun;
     int mRunCount;
@@ -181,6 +201,11 @@ private:
 };
 
 
+inline Preferences::ObjectLabelVisiblity Preferences::objectLabelVisibility() const
+{
+    return mObjectLabelVisibility;
+}
+
 inline QDate Preferences::firstRun() const
 {
     return mFirstRun;
@@ -196,6 +221,11 @@ inline bool Preferences::isPatron() const
     return mIsPatron;
 }
 
+inline bool Preferences::openLastFilesOnStartup() const
+{
+    return mOpenLastFilesOnStartup;
+}
+
 } // namespace Internal
 } // namespace Tiled
 
