@@ -45,15 +45,15 @@ namespace Internal {
 class ColorDelegate : public QStyledItemDelegate
 {
 public:
-    ColorDelegate(QObject *parent = 0)
+    ColorDelegate(QObject *parent = nullptr)
         : QStyledItemDelegate(parent)
     { }
 
     void paint(QPainter *painter, const QStyleOptionViewItem &option,
-               const QModelIndex &index) const;
+               const QModelIndex &index) const override;
 
     QSize sizeHint(const QStyleOptionViewItem &,
-                   const QModelIndex &) const;
+                   const QModelIndex &) const override;
 };
 
 } // namespace Internal
@@ -125,11 +125,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) :
     mUi->objectTypesTable->setItemDelegateForColumn(1, new ColorDelegate(this));
 
     QHeaderView *horizontalHeader = mUi->objectTypesTable->horizontalHeader();
-#if QT_VERSION >= 0x050000
     horizontalHeader->setSectionResizeMode(QHeaderView::Stretch);
-#else
-    horizontalHeader->setResizeMode(QHeaderView::Stretch);
-#endif
 
     Utils::setThemeIcon(mUi->addObjectTypeButton, "add");
     Utils::setThemeIcon(mUi->removeObjectTypeButton, "remove");
@@ -167,6 +163,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) :
 
     connect(mUi->autoMapWhileDrawing, SIGNAL(toggled(bool)),
             SLOT(useAutomappingDrawingToggled(bool)));
+    connect(mUi->openLastFiles, SIGNAL(toggled(bool)), SLOT(openLastFilesToggled(bool)));
 }
 
 PreferencesDialog::~PreferencesDialog()
@@ -304,6 +301,7 @@ void PreferencesDialog::fromPreferences()
     const Preferences *prefs = Preferences::instance();
     mUi->reloadTilesetImages->setChecked(prefs->reloadTilesetsOnChange());
     mUi->enableDtd->setChecked(prefs->dtdEnabled());
+    mUi->openLastFiles->setChecked(prefs->openLastFilesOnStartup());
     if (mUi->openGL->isEnabled())
         mUi->openGL->setChecked(prefs->useOpenGL());
 
@@ -326,9 +324,15 @@ void PreferencesDialog::toPreferences()
     prefs->setReloadTilesetsOnChanged(mUi->reloadTilesetImages->isChecked());
     prefs->setDtdEnabled(mUi->enableDtd->isChecked());
     prefs->setAutomappingDrawing(mUi->autoMapWhileDrawing->isChecked());
+    prefs->setOpenLastFilesOnStartup(mUi->openLastFiles->isChecked());
 }
 
 void PreferencesDialog::useAutomappingDrawingToggled(bool enabled)
 {
     Preferences::instance()->setAutomappingDrawing(enabled);
 }
+
+void PreferencesDialog::openLastFilesToggled(bool enabled)
+{
+    Preferences::instance()->setOpenLastFilesOnStartup(enabled);
+}
