@@ -43,7 +43,7 @@ namespace Internal {
 
 PropertiesDock::PropertiesDock(QWidget *parent)
     : QDockWidget(parent)
-    , mMapDocument(0)
+    , mMapDocument(nullptr)
     , mPropertyBrowser(new PropertyBrowser)
 {
     setObjectName(QLatin1String("propertiesDock"));
@@ -123,7 +123,7 @@ void PropertiesDock::mapDocumentChanged(MapDocument *mapDocument)
 
         currentObjectChanged(mapDocument->currentObject());
     } else {
-        currentObjectChanged(0);
+        currentObjectChanged(nullptr);
     }
 }
 
@@ -148,7 +148,7 @@ void PropertiesDock::currentObjectChanged(Object *object)
 {
     mPropertyBrowser->setObject(object);
 
-    const bool enabled = object != 0 && !isExternal(object);
+    const bool enabled = object != nullptr && !isExternal(object);
     mPropertyBrowser->setEnabled(enabled);
     mActionAddProperty->setEnabled(enabled);
 }
@@ -223,9 +223,16 @@ void PropertiesDock::removeProperty()
 
     const QString name = item->property()->propertyName();
     QUndoStack *undoStack = mMapDocument->undoStack();
+    QList<QtBrowserItem *> items = item->parent()->children();
+    if (items.count() > 1) {
+        int currentItemIndex = items.indexOf(item);
+        if (item == items.last()) {
+            mPropertyBrowser->setCurrentItem(items.at(currentItemIndex - 1));
+        } else {
+            mPropertyBrowser->setCurrentItem(items.at(currentItemIndex + 1));
+        }
+    }
     undoStack->push(new RemoveProperty(mMapDocument, mMapDocument->currentObjects(), name));
-
-    // TODO: Would be nice to automatically select the next property
 }
 
 void PropertiesDock::renameProperty()
@@ -269,7 +276,8 @@ bool PropertiesDock::event(QEvent *event)
     case QEvent::ShortcutOverride: {
         QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
         if (keyEvent->matches(QKeySequence::Delete) || keyEvent->key() == Qt::Key_Backspace) {
-            removeProperty();
+            if (event->type() == QEvent::KeyPress)
+                removeProperty();
             event->accept();
             return true;
         }
