@@ -40,13 +40,13 @@ class PropertiesDock : public QDockWidget
     Q_OBJECT
 
 public:
-    explicit PropertiesDock(QWidget *parent = 0);
+    explicit PropertiesDock(QWidget *parent = nullptr);
 
 public slots:
     void bringToFront();
 
 protected:
-    bool event(QEvent *event);
+    bool event(QEvent *event) override;
 
 private slots:
     void mapDocumentChanged(MapDocument *mapDocument);
