@@ -59,11 +59,11 @@ namespace Internal {
 PropertyBrowser::PropertyBrowser(QWidget *parent)
     : QtTreePropertyBrowser(parent)
     , mUpdating(false)
-    , mObject(0)
-    , mMapDocument(0)
+    , mObject(nullptr)
+    , mMapDocument(nullptr)
     , mVariantManager(new VariantPropertyManager(this))
     , mGroupManager(new QtGroupPropertyManager(this))
-    , mCustomPropertiesGroup(0)
+    , mCustomPropertiesGroup(nullptr)
 {
     setFactoryForManager(mVariantManager, new VariantEditorFactory(this));
     setResizeMode(ResizeToContents);
@@ -107,45 +107,10 @@ void PropertyBrowser::setObject(Object *object)
     if (mObject == object)
         return;
 
-    // Destroy all previous properties
-    mVariantManager->clear();
-    mGroupManager->clear();
-    mPropertyToId.clear();
-    mIdToProperty.clear();
-    mNameToProperty.clear();
-    mCustomPropertiesGroup = 0;
-
+    removeProperties();
     mObject = object;
 
-    if (!mObject)
-        return;
-
-    mUpdating = true;
-
-    // Add the built-in properties for each object type
-    switch (object->typeId()) {
-    case Object::MapType:               addMapProperties(); break;
-    case Object::MapObjectType:         addMapObjectProperties(); break;
-    case Object::LayerType:
-        switch (static_cast<Layer*>(object)->layerType()) {
-        case Layer::TileLayerType:      addTileLayerProperties();   break;
-        case Layer::ObjectGroupType:    addObjectGroupProperties(); break;
-        case Layer::ImageLayerType:     addImageLayerProperties();  break;
-        }
-        break;
-    case Object::TilesetType:           addTilesetProperties(); break;
-    case Object::TileType:              addTileProperties(); break;
-    case Object::TerrainType:           addTerrainProperties(); break;
-    }
-
-    // Add a node for the custom properties
-    mCustomPropertiesGroup = mGroupManager->addProperty(tr("Custom Properties"));
-    addProperty(mCustomPropertiesGroup);
-
-    mUpdating = false;
-
-    updateProperties();
-    updateCustomProperties();
+    addProperties();
 }
 
 void PropertyBrowser::setMapDocument(MapDocument *mapDocument)
@@ -216,6 +181,14 @@ void PropertyBrowser::editCustomProperty(const QString &name)
         editItem(propertyItems.first());
 }
 
+bool PropertyBrowser::event(QEvent *event)
+{
+    if (event->type() == QEvent::LanguageChange)
+        retranslateUi();
+
+    return QtTreePropertyBrowser::event(event);
+}
+
 void PropertyBrowser::mapChanged()
 {
     if (mObject == mMapDocument->map())
@@ -279,7 +252,7 @@ void PropertyBrowser::propertyAdded(Object *object, const QString &name)
         // Determine the property preceding the new property, if any
         const int index = mObject->properties().keys().indexOf(name);
         const QList<QtProperty *> properties = mCustomPropertiesGroup->subProperties();
-        QtProperty *precedingProperty = (index > 0) ? properties.at(index - 1) : 0;
+        QtProperty *precedingProperty = (index > 0) ? properties.at(index - 1) : nullptr;
 
         mUpdating = true;
         QtVariantProperty *property = mVariantManager->addProperty(QVariant::String, name);
@@ -298,7 +271,7 @@ void PropertyBrowser::propertyRemoved(Object *object, const QString &name)
         return;
     if (mObject == object) {
         bool deleteProperty = true;
-        foreach (Object *obj, mMapDocument->currentObjects()) {
+        for (Object *obj : mMapDocument->currentObjects()) {
             if (mObject == obj)
                 continue;
             if (obj->properties().contains(name)) {
@@ -378,9 +351,6 @@ void PropertyBrowser::addMapProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Map"));
 
-    createProperty(SizeProperty, QVariant::Size, tr("Size"), groupProperty)->setEnabled(false);
-    createProperty(TileSizeProperty, QVariant::Size, tr("Tile Size"), groupProperty);
-
     QtVariantProperty *orientationProperty =
             createProperty(OrientationProperty,
                            QtVariantPropertyManager::enumTypeId(),
@@ -389,6 +359,11 @@ void PropertyBrowser::addMapProperties()
 
     orientationProperty->setAttribute(QLatin1String("enumNames"), mOrientationNames);
 
+    createProperty(WidthProperty, QVariant::Int, tr("Width"), groupProperty)->setEnabled(false);
+    createProperty(HeightProperty, QVariant::Int, tr("Height"), groupProperty)->setEnabled(false);
+    createProperty(TileWidthProperty, QVariant::Int, tr("Tile Width"), groupProperty);
+    createProperty(TileHeightProperty, QVariant::Int, tr("Tile Height"), groupProperty);
+
     createProperty(HexSideLengthProperty, QVariant::Int, tr("Tile Side Length (Hex)"), groupProperty);
 
     QtVariantProperty *staggerAxisProperty =
@@ -430,7 +405,7 @@ void PropertyBrowser::addMapProperties()
 static QStringList objectTypeNames()
 {
     QStringList names;
-    foreach (const ObjectType &type, Preferences::instance()->objectTypes())
+    for (const ObjectType &type : Preferences::instance()->objectTypes())
         names.append(type.name);
     return names;
 }
@@ -447,8 +422,10 @@ void PropertyBrowser::addMapObjectProperties()
     typeProperty->setAttribute(QLatin1String("suggestions"), objectTypeNames());
 
     createProperty(VisibleProperty, QVariant::Bool, tr("Visible"), groupProperty);
-    createProperty(PositionProperty, QVariant::PointF, tr("Position"), groupProperty);
-    createProperty(SizeProperty, QVariant::SizeF, tr("Size"), groupProperty);
+    createProperty(XProperty, QVariant::Double, tr("X"), groupProperty);
+    createProperty(YProperty, QVariant::Double, tr("Y"), groupProperty);
+    createProperty(WidthProperty, QVariant::Double, tr("Width"), groupProperty);
+    createProperty(HeightProperty, QVariant::Double, tr("Height"), groupProperty);
     createProperty(RotationProperty, QVariant::Double, tr("Rotation"), groupProperty);
 
     if (!static_cast<const MapObject*>(mObject)->cell().isEmpty()) {
@@ -478,6 +455,8 @@ void PropertyBrowser::addTileLayerProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Tile Layer"));
     addLayerProperties(groupProperty);
+    createProperty(OffsetXProperty, QVariant::Double, tr("Horizontal Offset"), groupProperty);
+    createProperty(OffsetYProperty, QVariant::Double, tr("Vertical Offset"), groupProperty);
     addProperty(groupProperty);
 }
 
@@ -485,6 +464,8 @@ void PropertyBrowser::addObjectGroupProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Object Layer"));
     addLayerProperties(groupProperty);
+    createProperty(OffsetXProperty, QVariant::Double, tr("Horizontal Offset"), groupProperty);
+    createProperty(OffsetYProperty, QVariant::Double, tr("Vertical Offset"), groupProperty);
 
     createProperty(ColorProperty, QVariant::Color, tr("Color"), groupProperty);
 
@@ -512,7 +493,8 @@ void PropertyBrowser::addImageLayerProperties()
                                       Utils::readableImageFormatsFilter());
 
     createProperty(ColorProperty, QVariant::Color, tr("Transparent Color"), groupProperty);
-    createProperty(PositionProperty, QVariant::Point, tr("Position"), groupProperty);
+    createProperty(XProperty, QVariant::Int, tr("X"), groupProperty);
+    createProperty(YProperty, QVariant::Int, tr("Y"), groupProperty);
     addProperty(groupProperty);
 }
 
@@ -521,6 +503,23 @@ void PropertyBrowser::addTilesetProperties()
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Tileset"));
     createProperty(NameProperty, QVariant::String, tr("Name"), groupProperty);
     createProperty(TileOffsetProperty, QVariant::Point, tr("Drawing Offset"), groupProperty);
+
+    // Next properties we should add only for non 'Collection of Images' tilesets
+    const Tileset *currentTileset = dynamic_cast<const Tileset*>(mObject);
+    if (!currentTileset->imageSource().isEmpty()) {
+        QtVariantProperty *srcImgProperty =
+                createProperty(SourceImageProperty, QVariant::String, tr("Source Image"), groupProperty);
+        QtVariantProperty *tileWidthProperty = createProperty(TileWidthProperty, QVariant::Int, tr("Tile Width"), groupProperty);
+        QtVariantProperty *tileHeightProperty = createProperty(TileHeightProperty, QVariant::Int, tr("Tile Height"), groupProperty);
+        QtVariantProperty *marginProperty = createProperty(MarginProperty, QVariant::Int, tr("Margin"), groupProperty);
+        QtVariantProperty *spacingProperty = createProperty(SpacingProperty, QVariant::Int, tr("Spacing"), groupProperty);
+        // Make these properties read-only
+        srcImgProperty->setEnabled(false);
+        tileWidthProperty->setEnabled(false);
+        tileHeightProperty->setEnabled(false);
+        marginProperty->setEnabled(false);
+        spacingProperty->setEnabled(false);
+    }
     addProperty(groupProperty);
 }
 
@@ -534,8 +533,7 @@ void PropertyBrowser::addTileProperties()
                                                             tr("Probability"),
                                                             groupProperty);
     probabilityProperty->setAttribute(QLatin1String("decimals"), 3);
-    probabilityProperty->setToolTip(tr("Relative chance this tile will be "
-                                       "picked while painting terrain"));
+    probabilityProperty->setToolTip(tr("Relative chance this tile will be picked"));
 
     addProperty(groupProperty);
 }
@@ -549,23 +547,17 @@ void PropertyBrowser::addTerrainProperties()
 
 void PropertyBrowser::applyMapValue(PropertyId id, const QVariant &val)
 {
-    QUndoCommand *command = 0;
+    QUndoCommand *command = nullptr;
 
     switch (id) {
-    case TileSizeProperty: {
-        const Map *map = static_cast<Map*>(mObject);
-        const QSize tileSize = val.toSize();
-        if (tileSize.width() != map->tileWidth()) {
-            command = new ChangeMapProperty(mMapDocument,
-                                            ChangeMapProperty::TileWidth,
-                                            tileSize.width());
-        } else if (tileSize.height() != map->tileHeight()) {
-            command = new ChangeMapProperty(mMapDocument,
-                                            ChangeMapProperty::TileHeight,
-                                            tileSize.height());
-        }
+    case TileWidthProperty:
+        command = new ChangeMapProperty(mMapDocument, ChangeMapProperty::TileWidth,
+                                        val.toInt());
+        break;
+    case TileHeightProperty:
+        command = new ChangeMapProperty(mMapDocument, ChangeMapProperty::TileHeight,
+                                        val.toInt());
         break;
-    }
     case OrientationProperty: {
         Map::Orientation orientation = static_cast<Map::Orientation>(val.toInt() + 1);
         command = new ChangeMapProperty(mMapDocument, orientation);
@@ -609,7 +601,7 @@ void PropertyBrowser::applyMapValue(PropertyId id, const QVariant &val)
 
 QUndoCommand *PropertyBrowser::applyMapObjectValueTo(PropertyId id, const QVariant &val, MapObject *mapObject)
 {
-    QUndoCommand *command = 0;
+    QUndoCommand *command = nullptr;
 
     switch (id) {
     case NameProperty:
@@ -621,15 +613,27 @@ QUndoCommand *PropertyBrowser::applyMapObjectValueTo(PropertyId id, const QVaria
     case VisibleProperty:
         command = new SetMapObjectVisible(mMapDocument, mapObject, val.toBool());
         break;
-    case PositionProperty: {
-        const QPointF newPos = val.toPointF();
+    case XProperty: {
         const QPointF oldPos = mapObject->position();
+        const QPointF newPos(val.toReal(), oldPos.y());
         command = new MoveMapObject(mMapDocument, mapObject, newPos, oldPos);
         break;
     }
-    case SizeProperty: {
-        const QSizeF newSize = val.toSizeF();
+    case YProperty: {
+        const QPointF oldPos = mapObject->position();
+        const QPointF newPos(oldPos.x(), val.toReal());
+        command = new MoveMapObject(mMapDocument, mapObject, newPos, oldPos);
+        break;
+    }
+    case WidthProperty: {
         const QSizeF oldSize = mapObject->size();
+        const QSizeF newSize(val.toReal(), oldSize.height());
+        command = new ResizeMapObject(mMapDocument, mapObject, newSize, oldSize);
+        break;
+    }
+    case HeightProperty: {
+        const QSizeF oldSize = mapObject->size();
+        const QSizeF newSize(oldSize.width(), val.toReal());
         command = new ResizeMapObject(mMapDocument, mapObject, newSize, oldSize);
         break;
     }
@@ -674,9 +678,10 @@ void PropertyBrowser::applyMapObjectValue(PropertyId id, const QVariant &val)
     //Used to share non-custom properties.
     QList<MapObject*> selectedObjects = mMapDocument->selectedObjects();
     if (selectedObjects.size() > 1) {
-        foreach (MapObject *obj, selectedObjects) {
+        for (MapObject *obj : selectedObjects) {
             if (obj != mapObject) {
-                mMapDocument->undoStack()->push(applyMapObjectValueTo(id, val, obj));
+                if (QUndoCommand *cmd = applyMapObjectValueTo(id, val, obj))
+                    mMapDocument->undoStack()->push(cmd);
             }
         }
     }
@@ -688,7 +693,7 @@ void PropertyBrowser::applyLayerValue(PropertyId id, const QVariant &val)
 {
     Layer *layer = static_cast<Layer*>(mObject);
     const int layerIndex = mMapDocument->map()->layers().indexOf(layer);
-    QUndoCommand *command = 0;
+    QUndoCommand *command = nullptr;
 
     switch (id) {
     case NameProperty:
@@ -700,6 +705,17 @@ void PropertyBrowser::applyLayerValue(PropertyId id, const QVariant &val)
     case OpacityProperty:
         command = new SetLayerOpacity(mMapDocument, layerIndex, val.toDouble());
         break;
+    case OffsetXProperty:
+    case OffsetYProperty: {
+        QPointF offset = layer->offset();
+
+        if (id == OffsetXProperty)
+            offset.setX(val.toDouble());
+        else
+            offset.setY(val.toDouble());
+
+        command = new SetLayerOffset(mMapDocument, layerIndex, offset);
+    }
     default:
         switch (layer->layerType()) {
         case Layer::TileLayerType:   applyTileLayerValue(id, val);   break;
@@ -722,7 +738,7 @@ void PropertyBrowser::applyTileLayerValue(PropertyId id, const QVariant &val)
 void PropertyBrowser::applyObjectGroupValue(PropertyId id, const QVariant &val)
 {
     ObjectGroup *objectGroup = static_cast<ObjectGroup*>(mObject);
-    QUndoCommand *command = 0;
+    QUndoCommand *command = nullptr;
 
     switch (id) {
     case ColorProperty: {
@@ -779,9 +795,15 @@ void PropertyBrowser::applyImageLayerValue(PropertyId id, const QVariant &val)
                                                        imageSource));
         break;
     }
-    case PositionProperty: {
-        QPoint pos = val.value<QPoint>();
-
+    case XProperty: {
+        QPoint pos(val.toReal(), imageLayer->y());
+        undoStack->push(new ChangeImageLayerPosition(mMapDocument,
+                                                     imageLayer,
+                                                     pos));
+        break;
+    }
+    case YProperty: {
+        QPoint pos(imageLayer->x(), val.toReal());
         undoStack->push(new ChangeImageLayerPosition(mMapDocument,
                                                      imageLayer,
                                                      pos));
@@ -848,14 +870,63 @@ QtVariantProperty *PropertyBrowser::createProperty(PropertyId id, int type,
     parent->addSubProperty(property);
     mPropertyToId.insert(property, id);
 
-    if (id != CustomProperty)
+    if (id != CustomProperty) {
+        Q_ASSERT(!mIdToProperty.contains(id));
         mIdToProperty.insert(id, property);
-    else
+    } else {
         mNameToProperty.insert(name, property);
+    }
 
     return property;
 }
 
+void PropertyBrowser::addProperties()
+{
+    if (!mObject)
+        return;
+
+    mUpdating = true;
+
+    // Add the built-in properties for each object type
+    switch (mObject->typeId()) {
+    case Object::MapType:               addMapProperties(); break;
+    case Object::MapObjectType:         addMapObjectProperties(); break;
+    case Object::LayerType:
+        switch (static_cast<Layer*>(mObject)->layerType()) {
+        case Layer::TileLayerType:      addTileLayerProperties();   break;
+        case Layer::ObjectGroupType:    addObjectGroupProperties(); break;
+        case Layer::ImageLayerType:     addImageLayerProperties();  break;
+        }
+        break;
+    case Object::TilesetType:           addTilesetProperties(); break;
+    case Object::TileType:              addTileProperties(); break;
+    case Object::TerrainType:           addTerrainProperties(); break;
+    }
+
+    // Make sure the color property is collapsed, to save space
+    if (QtProperty *colorProperty = mIdToProperty.value(ColorProperty))
+        setExpanded(items(colorProperty).first(), false);
+
+    // Add a node for the custom properties
+    mCustomPropertiesGroup = mGroupManager->addProperty(tr("Custom Properties"));
+    addProperty(mCustomPropertiesGroup);
+
+    mUpdating = false;
+
+    updateProperties();
+    updateCustomProperties();
+}
+
+void PropertyBrowser::removeProperties()
+{
+    mVariantManager->clear();
+    mGroupManager->clear();
+    mPropertyToId.clear();
+    mIdToProperty.clear();
+    mNameToProperty.clear();
+    mCustomPropertiesGroup = nullptr;
+}
+
 void PropertyBrowser::updateProperties()
 {
     mUpdating = true;
@@ -863,8 +934,10 @@ void PropertyBrowser::updateProperties()
     switch (mObject->typeId()) {
     case Object::MapType: {
         const Map *map = static_cast<const Map*>(mObject);
-        mIdToProperty[SizeProperty]->setValue(map->size());
-        mIdToProperty[TileSizeProperty]->setValue(map->tileSize());
+        mIdToProperty[WidthProperty]->setValue(map->width());
+        mIdToProperty[HeightProperty]->setValue(map->height());
+        mIdToProperty[TileWidthProperty]->setValue(map->tileWidth());
+        mIdToProperty[TileHeightProperty]->setValue(map->tileHeight());
         mIdToProperty[OrientationProperty]->setValue(map->orientation() - 1);
         mIdToProperty[HexSideLengthProperty]->setValue(map->hexSideLength());
         mIdToProperty[StaggerAxisProperty]->setValue(map->staggerAxis());
@@ -883,8 +956,10 @@ void PropertyBrowser::updateProperties()
         mIdToProperty[NameProperty]->setValue(mapObject->name());
         mIdToProperty[TypeProperty]->setValue(mapObject->type());
         mIdToProperty[VisibleProperty]->setValue(mapObject->isVisible());
-        mIdToProperty[PositionProperty]->setValue(mapObject->position());
-        mIdToProperty[SizeProperty]->setValue(mapObject->size());
+        mIdToProperty[XProperty]->setValue(mapObject->x());
+        mIdToProperty[YProperty]->setValue(mapObject->y());
+        mIdToProperty[WidthProperty]->setValue(mapObject->width());
+        mIdToProperty[HeightProperty]->setValue(mapObject->height());
         mIdToProperty[RotationProperty]->setValue(mapObject->rotation());
 
         if (QtVariantProperty *property = mIdToProperty[FlippingProperty]) {
@@ -906,8 +981,12 @@ void PropertyBrowser::updateProperties()
 
         switch (layer->layerType()) {
         case Layer::TileLayerType:
+            mIdToProperty[OffsetXProperty]->setValue(layer->offset().x());
+            mIdToProperty[OffsetYProperty]->setValue(layer->offset().y());
             break;
         case Layer::ObjectGroupType: {
+            mIdToProperty[OffsetXProperty]->setValue(layer->offset().x());
+            mIdToProperty[OffsetYProperty]->setValue(layer->offset().y());
             const ObjectGroup *objectGroup = static_cast<const ObjectGroup*>(layer);
             QColor color = objectGroup->color();
             if (!color.isValid())
@@ -920,7 +999,8 @@ void PropertyBrowser::updateProperties()
             const ImageLayer *imageLayer = static_cast<const ImageLayer*>(layer);
             mIdToProperty[ImageSourceProperty]->setValue(imageLayer->imageSource());
             mIdToProperty[ColorProperty]->setValue(imageLayer->transparentColor());
-            mIdToProperty[PositionProperty]->setValue(imageLayer->position());
+            mIdToProperty[XProperty]->setValue(imageLayer->x());
+            mIdToProperty[YProperty]->setValue(imageLayer->y());
             break;
         }
         break;
@@ -929,12 +1009,19 @@ void PropertyBrowser::updateProperties()
         const Tileset *tileset = static_cast<const Tileset*>(mObject);
         mIdToProperty[NameProperty]->setValue(tileset->name());
         mIdToProperty[TileOffsetProperty]->setValue(tileset->tileOffset());
+        if (!tileset->imageSource().isEmpty()) {
+            mIdToProperty[SourceImageProperty]->setValue(tileset->imageSource());
+            mIdToProperty[TileWidthProperty]->setValue(tileset->tileWidth());
+            mIdToProperty[TileHeightProperty]->setValue(tileset->tileHeight());
+            mIdToProperty[MarginProperty]->setValue(tileset->margin());
+            mIdToProperty[SpacingProperty]->setValue(tileset->tileSpacing());
+        }
         break;
     }
     case Object::TileType: {
         const Tile *tile = static_cast<const Tile*>(mObject);
         mIdToProperty[IdProperty]->setValue(tile->id());
-        mIdToProperty[TileProbabilityProperty]->setValue(tile->terrainProbability());
+        mIdToProperty[TileProbabilityProperty]->setValue(tile->probability());
         break;
     }
     case Object::TerrainType: {
@@ -959,7 +1046,7 @@ void PropertyBrowser::updateCustomProperties()
 
     mCombinedProperties = mObject->properties();
     // Add properties from selected objects which mObject does not contain to mCombinedProperties.
-    foreach (Object *obj, mMapDocument->currentObjects()) {
+    for (Object *obj : mMapDocument->currentObjects()) {
         if (obj == mObject)
             continue;
 
@@ -967,9 +1054,8 @@ void PropertyBrowser::updateCustomProperties()
 
         while (it.hasNext()) {
             it.next();
-            if (!mCombinedProperties.contains(it.key())) {
+            if (!mCombinedProperties.contains(it.key()))
                 mCombinedProperties.insert(it.key(), tr(""));
-            }
         }
     }
 
@@ -999,7 +1085,7 @@ void PropertyBrowser::updatePropertyColor(const QString &name)
     QString propertyValue = property->valueText();
 
     // If one of the objects doesn't have this property then gray out the name and value.
-    foreach (Object *obj, mMapDocument->currentObjects()) {
+    for (Object *obj : mMapDocument->currentObjects()) {
         if (!obj->hasProperty(propertyName)) {
             property->setNameColor(Qt::gray);
             property->setValueColor(Qt::gray);
@@ -1008,7 +1094,7 @@ void PropertyBrowser::updatePropertyColor(const QString &name)
     }
 
     // If one of the objects doesn't have the same property value then gray out the value.
-    foreach (Object *obj, mMapDocument->currentObjects()) {
+    for (Object *obj : mMapDocument->currentObjects()) {
         if (obj == mObject)
             continue;
         if (obj->property(propertyName) != propertyValue) {
