@@ -51,7 +51,7 @@ class PropertyBrowser : public QtTreePropertyBrowser
     Q_OBJECT
 
 public:
-    explicit PropertyBrowser(QWidget *parent = 0);
+    explicit PropertyBrowser(QWidget *parent = nullptr);
 
     /**
      * Sets the \a object for which to display the properties.
@@ -80,6 +80,9 @@ public:
      */
     void editCustomProperty(const QString &name);
 
+protected:
+    bool event(QEvent *event) override;
+
 private slots:
     void mapChanged();
     void objectsChanged(const QList<MapObject*> &objects);
@@ -103,13 +106,18 @@ private:
     enum PropertyId {
         NameProperty,
         TypeProperty,
-        PositionProperty,
-        SizeProperty,
+        XProperty,
+        YProperty,
+        WidthProperty,
+        HeightProperty,
         RotationProperty,
         VisibleProperty,
         OpacityProperty,
+        OffsetXProperty,
+        OffsetYProperty,
         ColorProperty,
-        TileSizeProperty,
+        TileWidthProperty,
+        TileHeightProperty,
         OrientationProperty,
         HexSideLengthProperty,
         StaggerAxisProperty,
@@ -120,6 +128,9 @@ private:
         FlippingProperty,
         DrawOrderProperty,
         TileOffsetProperty,
+        SourceImageProperty,
+        MarginProperty,
+        SpacingProperty,
         TileProbabilityProperty,
         IdProperty,
         CustomProperty
@@ -151,8 +162,11 @@ private:
                                       const QString &name,
                                       QtProperty *parent);
 
+    void addProperties();
+    void removeProperties();
     void updateProperties();
     void updateCustomProperties();
+    void retranslateUi();
     bool mUpdating;
 
     void updatePropertyColor(const QString &name);
@@ -168,12 +182,11 @@ private:
     QHash<PropertyId, QtVariantProperty *> mIdToProperty;
     QHash<QString, QtVariantProperty *> mNameToProperty;
 
+    Properties mCombinedProperties;
+
     QStringList mStaggerAxisNames;
     QStringList mStaggerIndexNames;
     QStringList mOrientationNames;
-
-    Properties mCombinedProperties;
-
     QStringList mLayerFormatNames;
     QStringList mRenderOrderNames;
     QStringList mFlippingFlagNames;
@@ -185,6 +198,12 @@ inline Object *PropertyBrowser::object() const
     return mObject;
 }
 
+inline void PropertyBrowser::retranslateUi()
+{
+    removeProperties();
+    addProperties();
+}
+
 } // namespace Internal
 } // namespace Tiled
 
