@@ -28,6 +28,7 @@
 #include "mapdocument.h"
 #include "mapscene.h"
 #include "painttilelayer.h"
+#include "randompicker.h"
 #include "tilelayer.h"
 #include "tileset.h"
 #include "tile.h"
@@ -46,7 +47,7 @@ TerrainBrush::TerrainBrush(QObject *parent)
                                ":images/24x24/terrain-edit.png")),
                        QKeySequence(tr("T")),
                        parent)
-    , mTerrain(0)
+    , mTerrain(nullptr)
     , mPaintX(0), mPaintY(0)
     , mOffsetX(0), mOffsetY(0)
     , mIsActive(false)
@@ -83,9 +84,6 @@ void TerrainBrush::tilePositionChanged(const QPoint &pos)
             updateBrush(p);
             doPaint(true, p.x(), p.y());
         }
-        // HACK-ish: because the line may traverse in the reverse direction, updateBrush() leaves these at the line start point
-        mPaintX = pos.x();
-        mPaintY = pos.y();
         break;
     }
     case LineStartSet: {
@@ -166,13 +164,13 @@ void TerrainBrush::languageChanged()
 static Terrain *firstTerrain(MapDocument *mapDocument)
 {
     if (!mapDocument)
-        return 0;
+        return nullptr;
 
-    foreach (Tileset *tileset, mapDocument->map()->tilesets())
+    foreach (const SharedTileset &tileset, mapDocument->map()->tilesets())
         if (tileset->terrainCount() > 0)
             return tileset->terrain(0);
 
-    return 0;
+    return nullptr;
 }
 
 void TerrainBrush::mapDocumentChanged(MapDocument *oldDocument,
@@ -181,7 +179,7 @@ void TerrainBrush::mapDocumentChanged(MapDocument *oldDocument,
     AbstractTileTool::mapDocumentChanged(oldDocument, newDocument);
 
     // Reset the brush, since it probably became invalid
-    brushItem()->setTileLayer(0);
+    brushItem()->clear();
 
     // Don't use setTerrain since we do not want to update the brush right now
     mTerrain = firstTerrain(newDocument);
@@ -229,7 +227,7 @@ void TerrainBrush::capture()
 
 void TerrainBrush::doPaint(bool mergeable, int whereX, int whereY)
 {
-    TileLayer *stamp = brushItem()->tileLayer();
+    TileLayer *stamp = brushItem()->tileLayer().data();
 
     if (!stamp)
         return;
@@ -256,28 +254,28 @@ static inline unsigned makeTerrain(int t)
     return t << 24 | t << 16 | t << 8 | t;
 }
 
-static Tile *findBestTile(Tileset *tileset, unsigned terrain, unsigned considerationMask)
+static Tile *findBestTile(const Tileset &tileset, unsigned terrain, unsigned considerationMask)
 {
     // we should have hooked 0xFFFFFFFF terrains outside this function
     Q_ASSERT(terrain != 0xFFFFFFFF);
 
     // if all quadrants are set to 'no terrain', then the 'empty' tile is the only choice we can deduce
     if (terrain == 0xFFFFFFFF)
-        return NULL;
+        return nullptr;
 
-    QList<Tile*> matches;
+    RandomPicker<Tile*> matches;
     int penalty = INT_MAX;
 
     // TODO: this is a slow linear search, perhaps we could use a better find algorithm...
-    foreach (Tile *t, tileset->tiles()) {
+    foreach (Tile *t, tileset.tiles()) {
         if ((t->terrain() & considerationMask) != (terrain & considerationMask))
             continue;
 
         // calculate the tile transition penalty based on shortest distance to target terrain type
-        int tr = tileset->terrainTransitionPenalty(t->terrain() >> 24, terrain >> 24);
-        int tl = tileset->terrainTransitionPenalty((t->terrain() >> 16) & 0xFF, (terrain >> 16) & 0xFF);
-        int br = tileset->terrainTransitionPenalty((t->terrain() >> 8) & 0xFF, (terrain >> 8) & 0xFF);
-        int bl = tileset->terrainTransitionPenalty(t->terrain() & 0xFF, terrain & 0xFF);
+        int tr = tileset.terrainTransitionPenalty(t->terrain() >> 24, terrain >> 24);
+        int tl = tileset.terrainTransitionPenalty((t->terrain() >> 16) & 0xFF, (terrain >> 16) & 0xFF);
+        int br = tileset.terrainTransitionPenalty((t->terrain() >> 8) & 0xFF, (terrain >> 8) & 0xFF);
+        int bl = tileset.terrainTransitionPenalty(t->terrain() & 0xFF, terrain & 0xFF);
 
         // if there is no path to the destination terrain, this isn't a useful transition
         if (tr < 0 || tl < 0 || br < 0 || bl < 0)
@@ -290,36 +288,16 @@ static Tile *findBestTile(Tileset *tileset, unsigned terrain, unsigned considera
                 matches.clear();
             penalty = transitionPenalty;
 
-            matches.push_back(t);
+            matches.add(t, t->probability());
         }
     }
 
-    // choose a candidate at random, with consideration for terrain probability
-    if (!matches.isEmpty()) {
-        // determine the range of probability
-        float sum = 0.f;
-        for (int i = 0; i < matches.size(); ++i) {
-            float probability = matches.at(i)->terrainProbability();
-            if (probability > 0.f)
-                sum += probability;
-        }
+    // choose a candidate at random, with consideration for probability
+    if (!matches.isEmpty())
+        return matches.pick();
 
-        float random = ((float)rand() / RAND_MAX) * sum;
-
-        // determine which match was hit
-        sum = 0.f;
-        for (int i = 0; i < matches.size(); ++i) {
-            float probability = matches.at(i)->terrainProbability();
-            if (probability > 0.f) {
-                sum += probability;
-                if (random <= sum)
-                    return matches[i];
-            }
-        }
-    }
-
-    // TODO: conveniently, the NULL tile doesn't currently work, but when it does, we need to signal a failure to find any matches some other way
-    return NULL;
+    // TODO: conveniently, the null tile doesn't currently work, but when it does, we need to signal a failure to find any matches some other way
+    return nullptr;
 }
 
 static unsigned terrain(const Tile *tile)
@@ -376,7 +354,7 @@ void TerrainBrush::updateBrush(QPoint cursorPos, const QVector<QPoint> *list)
     if (!currentLayer->bounds().contains(cursorPos))
         return;
 
-    Tileset *terrainTileset = 0;
+    Tileset *terrainTileset = nullptr;
     int terrainId = -1;
     if (mTerrain) {
         terrainTileset = mTerrain->tileset();
@@ -423,7 +401,7 @@ void TerrainBrush::updateBrush(QPoint cursorPos, const QVector<QPoint> *list)
         const unsigned currentTerrain = ::terrain(tile);
 
         // get the tileset for this tile
-        Tileset *tileset = 0;
+        Tileset *tileset = nullptr;
         if (terrainTileset) {
             // if we are painting a terrain, then we'll use the terrains tileset
             tileset = terrainTileset;
@@ -491,9 +469,9 @@ void TerrainBrush::updateBrush(QPoint cursorPos, const QVector<QPoint> *list)
         }
 
         // find the most appropriate tile in the tileset
-        Tile *paste = 0;
+        Tile *paste = nullptr;
         if (preferredTerrain != 0xFFFFFFFF) {
-            paste = findBestTile(tileset, preferredTerrain, mask);
+            paste = findBestTile(*tileset, preferredTerrain, mask);
             if (!paste)
                 continue;
         }
@@ -529,19 +507,25 @@ void TerrainBrush::updateBrush(QPoint cursorPos, const QVector<QPoint> *list)
     }
 
     // create a stamp for the terrain block
-    TileLayer *stamp = new TileLayer(QString(), 0, 0, brushRect.width(), brushRect.height());
+    SharedTileLayer stamp = SharedTileLayer(new TileLayer(QString(),
+                                                          brushRect.left(),
+                                                          brushRect.top(),
+                                                          brushRect.width(),
+                                                          brushRect.height()));
 
     for (int y = brushRect.top(); y <= brushRect.bottom(); ++y) {
         for (int x = brushRect.left(); x <= brushRect.right(); ++x) {
-            int i = y*layerWidth + x;
+            int i = y * layerWidth + x;
             if (!checked[i])
                 continue;
 
             Tile *tile = newTerrain[i];
-            if (tile)
-                stamp->setCell(x - brushRect.left(), y - brushRect.top(), Cell(tile));
-            else {
-                // TODO: we need to do something to erase tiles where checked[i] is true, and newTerrain[i] is NULL
+            if (tile) {
+                stamp->setCell(x - brushRect.left(),
+                               y - brushRect.top(),
+                               Cell(tile));
+            } else {
+                // TODO: we need to do something to erase tiles where checked[i] is true, and newTerrain[i] is null
                 // is there an eraser stamp? investigate how the eraser works...
             }
         }
@@ -553,8 +537,6 @@ void TerrainBrush::updateBrush(QPoint cursorPos, const QVector<QPoint> *list)
     delete[] checked;
     delete[] newTerrain;
 
-    brushItem()->setTileLayerPosition(brushRect.topLeft());
-
     mPaintX = cursorPos.x();
     mPaintY = cursorPos.y();
     mOffsetX = cursorPos.x() - brushRect.left();
