@@ -25,6 +25,7 @@
 #include "addremovemapobject.h"
 #include "addremovetiles.h"
 #include "addremovetileset.h"
+#include "containerhelpers.h"
 #include "documentmanager.h"
 #include "editterraindialog.h"
 #include "erasetiles.h"
@@ -38,11 +39,12 @@
 #include "terrain.h"
 #include "tile.h"
 #include "tilelayer.h"
-#include "tileset.h"
+#include "tilesetformat.h"
 #include "tilesetmodel.h"
 #include "tilesetview.h"
 #include "tilesetmanager.h"
-#include "tmxmapwriter.h"
+#include "tilestamp.h"
+#include "tmxmapformat.h"
 #include "utils.h"
 #include "zoomable.h"
 
@@ -91,8 +93,8 @@ public:
                                                 "Export Tileset"));
     }
 
-    void undo() { swap(); }
-    void redo() { swap(); }
+    void undo() override { swap(); }
+    void redo() override { swap(); }
 
 private:
     void swap()
@@ -111,7 +113,7 @@ private:
 class TilesetMenuButton : public QToolButton
 {
 public:
-    TilesetMenuButton(QWidget *parent = 0)
+    TilesetMenuButton(QWidget *parent = nullptr)
         : QToolButton(parent)
     {
         setArrowType(Qt::DownArrow);
@@ -124,7 +126,7 @@ public:
     }
 
 protected:
-    void paintEvent(QPaintEvent *)
+    void paintEvent(QPaintEvent *) override
     {
         QStylePainter p(this);
         QStyleOptionToolButton opt;
@@ -137,43 +139,6 @@ protected:
     }
 };
 
-/**
- * Determines whether a cell references a certain tileset.
- */
-class ReferencesTileset
-{
-public:
-    explicit ReferencesTileset(Tileset *tileset) : mTileset(tileset) {}
-
-    bool operator() (const Cell &cell) const
-    {
-        if (const Tile *tile = cell.tile)
-            return tile->tileset() == mTileset;
-        return false;
-    }
-
-private:
-    Tileset *mTileset;
-};
-
-/**
- * Determines whether a cell matches certain tiles.
- */
-class MatchesAnyTile
-{
-public:
-    MatchesAnyTile(const QList<Tile*> &tiles) : mTiles(tiles) {}
-
-    bool operator() (const Cell &cell) const
-    {
-        if (Tile *tile = cell.tile)
-            return mTiles.contains(tile);
-        return false;
-    }
-
-private:
-    QList<Tile*> mTiles;
-};
 
 template<typename Condition>
 static bool hasTileReferences(MapDocument *mapDocument, Condition condition)
@@ -218,12 +183,12 @@ static void removeTileReferences(MapDocument *mapDocument, Condition condition)
 
 TilesetDock::TilesetDock(QWidget *parent):
     QDockWidget(parent),
-    mMapDocument(0),
+    mMapDocument(nullptr),
     mTabBar(new QTabBar),
     mViewStack(new QStackedWidget),
     mToolBar(new QToolBar),
-    mCurrentTile(0),
-    mCurrentTiles(0),
+    mCurrentTile(nullptr),
+    mCurrentTiles(nullptr),
     mNewTileset(new QAction(this)),
     mImportTileset(new QAction(this)),
     mExportTileset(new QAction(this)),
@@ -235,7 +200,9 @@ TilesetDock::TilesetDock(QWidget *parent):
     mTilesetMenuButton(new TilesetMenuButton(this)),
     mTilesetMenu(new QMenu(this)),
     mTilesetActionGroup(new QActionGroup(this)),
-    mTilesetMenuMapper(0)
+    mTilesetMenuMapper(nullptr),
+    mEmittingStampCaptured(false),
+    mSynchronizingSelection(false)
 {
     setObjectName(QLatin1String("TilesetDock"));
 
@@ -314,10 +281,10 @@ TilesetDock::TilesetDock(QWidget *parent):
     mZoomable->connectToComboBox(mZoomComboBox);
     horizontal->addWidget(mZoomComboBox);
 
-    connect(mViewStack, SIGNAL(currentChanged(int)),
-            this, SLOT(updateCurrentTiles()));
-    connect(mViewStack, SIGNAL(currentChanged(int)),
-            this, SLOT(updateCurrentTile()));
+    connect(mViewStack, &QStackedWidget::currentChanged,
+            this, &TilesetDock::updateCurrentTiles);
+    connect(mViewStack, &QStackedWidget::currentChanged,
+            this, &TilesetDock::currentTilesetChanged);
 
     connect(TilesetManager::instance(), SIGNAL(tilesetChanged(Tileset*)),
             this, SLOT(tilesetChanged(Tileset*)));
@@ -345,9 +312,13 @@ void TilesetDock::setMapDocument(MapDocument *mapDocument)
         return;
 
     // Hide while we update the tab bar, to avoid repeated layouting
+    // But, this causes problems on OS X (issue #1055)
+#ifndef Q_OS_OSX
     widget()->hide();
+#endif
 
-    setCurrentTiles(0);
+    setCurrentTiles(nullptr);
+    setCurrentTile(nullptr);
 
     if (mMapDocument) {
         // Remember the last visible tileset for this map
@@ -372,7 +343,7 @@ void TilesetDock::setMapDocument(MapDocument *mapDocument)
     if (mMapDocument) {
         mTilesets = mMapDocument->map()->tilesets();
 
-        foreach (Tileset *tileset, mTilesets) {
+        foreach (const SharedTileset &tileset, mTilesets) {
             TilesetView *view = new TilesetView;
             view->setMapDocument(mMapDocument);
             view->setZoomable(mZoomable);
@@ -403,11 +374,81 @@ void TilesetDock::setMapDocument(MapDocument *mapDocument)
                 break;
             }
         }
+
+        if (Object *object = mMapDocument->currentObject())
+            if (object->typeId() == Object::TileType)
+                setCurrentTile(static_cast<Tile*>(object));
     }
 
     updateActions();
 
+#ifndef Q_OS_OSX
     widget()->show();
+#endif
+}
+
+/**
+ * Synchronizes the selection with the given stamp. Ignored when the stamp is
+ * changing because of a selection change in the TilesetDock.
+ */
+void TilesetDock::selectTilesInStamp(const TileStamp &stamp)
+{
+    if (mEmittingStampCaptured)
+        return;
+
+    QSet<Tile*> processed;
+    QMap<QItemSelectionModel*, QItemSelection> selections;
+
+    for (const TileStampVariation &variation : stamp.variations()) {
+        const TileLayer &tileLayer = *variation.tileLayer();
+        for (const Cell &cell : tileLayer) {
+            if (Tile *tile = cell.tile) {
+                if (processed.contains(tile))
+                    continue;
+
+                processed.insert(tile); // avoid spending time on duplicates
+
+                Tileset *tileset = tile->tileset();
+                int tilesetIndex = mTilesets.indexOf(tileset->sharedPointer());
+                if (tilesetIndex != -1) {
+                    TilesetView *view = tilesetViewAt(tilesetIndex);
+                    if (!view->model()) // Lazily set up the model
+                        setupTilesetModel(view, tileset);
+
+                    const TilesetModel *model = view->tilesetModel();
+                    const QModelIndex modelIndex = model->tileIndex(tile);
+                    QItemSelectionModel *selectionModel = view->selectionModel();
+                    selections[selectionModel].select(modelIndex, modelIndex);
+                }
+            }
+        }
+    }
+
+    if (!selections.isEmpty()) {
+        mSynchronizingSelection = true;
+
+        // Mark captured tiles as selected
+        for (auto i = selections.constBegin(); i != selections.constEnd(); ++i) {
+            QItemSelectionModel *selectionModel = i.key();
+            const QItemSelection &selection = i.value();
+            selectionModel->select(selection, QItemSelectionModel::SelectCurrent);
+        }
+
+        // Show/edit properties of all captured tiles
+        mMapDocument->setSelectedTiles(processed.toList());
+
+        // Update the current tile (useful for animation and collision editors)
+        auto first = selections.begin();
+        QItemSelectionModel *selectionModel = first.key();
+        const QItemSelection &selection = first.value();
+        const QModelIndex currentIndex = selection.first().topLeft();
+        if (selectionModel->currentIndex() != currentIndex)
+            selectionModel->setCurrentIndex(currentIndex, QItemSelectionModel::NoUpdate);
+        else
+            currentChanged(currentIndex);
+
+        mSynchronizingSelection = false;
+    }
 }
 
 void TilesetDock::changeEvent(QEvent *e)
@@ -443,10 +484,28 @@ void TilesetDock::dropEvent(QDropEvent *e)
     }
 }
 
+void TilesetDock::currentTilesetChanged()
+{
+    if (const TilesetView *view = currentTilesetView())
+        if (const QItemSelectionModel *s = view->selectionModel())
+            setCurrentTile(view->tilesetModel()->tileAt(s->currentIndex()));
+}
+
 void TilesetDock::selectionChanged()
 {
     updateActions();
-    updateCurrentTiles();
+
+    if (!mSynchronizingSelection)
+        updateCurrentTiles();
+}
+
+void TilesetDock::currentChanged(const QModelIndex &index)
+{
+    if (!index.isValid())
+        return;
+
+    const TilesetModel *model = static_cast<const TilesetModel*>(index.model());
+    setCurrentTile(model->tileAt(index));
 }
 
 void TilesetDock::updateActions()
@@ -454,26 +513,16 @@ void TilesetDock::updateActions()
     bool external = false;
     bool hasImageSource = false;
     bool hasSelection = false;
-    TilesetView *view = 0;
+    TilesetView *view = nullptr;
     const int index = mTabBar->currentIndex();
 
     if (index > -1) {
         view = tilesetViewAt(index);
         if (view) {
-            Tileset *tileset = mTilesets.at(index);
+            Tileset *tileset = mTilesets.at(index).data();
 
-            if (!view->model()) {
-                // Lazily set up the model
-                view->setModel(new TilesetModel(tileset, view));
-
-                QItemSelectionModel *s = view->selectionModel();
-                connect(s, SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-                        SLOT(selectionChanged()));
-                connect(s, SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-                        SLOT(updateCurrentTile()));
-                connect(view, SIGNAL(pressed(QModelIndex)),
-                        SLOT(indexPressed(QModelIndex)));
-            }
+            if (!view->model()) // Lazily set up the model
+                setupTilesetModel(view, tileset);
 
             mViewStack->setCurrentIndex(index);
             external = tileset->isExternal();
@@ -482,8 +531,8 @@ void TilesetDock::updateActions()
         }
     }
 
-    const bool tilesetIsDisplayed = view != 0;
-    const bool mapIsDisplayed = mMapDocument != 0;
+    const bool tilesetIsDisplayed = view != nullptr;
+    const bool mapIsDisplayed = mMapDocument != nullptr;
 
     mNewTileset->setEnabled(mapIsDisplayed);
     mImportTileset->setEnabled(tilesetIsDisplayed && external);
@@ -538,13 +587,6 @@ void TilesetDock::updateCurrentTiles()
     setCurrentTiles(tileLayer);
 }
 
-void TilesetDock::updateCurrentTile()
-{
-    if (const TilesetView *view = currentTilesetView())
-        if (const QItemSelectionModel *s = view->selectionModel())
-            setCurrentTile(view->tilesetModel()->tileAt(s->currentIndex()));
-}
-
 void TilesetDock::indexPressed(const QModelIndex &index)
 {
     TilesetView *view = currentTilesetView();
@@ -558,7 +600,7 @@ void TilesetDock::tilesetAdded(int index, Tileset *tileset)
     view->setMapDocument(mMapDocument);
     view->setZoomable(mZoomable);
 
-    mTilesets.insert(index, tileset);
+    mTilesets.insert(index, tileset->sharedPointer());
     mTabBar->insertTab(index, tileset->name());
     mViewStack->insertWidget(index, view);
 
@@ -568,7 +610,7 @@ void TilesetDock::tilesetAdded(int index, Tileset *tileset)
 void TilesetDock::tilesetChanged(Tileset *tileset)
 {
     // Update the affected tileset model, if it exists
-    const int index = mTilesets.indexOf(tileset);
+    const int index = indexOf(mTilesets, tileset);
     if (index < 0)
         return;
 
@@ -579,10 +621,10 @@ void TilesetDock::tilesetChanged(Tileset *tileset)
 void TilesetDock::tilesetRemoved(Tileset *tileset)
 {
     // Delete the related tileset view
-    const int index = mTilesets.indexOf(tileset);
+    const int index = indexOf(mTilesets, tileset);
     Q_ASSERT(index != -1);
 
-    mTilesets.removeAt(index);
+    mTilesets.remove(index);
     mTabBar->removeTab(index);
     delete tilesetViewAt(index);
 
@@ -595,14 +637,20 @@ void TilesetDock::tilesetRemoved(Tileset *tileset)
         setCurrentTiles(cleaned);
     }
     if (mCurrentTile && mCurrentTile->tileset() == tileset)
-        setCurrentTile(0);
+        setCurrentTile(nullptr);
 
     updateActions();
 }
 
 void TilesetDock::tilesetMoved(int from, int to)
 {
+#if QT_VERSION >= 0x050200
     mTilesets.insert(to, mTilesets.takeAt(from));
+#else
+    SharedTileset tileset = mTilesets.at(from);
+    mTilesets.remove(from);
+    mTilesets.insert(to, tileset);
+#endif
 
     // Move the related tileset views
     QWidget *widget = mViewStack->widget(from);
@@ -614,7 +662,7 @@ void TilesetDock::tilesetMoved(int from, int to)
     const int start = qMin(from, to);
     const int end = qMax(from, to);
     for (int i = start; i <= end; ++i) {
-        const Tileset *tileset = mTilesets.at(i);
+        const SharedTileset &tileset = mTilesets.at(i);
         if (mTabBar->tabText(i) != tileset->name())
             mTabBar->setTabText(i, tileset->name());
     }
@@ -636,7 +684,7 @@ void TilesetDock::removeTileset()
  */
 void TilesetDock::removeTileset(int index)
 {
-    Tileset *tileset = mTilesets.at(index);
+    Tileset *tileset = mTilesets.at(index).data();
     const bool inUse = mMapDocument->map()->isTilesetUsed(tileset);
 
     // If the tileset is in use, warn the user and confirm removal
@@ -655,13 +703,18 @@ void TilesetDock::removeTileset(int index)
             return;
     }
 
-    QUndoCommand *remove = new RemoveTileset(mMapDocument, index, tileset);
+    QUndoCommand *remove = new RemoveTileset(mMapDocument, index);
     QUndoStack *undoStack = mMapDocument->undoStack();
 
     if (inUse) {
         // Remove references to tiles in this tileset from the current map
+        auto referencesTileset = [tileset] (const Cell &cell) {
+            if (const Tile *tile = cell.tile)
+                return tile->tileset() == tileset;
+            return false;
+        };
         undoStack->beginMacro(remove->text());
-        removeTileReferences(mMapDocument, ReferencesTileset(tileset));
+        removeTileReferences(mMapDocument, referencesTileset);
     }
     undoStack->push(remove);
     if (inUse)
@@ -693,9 +746,21 @@ void TilesetDock::setCurrentTiles(TileLayer *tiles)
             }
         }
         mMapDocument->setSelectedTiles(selectedTiles);
-    }
 
-    emit currentTilesChanged(mCurrentTiles);
+        // Create a tile stamp with these tiles
+        Map *map = mMapDocument->map();
+        Map *stamp = new Map(map->orientation(),
+                             tiles->width(),
+                             tiles->height(),
+                             map->tileWidth(),
+                             map->tileHeight());
+        stamp->addLayer(tiles->clone());
+        stamp->addTilesets(tiles->usedTilesets());
+
+        mEmittingStampCaptured = true;
+        emit stampCaptured(TileStamp(stamp));
+        mEmittingStampCaptured = false;
+    }
 }
 
 void TilesetDock::setCurrentTile(Tile *tile)
@@ -727,9 +792,9 @@ Tileset *TilesetDock::currentTileset() const
 {
     const int index = mTabBar->currentIndex();
     if (index == -1)
-        return 0;
+        return nullptr;
 
-    return mTilesets.at(index);
+    return mTilesets.at(index).data();
 }
 
 TilesetView *TilesetDock::currentTilesetView() const
@@ -742,6 +807,19 @@ TilesetView *TilesetDock::tilesetViewAt(int index) const
     return static_cast<TilesetView *>(mViewStack->widget(index));
 }
 
+void TilesetDock::setupTilesetModel(TilesetView *view, Tileset *tileset)
+{
+    view->setModel(new TilesetModel(tileset, view));
+
+    QItemSelectionModel *s = view->selectionModel();
+    connect(s, &QItemSelectionModel::selectionChanged,
+            this, &TilesetDock::selectionChanged);
+    connect(s, &QItemSelectionModel::currentChanged,
+            this, &TilesetDock::currentChanged);
+    connect(view, &TilesetView::pressed,
+            this, &TilesetDock::indexPressed);
+}
+
 void TilesetDock::editTilesetProperties()
 {
     Tileset *tileset = currentTileset();
@@ -758,21 +836,25 @@ void TilesetDock::exportTileset()
     if (!tileset)
         return;
 
-    Preferences *prefs = Preferences::instance();
+    const QString tsxFilter = tr("Tiled tileset files (*.tsx)");
 
-    const QLatin1String extension(".tsx");
+    FormatHelper<TilesetFormat> helper(FileFormat::ReadWrite, tsxFilter);
+
+    Preferences *prefs = Preferences::instance();
 
     QString suggestedFileName = prefs->lastPath(Preferences::ExternalTileset);
     suggestedFileName += QLatin1Char('/');
     suggestedFileName += tileset->name();
 
+    const QLatin1String extension(".tsx");
     if (!suggestedFileName.endsWith(extension))
         suggestedFileName.append(extension);
 
+    QString selectedFilter = tsxFilter;
     const QString fileName =
             QFileDialog::getSaveFileName(this, tr("Export Tileset"),
                                          suggestedFileName,
-                                         tr("Tiled tileset files (*.tsx)"));
+                                         helper.filter(), &selectedFilter);
 
     if (fileName.isEmpty())
         return;
@@ -780,12 +862,21 @@ void TilesetDock::exportTileset()
     prefs->setLastPath(Preferences::ExternalTileset,
                        QFileInfo(fileName).path());
 
-    TmxMapWriter writer;
+    TsxTilesetFormat tsxFormat;
+    TilesetFormat *format = helper.formatByNameFilter(selectedFilter);
+    if (!format)
+        format = &tsxFormat;
 
-    if (writer.writeTileset(tileset, fileName)) {
+    if (format->write(*tileset, fileName)) {
         QUndoCommand *command = new SetTilesetFileName(mMapDocument,
-                                                       tileset, fileName);
+                                                       tileset,
+                                                       fileName);
         mMapDocument->undoStack()->push(command);
+    } else {
+        QString error = format->errorString();
+        QMessageBox::critical(window(),
+                              tr("Export Tileset"),
+                              tr("Error saving tileset: %1").arg(error));
     }
 }
 
@@ -796,7 +887,8 @@ void TilesetDock::importTileset()
         return;
 
     QUndoCommand *command = new SetTilesetFileName(mMapDocument,
-                                                   tileset, QString());
+                                                   tileset,
+                                                   QString());
     mMapDocument->undoStack()->push(command);
 }
 
@@ -819,7 +911,7 @@ void TilesetDock::addTiles()
     Preferences *prefs = Preferences::instance();
     const QString startLocation = QFileInfo(prefs->lastPath(Preferences::ImageFile)).absolutePath();
     const QString filter = Utils::readableImageFormatsFilter();
-    const QStringList files = QFileDialog::getOpenFileNames(this,
+    const QStringList files = QFileDialog::getOpenFileNames(window(),
                                                             tr("Add Tiles"),
                                                             startLocation,
                                                             filter);
@@ -836,7 +928,7 @@ void TilesetDock::addTiles()
                                 tr("Add Tiles"),
                                 tr("Could not load \"%1\"!").arg(file),
                                 QMessageBox::Ignore | QMessageBox::Cancel,
-                                this);
+                                window());
             warning.setDefaultButton(QMessageBox::Ignore);
 
             if (warning.exec() != QMessageBox::Ignore) {
@@ -851,7 +943,9 @@ void TilesetDock::addTiles()
 
     prefs->setLastPath(Preferences::ImageFile, files.last());
 
-    mMapDocument->undoStack()->push(new AddTiles(mMapDocument, tileset, tiles));
+    mMapDocument->undoStack()->push(new AddTiles(mMapDocument,
+                                                 tileset,
+                                                 tiles));
 }
 
 void TilesetDock::removeTiles()
@@ -874,7 +968,11 @@ void TilesetDock::removeTiles()
         }
     }
 
-    const MatchesAnyTile matchesAnyTile(tiles);
+    auto matchesAnyTile = [&tiles] (const Cell &cell) {
+        if (Tile *tile = cell.tile)
+            return tiles.contains(tile);
+        return false;
+    };
     const bool inUse = hasTileReferences(mMapDocument, matchesAnyTile);
 
     // If the tileset is in use, warn the user and confirm removal
@@ -913,13 +1011,13 @@ void TilesetDock::removeTiles()
     undoStack->endMacro();
 
     // Clear the current tiles, will be referencing the removed tiles
-    setCurrentTiles(0);
-    setCurrentTile(0);
+    setCurrentTiles(nullptr);
+    setCurrentTile(nullptr);
 }
 
 void TilesetDock::tilesetNameChanged(Tileset *tileset)
 {
-    const int index = mTilesets.indexOf(tileset);
+    const int index = indexOf(mTilesets, tileset);
     Q_ASSERT(index != -1);
 
     mTabBar->setTabText(index, tileset->name());
