@@ -24,7 +24,6 @@
 #include "map.h"
 #include "mapdocument.h"
 #include "preferences.h"
-#include "tmxmapwriter.h"
 #include "tile.h"
 #include "tileset.h"
 #include "tilesetmodel.h"
@@ -33,7 +32,6 @@
 
 #include <QAbstractItemDelegate>
 #include <QCoreApplication>
-#include <QFileDialog>
 #include <QGesture>
 #include <QGestureEvent>
 #include <QHeaderView>
@@ -55,16 +53,16 @@ namespace {
 class TileDelegate : public QAbstractItemDelegate
 {
 public:
-    TileDelegate(TilesetView *tilesetView, QObject *parent = 0)
+    TileDelegate(TilesetView *tilesetView, QObject *parent = nullptr)
         : QAbstractItemDelegate(parent)
         , mTilesetView(tilesetView)
     { }
 
     void paint(QPainter *painter, const QStyleOptionViewItem &option,
-               const QModelIndex &index) const;
+               const QModelIndex &index) const override;
 
     QSize sizeHint(const QStyleOptionViewItem &option,
-                   const QModelIndex &index) const;
+                   const QModelIndex &index) const override;
 
 private:
     TilesetView *mTilesetView;
@@ -235,29 +233,36 @@ void TileDelegate::paint(QPainter *painter,
 
     // Overlay with film strip when animated
     if (mTilesetView->markAnimatedTiles() && tile->isAnimated()) {
-        QRectF strip(targetRect);
-        strip.setHeight(targetRect.height() / 5);
-        painter->fillRect(strip, Qt::black);
-        strip.moveBottom(targetRect.bottom() + 1);
+        painter->save();
+
+        qreal scale = qMin(tileImage.width() / 32.0,
+                           tileImage.height() / 32.0);
+
+        painter->setClipRect(targetRect);
+        painter->translate(targetRect.right(),
+                           targetRect.bottom());
+        painter->scale(scale * zoom, scale * zoom);
+        painter->translate(-18, 3);
+        painter->rotate(-45);
+        painter->setOpacity(0.8);
+
+        QRectF strip(0, 0, 32, 6);
         painter->fillRect(strip, Qt::black);
 
         painter->setRenderHint(QPainter::Antialiasing);
         painter->setBrush(Qt::white);
         painter->setPen(Qt::NoPen);
 
-        qreal step = qMax(strip.height(), qreal(3));
-        step = qMax(strip.width() / qFloor(strip.width() / step), qreal(3));
         QRectF hole(0, 0, strip.height() * 0.6, strip.height() * 0.6);
+        qreal step = (strip.height() - hole.height()) + hole.width();
         qreal margin = (strip.height() - hole.height()) / 2;
 
-        for (qreal x = strip.x() + (step - hole.width()) / 2;
-             x < strip.right();
-             x += step) {
-            hole.moveTo(x, targetRect.top() + margin);
-            painter->drawRoundedRect(hole, 25, 25, Qt::RelativeSize);
-            hole.moveTo(x, strip.top() + margin);
+        for (qreal x = (step - hole.width()) / 2; x < strip.right(); x += step) {
+            hole.moveTo(x, margin);
             painter->drawRoundedRect(hole, 25, 25, Qt::RelativeSize);
         }
+
+        painter->restore();
     }
 
     // Overlay with highlight color when selected
@@ -319,8 +324,8 @@ QSize TileDelegate::sizeHint(const QStyleOptionViewItem & /* option */,
 
 TilesetView::TilesetView(QWidget *parent)
     : QTableView(parent)
-    , mZoomable(0)
-    , mMapDocument(0)
+    , mZoomable(nullptr)
+    , mMapDocument(nullptr)
     , mMarkAnimatedTiles(true)
     , mEditTerrain(false)
     , mEraseTerrain(false)
@@ -337,13 +342,8 @@ TilesetView::TilesetView(QWidget *parent)
     QHeaderView *vHeader = verticalHeader();
     hHeader->hide();
     vHeader->hide();
-#if QT_VERSION >= 0x050000
     hHeader->setSectionResizeMode(QHeaderView::ResizeToContents);
     vHeader->setSectionResizeMode(QHeaderView::ResizeToContents);
-#else
-    hHeader->setResizeMode(QHeaderView::ResizeToContents);
-    vHeader->setResizeMode(QHeaderView::ResizeToContents);
-#endif
     hHeader->setMinimumSectionSize(1);
     vHeader->setMinimumSectionSize(1);
 
@@ -667,5 +667,5 @@ void TilesetView::finishTerrainChange()
 Tile *TilesetView::currentTile() const
 {
     const TilesetModel *model = tilesetModel();
-    return model ? model->tileAt(currentIndex()) : 0;
+    return model ? model->tileAt(currentIndex()) : nullptr;
 }
