@@ -31,9 +31,9 @@ using namespace Tiled::Internal;
 ToolManager::ToolManager(QObject *parent)
     : QObject(parent)
     , mActionGroup(new QActionGroup(this))
-    , mSelectedTool(0)
-    , mPreviouslyDisabledTool(0)
-    , mMapDocument(0)
+    , mSelectedTool(nullptr)
+    , mPreviouslyDisabledTool(nullptr)
+    , mMapDocument(nullptr)
 {
     mActionGroup->setExclusive(true);
     connect(mActionGroup, SIGNAL(triggered(QAction*)),
@@ -44,6 +44,9 @@ ToolManager::~ToolManager()
 {
 }
 
+/**
+ * Sets the MapDocument on which the registered tools will operate.
+ */
 void ToolManager::setMapDocument(MapDocument *mapDocument)
 {
     if (mMapDocument == mapDocument)
@@ -57,6 +60,12 @@ void ToolManager::setMapDocument(MapDocument *mapDocument)
     }
 }
 
+/**
+ * Registers a new tool. The tool manager does not take ownership over the
+ * tool.
+ *
+ * @return The action for activating the tool.
+ */
 QAction *ToolManager::registerTool(AbstractTool *tool)
 {
     tool->setMapDocument(mMapDocument);
@@ -83,6 +92,9 @@ QAction *ToolManager::registerTool(AbstractTool *tool)
     return toolAction;
 }
 
+/**
+ * Selects the given tool. It should be previously added using registerTool().
+ */
 void ToolManager::selectTool(AbstractTool *tool)
 {
     if (tool && !tool->isEnabled()) // Refuse to select disabled tools
@@ -98,7 +110,7 @@ void ToolManager::selectTool(AbstractTool *tool)
     // The given tool was not found. Don't select any tool.
     foreach (QAction *action, mActionGroup->actions())
         action->setChecked(false);
-    setSelectedTool(0);
+    setSelectedTool(nullptr);
 }
 
 void ToolManager::actionTriggered(QAction *action)
@@ -165,7 +177,7 @@ AbstractTool *ToolManager::firstEnabledTool() const
             if (tool->isEnabled())
                 return tool;
 
-    return 0;
+    return nullptr;
 }
 
 void ToolManager::setSelectedTool(AbstractTool *tool)
